#include "Camac.h"
#include <iostream>

bool Camac::setData(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, const uint32_t data, const uint32_t stb)
//Set dataset of the station's using function code and subaddress. If the returns agrees with stb, it returns true
{
#ifdef DEBUG_CAMAC
  std::cout<<"Send: N: "<<(long)station<<", A: "<<(long)subaddress<<", F: "<<(long)functioncode<<", DATA: "<<data<<std::endl;
#endif
  uint8_t response=setdata(subaddress, functioncode, station, data);//Get the response
  //Set the return value
#ifdef DEBUG_CAMAC
  std::cout<<"Response: "<<(long)response<<std::endl;
#endif
  if ((response&stb)==stb) return true;
  else return false;
}

bool Camac::Control(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, const uint8_t stb)
//This function transmits control message and provides information on the status bit: if it fulfills the minimum requirement written in stb, then it returns with true
{
#ifdef DEBUG_CAMAC
  std::cout<<"Control message: N: "<<(long)station<<", A: "<<(long)subaddress<<", F: "<<(long)functioncode<<std::endl;
  uint32_t answer=getdata(subaddress, functioncode, station, 0, true);
  std::cout<<"Status: "<<(long)answer<<std::endl;
  if ((answer&stb)==stb) return true;//Get the response
  else return false;
#else
  if ((getdata(subaddress, functioncode, station, 0, true)&stb)==stb) return true;//Get the response
  else return false;
#endif
}

bool Camac::getData(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, uint32_t& received_data, const uint8_t stb)
//This function returns with the dataset in received data. Additionally, it provides information on the status bit: it returns true, if it fullfills the minimum requirements written in stb
{
  uint8_t resp=0x0;//Status bit information
#ifdef DEBUG_CAMAC
  std::cout<<"Get data: N: "<<(long)station<<", A: "<<(long)subaddress<<", F: "<<(long)functioncode<<std::endl;
#endif
  received_data=getdata(subaddress, functioncode, station, &resp, false);//Get the response
#ifdef DEBUG_CAMAC
  std::cout<<"Status: "<<(long)resp<<", answer: "<<(long)received_data<<std::endl;
#endif
  if ((resp&stb) == stb) return true;
  else return false;
}