#include <cstdint>
#include <string>

#ifndef CAMAC_H
#define CAMAC_H

class Camac
{
  public:
    //static variables
    static const uint8_t STATUS_X=0x01;//X response
    static const uint8_t STATUS_Q=0x02;//Q response
    static const uint8_t STATUS_BLANK=0x0;//Blank response: module does not connected to X nor Q lines
    static const uint32_t CAMAC_ERR=0xfffffffe;//Some trouble happened during the communication
    //methods
    Camac(const uint8_t cNo, const std::string nm){ crateNo=cNo; setName(nm);};//Constructor: crate number
    ~Camac(){};//Destructor
    void setName(std::string nm){name=nm;};//Set the name of the controller
    std::string getName(){return name;};//Returns with the name of the device
    //Crate control
    virtual bool setCrate(const uint8_t cn){crateNo=cn; return true;}; //Sets the crate address
    uint8_t getCrate(){ return crateNo;}; //Returns with the crate address
    //Module communication
    //Set dataset
    bool setData(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, const uint32_t data, const uint32_t stb=STATUS_X);//Set dataset of the station's using function code and subaddress. If the returns agrees with stb, it returns true
    bool Control(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, const uint8_t stb);//This function transmits control message and provides information on the status bit: if it fulfills the minimum requirement written in stb, then it returns with true
    bool getData(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, uint32_t& received_data, const uint8_t stb=STATUS_Q);//This function returns with the dataset in received data. Additionally, it provides information on the status bit: it returns true, if it fullfills the minimum requirements written in stb
    //virtual method
    virtual bool Reset()=0;//Resets the Camac unit
    virtual bool SetResetInhibit(bool reset=true)=0;//Set(f)/clear(t) dataway inhibit. Returns with communication success or error
    virtual bool GetInhibit(bool test=false)=0;//Get the dataway inhibit value (t: set, f: unset). If test is true, it returns the setting of the controller (it might be other sources of dataway inhibit: test is tests only the controller, but general the dataway directly). 
  protected:
    //protected vitrual methods
    virtual uint32_t getdata(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, uint8_t *response, const bool only_status=false)=0;
    //Get dataset from the station using function code and subaddress.
    //If the only_status bit is false, then it returns with the dataset and the response code
    //If it were true, it returns only with the status byte information, regardless the returning value
    virtual uint8_t setdata(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, const uint32_t data)=0;
    //Set dataset of the station's using function code and subaddress. It returns with proper response. Otherwise: with error codes 00
  private:
    //members
    uint8_t crateNo;//Crate number to communicate
    std::string name;//Identifier of the controller
};
#endif