#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include "Camac.h"

#ifndef CAMAC_TEST_H
#define CAMAC_TEST_H

class CamacTest: public Camac
//This class created to simulate the Camac communication to and from Camac module devices by software way
{
  public:
    //public methods
    CamacTest(const uint8_t crate, const std::string nm):Camac(crate, nm)
    {
      inhibit=true;
    };//Constructor
    ~CamacTest(){};//Destructor
    bool Reset(){inhibit=true; std::cout<<"CAMAC: RESET"<<std::endl; return true;};//Resets the Camac unit
    bool SetResetInhibit(bool reset=true){inhibit=(!reset); std::cout<<"CAMAC: Inhibit "<<(inhibit ? "ON" : "OFF")<<std::endl; return true;};//Set(f)/clear(t) dataway inhibit. Returns with communication success or error
    bool GetInhibit(bool test=false){ std::cout<<"CAMAC: Inhibit status ("<<(inhibit ? "ON" : "OFF")<<") asked"<<std::endl;return inhibit;};//Get the dataway inhibit value (t: set, f: unset). If test is true, it returns the setting of the controller (it might be other sources of dataway inhibit: test is tests only the controller, but general the dataway directly).
  private:
    //private methods
    uint32_t getdata(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, uint8_t *response, const bool only_status=false)
    {
	std::cout<<"CAMAC "<<(only_status ? "CONTROL" :"READ")<<": N: "<<(int)station<<", F: "<<(int)functioncode<<", A: "<<(int)subaddress<<", Inhibit: "<<(inhibit ? "ON" : "OFF")<<std::endl;
	std::string answer;
	//Control question: returning with status
	if (only_status)
	{  
	  std::cout<<"Answer: (QX) > ";
	  std::cin>>answer;
	  uint32_t ans=0;
	  if(answer.find_first_of("Qq") !=std::string::npos) ans+=Camac::STATUS_Q;
	  if(answer.find_first_of("Xx") !=std::string::npos) ans+=Camac::STATUS_X;
	  return ans;
	}
	else 
	{
	  uint32_t data;
	  std::cout<<"Answer: (data QX) > ";
	  std::cin>>data>>answer;
	  if(response!=0)
	  {
	    *response=0;
	    if(answer.find_first_of("Qq") !=std::string::npos) (*response)+=Camac::STATUS_Q;
	    if(answer.find_first_of("Xx") !=std::string::npos) (*response)+=Camac::STATUS_X;
	  }
	  return data;
	}
    }
    //Get dataset from the station using function code and subaddress.
    //If the only_status bit is false, then it returns with the dataset and the response code
    //If it were true, it returns only with the status byte information, regardless the returning value
    uint8_t setdata(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, const uint32_t data)
    {
	std::cout<<"CAMAC WRITE: N: "<<(int)station<<", F: "<<(int)functioncode<<", A: "<<(int)subaddress<<", W: "<<(long)data<<" "<<std::hex<<(long)data<<std::dec<<", Inhibit: "<<(inhibit ? "ON" : "OFF")<<std::endl;
	std::string answer;
	std::cout<<"Answer: (QX) > ";
	std::cin>>answer;
	uint8_t ans=0;
	if(answer.find_first_of("Qq") !=std::string::npos) ans+=Camac::STATUS_Q;
	if(answer.find_first_of("Xx") !=std::string::npos) ans+=Camac::STATUS_X;
	return ans;	
    }
    
    //Set dataset of the station's using function code and subaddress. It returns with proper response. Otherwise: with error codes 00
    
    //private variables
    bool inhibit;//Actual value of the inhibit
};
#endif