#include "HV_CAEN_N1471.h"

const double HV_CAEN_N1471::TIMEOUT=.1;//0.1 s
const double HV_CAEN_N1471::WAITSUCCESS=0.1;//0.1 s

HV_CAEN_N1471::HV_CAEN_N1471(const std::string nPort, const int nBaud, const double timeout)
//Constructor: serial port name, timeout
{
  //Setting the communication lime and address
  communicator=NULL;
  if(!ResetCommunication(nPort, nBaud, timeout)) std::cout<<"HV_CAEN_N1471::HV_CAEN_N1471: error in opening the communication channel"<<std::endl;
}

bool HV_CAEN_N1471::ResetCommunication(const std::string nPort, const int nBaud, const double timeout)
//Reset the communication parameters: serial port, transmission rate, timeout
{
  bool retval;//Returning value
  //Modify the settings or create a new channel
  if(communicator==NULL)
  {
    communicator= new Serial(nPort, nBaud, 8, 'N', 1, true, false);
    retval=communicator->isOpened();
  }
  else retval=communicator->Open(nPort, nBaud, 8, 'N', 1, true, false);
  
  //Set timeout
  if (retval) retval=communicator->setTimeout(timeout);
  
  //Returning with the results
  return retval;
}

bool HV_CAEN_N1471::getStatus(const int module_address, std::vector<bool> &on, std::vector<bool> &ramp_up, std::vector<bool> &ramp_dwn, std::vector<bool> &curr_ne_set, std::vector<bool> &volt_overshot, std::vector<bool> &volt_undershot, std::vector<bool> &maxv_protect, std::vector<bool> &off_bytrip, std::vector<bool> &over_pwr, std::vector<bool> &overheat, std::vector<bool> &disabled_remote_and_off, std::vector<bool> &kill_bypanel, std::vector<bool> &interlck_bypanel, std::vector<bool> &cal_error, const int channel_no)
//Provides status information: over/undershot set+/-250V, over power: >1.7W, overheat: >105C
{
    std::vector<int> answer;
    on.clear(); ramp_up.clear(); ramp_dwn.clear(); curr_ne_set.clear(); volt_overshot.clear(); volt_undershot.clear(); maxv_protect.clear(); off_bytrip.clear(); over_pwr.clear(); overheat.clear(); disabled_remote_and_off.clear(); kill_bypanel.clear(); interlck_bypanel.clear(); cal_error.clear();
    if(chvalue<int>(module_address,"STAT",answer,channel_no))
    {
        for(int i=0; i<(channel_no==4 ? 4 : 1); i++)
        {
            int &a=answer[i];
            on.push_back((a&0x1)==0x1);
            ramp_up.push_back(((a>>1)&0x1)==0x1);
            ramp_dwn.push_back(((a>>2)&0x1)==0x1);
            curr_ne_set.push_back(((a>>3)&0x1)==0x1);
            volt_overshot.push_back(((a>>4)&0x1)==0x1);
            volt_undershot.push_back(((a>>5)&0x1)==0x1);
            maxv_protect.push_back(((a>>6)&0x1)==0x1);
            off_bytrip.push_back(((a>>7)&0x1)==0x1);
            over_pwr.push_back(((a>>8)&0x1)==0x1);
            overheat.push_back(((a>>9)&0x1)==0x1);
            disabled_remote_and_off.push_back(((a>>10)&0x1)==0x1);
            kill_bypanel.push_back(((a>>11)&0x1)==0x1);
            interlck_bypanel.push_back(((a>>12)&0x1)==0x1);
            cal_error.push_back(((a>>13)&0x1)==0x1);
        }
        return true;
    }
    else return false;
}

bool HV_CAEN_N1471::getAbout(const int module_address, std::string &name, int &no_channels, std::string &firmware_release, int &serial_no)
//Get general information about the module as type(name), number of channels, firmware release and serial number
{
    std::vector<std::string> nam,fw;
    std::vector<int> chn,ser;
    if(chvalue<std::string>(module_address,"BDNAME",nam)&&chvalue<int>(module_address,"BDNCH",chn)&&chvalue<std::string>(module_address,"BDFREL",fw)&&chvalue<int>(module_address,"BDSNUM",ser))
    {
        name=nam[0];
        no_channels=chn[0];
        firmware_release=fw[0];
        serial_no=ser[0];
        return true;
    }
    else return false;
}

bool HV_CAEN_N1471::getInterlock(const int module_address, bool &interlockmode_opent_closef, bool &interlockstatus_tyes_fno)
//Get interlock related parameters: mode=OPEN(t)/CLOSE(f) and current status=YES(1)/NO(0)
{
    std::vector<bool> mode,status;
    if(chstr2bool(module_address,"BDILKM","OPEN","CLOSED",mode)&&chstr2bool(module_address,"BDILK","YES","NO",status))
    {
        interlockmode_opent_closef=mode[0];
        interlockstatus_tyes_fno=status[0];
        return true;
    }
    else return false;
}

bool  HV_CAEN_N1471::getCommunication(const int module_address, bool &controlmode_remotet_localf, bool &localtermination_ont_offf)
//Get communication information, like control mode(remote(t)/local(f)) and the status of termination status in daisy chaned modules (on(t)/off(f))
{
    std::vector<bool> mode,termination;
    if(chstr2bool(module_address,"BDCTR","REMOTE","LOCAL",mode)&&chstr2bool(module_address,"BDTERM","ON","OFF",termination))
    {
        controlmode_remotet_localf=mode[0];
        localtermination_ont_offf=termination[0];
        return true;
    }
    else return false;
}

bool HV_CAEN_N1471::getAlarms(const int module_address, std::vector<bool> &ch_alarm, bool &pwr_fail, bool &over_pwr, bool &hvclk_fail)
//Returns with the value of alarms: of each channel, POWER_FAIL, OVER_POWER, HV Clock FAIL (outside 200+/-10 kHz range)
{
    std::vector<int> answer;
    ch_alarm.clear();
    if(chvalue<int>(module_address,"BDALARM",answer))
    {
        int &ans=answer[0];
        ch_alarm.push_back((ans&0x1)==0x1);
        ch_alarm.push_back(((ans>>1)&0x1)==0x1);
        ch_alarm.push_back(((ans>>2)&0x1)==0x1);
        ch_alarm.push_back(((ans>>3)&0x1)==0x1);
        pwr_fail=(((ans>>4)&0x1)==0x1);
        over_pwr=(((ans>>5)&0x1)==0x1);
        hvclk_fail=(((ans>>6)&0x1)==0x1);
        return true;
    }
    else return false;
}

//Mid level communication

bool HV_CAEN_N1471::chabout(const int module_address, const std::string &parameter, std::vector<double> &minimum_value, std::vector<double> &maximal_value, std::vector<int> &no_digits, const int channel_no, const std::string ext_dec)
//Get the "about" information by setting the module address, the first common letter(s) for parameter (this is the "parameter"), the channel number (0..3 for each, 4=all) and the extension letters after the "parameter" for decimal digits. Returning values are minimum_value, maximal_value and number of digits vectors, whose content is cleared. Returns with true for successful communication, otherwise error information is asserted.
{
    return chvalue<double>(module_address,parameter+"MIN",minimum_value,channel_no)&&chvalue<double>(module_address,parameter+"MAX",maximal_value,channel_no)&&chvalue<int>(module_address,parameter+ext_dec+"DEC",no_digits,channel_no);
}

bool HV_CAEN_N1471::chstr2bool(const int module_address, const std::string &parameter, const std::string &trueans, const std::string &falseans, std::vector<bool> &vals, const int channel_no)
//Get the actual value of a parameter and transforming it to boolean by setting the module address, the command letter, the true and false answers and the channel number (0..3 for each, 4=all). Returning values are the requested value. Returns with true for successful communication, otherwise error information is asserted.
{
    std::vector<std::string> answers;
    vals.clear();
    if(chvalue<std::string>(module_address,parameter,answers,channel_no))
    {
        for(int i=0; i<static_cast<int>(answers.size()); i++)
        {
            if(answers[i]==trueans) vals.push_back(true);
            else if(answers[i]==falseans) vals.push_back(false);
            else
            {
                std::cerr<<"--ERROR--(HV_CAEN_N1471::chstr2bool): the \""<<answers[i]<<"\" string cannot be categorized as true (\""<<trueans<<"\") nor as false (\""<<falseans<<"\")";
                return false;
            }
        }
        return true;
    }
    else return false;
}

bool HV_CAEN_N1471::bool2strch(const int module_address, const std::string &parameter, const std::string &trueans, const std::string &falseans, const bool &vals, const int channel_no)
//Set the requested value of a parameter by transforming it from boolean by setting the module address, the command letter, the true and false answer strings and the channel number (0..3 for each, 4=all). Returns with true for successful communication, otherwise error information is asserted.
{
    std::string sr(vals ? trueans : falseans);
    return communicate_errorcheck(module_address,false,parameter,sr,channel_no);
}


bool HV_CAEN_N1471::setvaldouble(const int module_address, const std::string &parameter, const double &val,bool (HV_CAEN_N1471::*about)(const int, std::vector<double> &, std::vector<double>&, std::vector<int>&, const int), const int channel_no)
//Set the value of the parameter of module related to channel (0..3 for each, 4=all) by val. The about function is required as it provides decimal digits. Returns with true for successful communication, otherwise error information is asserted.
{
    std::vector<double> min,max;
    std::vector<int> dec_dig;
    if((this->*about)(module_address,min,max,dec_dig,channel_no))
    {
        if(val<min[0])
        {
            std::cerr<<"--ERROR--(HV_CAEN_N1471::setvaldouble): the value you provided ("<<val<<") is less than the minimal allowed value ("<<min[0]<<")"<<std::endl;
            return false;
        }
        else if(val>max[0])
        {
            std::cerr<<"--ERROR--(HV_CAEN_N1471::setvaldouble): the value you provided ("<<val<<") is higher than the maximal allowed value ("<<max[0]<<")"<<std::endl;
            return false;
        }
        else return (this->*about)(module_address,min,max,dec_dig,channel_no)&&valuech<double>(module_address,parameter,val,dec_dig[0],channel_no);
    }
    else return false;
}
//Low level communication

bool HV_CAEN_N1471::communicate(const int module_address, const bool mont_setf, const std::string &parameter, std::string &value, const int channel_no)
//Communicate with serial interface: it sets the module_address, the command: type MONitor(t)/SET(f), the parameter, the requested value (only for set input) and channel number (0..3 specific channel, 4: all)
//It returns with true in the case of successful end when the requested value(s) will be set to value (only for monitor).
//Otherwise with false and the reason of the error is indicated in the value part as:
// CMD (wrong command format or command not recognized); CH (channel field not present or wrong channel value);
// PAR (parameter field not present or not recognized); VAL (wrong set value <min or >max); LOC (command set with module in LOCAL mode)
// COE (communication error); WRA (wrong address)
{
  //Checking the input range
  if((module_address<0)||(module_address>31))
  {
    value=std::string("WRA");
    return false;
  }
  if((channel_no<0)||(channel_no>4))
  {
    value=std::string("CH");
    return false;
  }
  //Creating buffer to send
  std::stringstream sbuffer,tbuff,hbuff;
  tbuff<<CR<<LF;
  std::string tail=tbuff.str();
  hbuff<<"BD:"<<std::setfill('0')<<std::setw(2)<<module_address;
  std::string head=hbuff.str();
  sbuffer<<"$"<<head<<",CMD:"<<(mont_setf ? "MON" : "SET");
  if(parameter[0]!='B') sbuffer<<",CH:"<<std::setw(1)<<channel_no;
  sbuffer<<",PAR:"<<parameter;
  if(value.size()!=0) sbuffer<<",VAL:"<<value;
  sbuffer<<tail;
  uint8_t replybuffer[100];
  int rlen=100;
  const std::string &tmp=sbuffer.str();
  int slen=tmp.size();
  const uint8_t *sendbuffer=reinterpret_cast<const uint8_t*>( tmp.c_str() );
  
  //Writing it to the serial line
  int counterwhole=0;//Whole procedure counter
  int rep;//Local reply
  
  bool repeatfull=false;//Reapeat the full send and receive procedure
  do
  {
    repeatfull=false;
    int counter=0;//Local counter for send and reply
    bool repeatsend=false;//Repeat send procedure
    bool repeatreply=false;//Repeat reply procedure
    
    //Perform error checked writing
    do
    {
#ifdef DEBUG_HV_CAEN    
      std::cout<<"Sendbuffer containing "<<slen<<" characters"<<std::endl;
      for(int i=0; i<slen; i++) std::cout<<i<<": (hex)"<<std::hex<<(int)sendbuffer[i]<<" dec:"<<std::dec<<sendbuffer[i]<<std::endl;
#endif
      //Send bytes
      rep=communicator->Send(sendbuffer,slen);
      
      if(rep!=slen)
      {
        repeatsend=true;
        //Communication error
        std::cerr<<"Sending is unsuccessful(HV_CAEN_N1471::communicate) on the serial line!\n";
        if(counter++<1) 
        {
            std::cerr<<"Trying again!"<<std::endl;
            usleep((int)(100000.0*WAITSUCCESS));//Wait some time until the next trial
        }
        else
        {
            value=std::string("COE");
            return false;
        }
      }
    }while(repeatsend);
    usleep((int)(100000.0*WAITSUCCESS));//Wait some time until the answer
    
    //Perform error checked reading
    counter=0;
    do
    {
      //Receiving the reply
      rep=communicator->Receive(replybuffer,rlen,false);
#ifdef DEBUG_HV_CAEN	    
      std::cout<<"Replybuffer:";
      for(int i=0; i<rep; i++) std::cout<<i<<": (hex)"<<std::hex<<(int)replybuffer[i]<<" dec:"<<std::dec<<(int)replybuffer[i]<<" "<<(char)replybuffer[i]<<std::endl;
      std::cout<<rep<<std::endl;
#endif
      if (rep<1)
      {	    
        //Possible reasons: 
        //timeout
        //SERIAL_ERROR --- try it igain
        repeatreply=true;
        usleep((int)(100000.0*WAITSUCCESS));//Wait some time until the next trial
        if(counter++==1)
        {
            std::cerr<<"Communication error on the serial line (HV_CAEN_N1471::communicate) during the reading!\n";
            if (counterwhole++==0) {std::cerr<<"Sending messages again\n"; repeatreply=false; repeatfull=true;} 
            else
            {
                value=std::string("COE");
                return false;
            }
        }
      }
      else 
      {
        //Leaving the cycle
        //*rlen=rep;
        repeatreply=false;
      }
    }while(repeatreply);
  }while(repeatfull);
  
  //Processing the answer
  std::string answer(reinterpret_cast<const char*>(replybuffer),static_cast<std::size_t>(rep));
  std::size_t tpos=answer.find(tail);
  if((tpos==std::string::npos)||(answer.find(std::string("#"+head))!=0))
  {
    //Lack of tail closing characters or the #BD__ signature
    value=std::string("COE");
    return false;
  }
  answer=answer.substr(0,tpos);
  if(answer.find(",CMD:OK")==std::string::npos)
  {
      //Error reported
      tpos=answer.find(":ERR");
      value=answer.substr(7,tpos-7);
      return false;
  }
  //Set the value to be returned
  tpos=answer.find(",VAL:");
  if(tpos==std::string::npos) answer="";
  else answer=answer.substr(tpos+5,answer.size()-tpos-5);
  //Removing the remaining commas
  std::replace(answer.begin(),answer.end(), ';', ' ');
#ifdef DEBUG_HV_CAEN
  std::cout<<"The reply from communicate: \""<<answer<<"\""<<std::endl;
#endif

  value=answer;
  return true;
}

bool HV_CAEN_N1471::communicate_errorcheck(const int module_address, const bool mont_setf, const std::string &parameter, std::string &value, const int channel_no)
//Communicate with serial interface: it sets the module_address, the command: type MONitor(t)/SET(f), the parameter, the requested value (only for set input) and channel number (0..3 specific channel, 4: all)
//It returns with true in the case of successful end when the requested value(s) will be set to value (only for monitor).
//Otherwise with false and the reason of error is written to the standard error screen
{
    
    bool retval=communicate(module_address,mont_setf,parameter,value,channel_no);
    if(!retval)
    {
        std::cerr<<"--ERROR--(HV_CAEN_N1471::communicate_errorcheck): ";
        if(value=="CMD") std::cerr<<"wrong command format or command not recognized";
        else if(value=="CH") std::cerr<<"channel field not present or wrong channel value";
        else if(value=="PAR") std::cerr<<"parameter field not present or not recognized";
        else if(value=="VAL") std::cerr<<"wrong set value <min or >max";
        else if(value=="LOC") std::cerr<<"command set with module in LOCAL mode";
        else if(value=="COE") std::cerr<<"communication error";
        else if(value=="WRA") std::cerr<<"wrong address";
        else std::cerr<<"error happend, but error code non-recognized. Error code: \""<<value<<"\"";
        std::cerr<<std::endl;
    }
    return retval;
}
