//This module has not been tested yet

#include <iostream>
#include <list>
#include <map>
#include <string>
#include "Camac.h"


#ifndef IO_OP_H
#define IO_OP_H

namespace IOOperation
{
  class InterruptReceiver;//Needed due to cross-definition
  
  class InterruptHandle
  //This virtual class describes an interrupt handler, which registers connections to those classes, which do some actions, when the proper interrupt happens
  {
    public:
      //public methods
      InterruptHandle(bool exactbit=false);//Constructor: if exactbit is true, then it notifies only those, whose value exactly the requested number. If it is false (bit), then it notifies those, whoose bit has been changed 
      ~InterruptHandle();//Destructor: it calls the DisableInterrupt method of each associated InterruptReceivers
      void Register(InterruptReceiver *ir, uint32_t mask);//Registers an interrupt receiver related to mask events.
      bool UnRegister(InterruptReceiver *ir, uint32_t mask);//Unregisters the entries kept by ir for the mask events. Returns with true, if all of the entries related to ir are cleared
      bool UnRegister(InterruptReceiver *ir);//Unregisters the entries kept by ir for all of the events.  Returns always with true
      void Notify(uint32_t status, uint32_t mask);//Notifies the mask related InterruptReceivers with status: this method should be called if any change has been happened
    protected:
      //protected methods
      uint32_t getMask(){return interrupt_attention;};//Returns with the mask
      bool isChanged(){return changed;};//Returns with true, if new bit/number has done during the last Register/Unregister cycle
    private:
      //private members
      std::multimap<uint32_t,InterruptReceiver*> *receiver_list;//Associative array stores the number/mask  -> InterruptReceivers list
      bool number_or_bit_mode;//Number mode: exact number , bit mode: masking
      bool changed;//It is true, if any new number/bit appeared/removed on/from interrupt_attention during the last Register/Unregister procedure
      uint32_t interrupt_attention;//Contains the bits/numbers, whose change would be interesting
      //private methods
      void addremove_element(InterruptReceiver *ir, uint32_t mask, bool remove); //It adds(f)/remove(t) an element of InterruptReceiver* for the list to the mask key (it avoids multiple entry of the same mask and ir pair for add operation)
  };
  
  class InterruptReceiver
  //This virtual class do some action, when the proper interrupt happens, notified by the InterruptHandle class
  {
    friend class InterruptHandle;//Needed for destruction of handler earlier than receiver: to call the DisableInterrupt
    public:
      //public methods
      InterruptReceiver();//Constructor
      ~InterruptReceiver();//Destructor
      virtual void Action(InterruptHandle *ih, uint32_t mask)=0;//Performs the action related to the mask and interrupt handler.
      void EnableInterrupt(InterruptHandle *ih, uint32_t mask);//Enable interrupt service at ih for mask dataset
      void DisableInterrupt(InterruptHandle *ih, uint32_t mask, bool all=false);//Disables the interrupt service at if for mask dataset(all=false) or each (all=true)
    private:
      //Private members
      std::list<InterruptHandle*> *handlers;//List of assigned interrupt handlers
  };
  
  class InterruptRegister: public InterruptHandle
  //This class describes the Borer Type 1051A 16 bit interrupt register, which deals with external events
  {
    public:
      //public methods
      InterruptRegister(Camac &camac, uint8_t station_number=6):InterruptHandle(false){comm=&camac; slot=station_number; lservice=false;previous=0x0;};//Constructor (as Interrupt handle works in bitwise mode)
      ~InterruptRegister(){comm=0;};//Destructor
      void Register(InterruptReceiver *ir, uint32_t mask) {InterruptHandle::Register(ir,mask); if(isChanged()) reset(getMask()); return;};//Registers an interrupt receiver related to mask events and reset the LAM service
      bool UnRegister(InterruptReceiver *ir, uint32_t mask){bool retval=InterruptHandle::UnRegister(ir,mask); if(isChanged()) reset(getMask()); return retval;};//Unregisters the entries kept by ir for the mask events and resets the LAM service. Returns with true, if all of the entries related to ir are cleared
      bool UnRegister(InterruptReceiver *ir){bool retval=InterruptHandle::UnRegister(ir); if(isChanged()) reset(getMask()); return retval;};//Unregisters the entries kept by ir for all of the events and reset the LAM service.  Returns always with true
      void Check() { if(testlam(false)) {uint32_t input=0x0; bool retval=read(input,true); if(retval) {Notify(previous^input, input); previous^=input;}} return;};//This method checks the attended inputs and notifies the related classes about its (read difference: if it has any change: the read operation update status register, clears LAM ff.) 
      std::string getCAMName(){return comm->getName();}//Returns with the ID of the CAMAC controller
      uint8_t getStation(){return slot;};//Returns with the station number
      //static members - inputs
      static const uint16_t LIMIT_HI_OFF=0x1;//High-limit switch is in OFF state
      static const uint16_t LIMIT_LO_OFF=0x1<<1;//Low-limit switch is in OFF state
      static const uint16_t INP2=0x1<<2;//***NOT USED***
      static const uint16_t INP3=0x1<<3;//***NOT USED***
      static const uint16_t COMPRESSEDAIR_ON=0x1<<4;//Compressed air is ON
      static const uint16_t INP5=0x1<<5;//***NOT USED***
      static const uint16_t FURCOOL_ON=0x1<<6;//Furnace cooling operates ***EXPERIMENTAL*** (N45 H(II)2- common: 1)
      static const uint16_t TURBOPUMP_ON=0x1<<7;//Turbomolecular pump reached normal speed and there is not any errors ***EXPERIMENTAL*** (N45 H(II)3- common: 1)
      static const uint16_t HIGH_VACUUM_ON=0x1<<8;//High vacuum reached ***EXPERIMENTAL*** (N45 H(II)4- common: 1)
      static const uint16_t INP9=0x1<<9;//***NOT USED***
      static const uint16_t INP10=0x1<<10;//***NOT USED***
      static const uint16_t INP11=0x1<<11;//***NOT USED***
      static const uint16_t INP12=0x1<<12;//***NOT USED***
      static const uint16_t INP13=0x1<<13;//***NOT USED***
      static const uint16_t INP14=0x1<<14;//***NOT USED***
      static const uint16_t INP15=0x1<<15;//***NOT USED*** (N10 R(I)1 - common:3 )
    private:
      //private members
      Camac *comm;//Camac communicator object
      uint8_t slot;//Slot number
      bool lservice;//LAM service indicator
      uint32_t previous;//Previous value of the interrupt register
      //static members
      static const uint8_t READ_F=0;//Read function code
      static const uint8_t TESTLFF_F=8;//Test LAM f-f function code
      static const uint8_t CLRLFF_F=10;//Clears LAM f-f function code
      static const uint8_t WRITE_F=16;//Write function code
      static const uint8_t DISABLELAM_F=24;//Disable LAM function code
      static const uint8_t ENABLELAM_F=26;//Enable LAM function code
      static const uint8_t TESTLAM_F=27;//Tests enable LAM function code
      static const uint8_t BASIC_A=0;//Contains basic subaddress
      static const uint8_t STREG_A=1;//Storage register related subaddress
      //private methods
     bool reset(uint32_t mask);//This method resets the LAM service and set mask, which contains the variables to be attended. If LAM service not started yet it starts and mask=0, it disable LAM service
      //private methods: CAMAC primitives
      bool read(uint32_t &actual, bool changed=false) {return comm->getData( changed ? STREG_A : BASIC_A, READ_F, slot, actual, Camac::STATUS_X | Camac::STATUS_Q);};//Reads the actual(f)/changed(t) content of Interrupt register. In least, it updates storage register and reset LAM f-f
      bool write(uint16_t data, bool maskstorage=true) {return comm->setData(maskstorage ? BASIC_A : STREG_A, WRITE_F, slot, (uint32_t) data, Camac::STATUS_X | Camac::STATUS_Q);};//Writes mask(t) or strorage (f) register. In the case of mask, the masked bits does not gives lam
      bool enabledisable_lam(bool enable=false){return comm->Control(BASIC_A, enable ? ENABLELAM_F : DISABLELAM_F, slot, Camac::STATUS_X | Camac::STATUS_Q);};//Enable(t)/disable(f) LAM
      bool testlam(bool enabledff=true){return comm->Control(BASIC_A, enabledff ? TESTLAM_F : TESTLFF_F, slot, Camac::STATUS_X | Camac::STATUS_Q);};//Tests the LAM enabled(t) or LAM flip-flop (f) status and sets into result
      bool resetlamff(){return comm->Control(BASIC_A, CLRLFF_F, slot, Camac::STATUS_X | Camac::STATUS_Q);};//Resets LAM flip-flop
  };
  
  class OutputRegister
  //This class describes a Borer Type 1082A Output register which controls external relays
  {
    public:
     OutputRegister(Camac &camac, uint8_t station_number=7, bool jumper_execute=true);//Constructor: camac controller and station number and execute jumper setup (1-true, 2-false)
     ~OutputRegister();//Destructor
     bool Set(uint32_t pattern, bool direct=false);//This method set bits of pattern to on, if direct is false. Otherwise, it writes the pattern directly.
     bool Reset(uint32_t pattern);//This method set bits of pattern to off.
     bool Read(uint32_t *pattern, bool software=true);//Read the currently set pattern, if software is true, does not ask camac module; returns with the success of communication
     std::string getCAMName(){return cam->getName();}//Returns with the ID of the CAMAC controller
     uint8_t getStation(){return station;};//Returns with the station number
     bool getExecuteJumper(){return jumper;};//Returns with the jumper setup: 1-true (command&execute), 2-false (separate execute)
     //static members
     static const uint32_t OR00=0x1;//Output relay(0): ***NOT USED*** (36p 13 -- N44 R(I)1)
     static const uint32_t OR01=0x1<<1;//Output relay(1): ***NOT USED*** (36p 14 -- N44 R(I)2)
     static const uint32_t OR02=0x1<<2;//Output relay(2): ***NOT USED*** (36p 15 -- N44 R(I)3)
     static const uint32_t OR03=0x1<<3;//Output relay(3): ***NOT USED*** (36p 16 -- N44 R(I)4)
     static const uint32_t OR04=0x1<<4;//Output relay(4): ***NOT USED*** (36p 17 -- N44 R(I)5)
     static const uint32_t OR05=0x1<<5;//Output relay(5): ***NOT USED*** (36p 18 -- N44 R(I)6)
     static const uint32_t OR06=0x1<<6;//Output relay(6): ***NOT USED*** (36p 19 -- N44 R(I)7)
     static const uint32_t OR07=0x1<<7;//Output relay(7): ***NOT USED*** (36p 20 -- N44 R(I)8)
     static const uint32_t OR08=0x1<<8;//Output relay(8): ***NOT USED*** (36p 21 -- N44 H(II)1)
     static const uint32_t OR09=0x1<<9;//Output relay(9): ***NOT USED*** (36p 22 -- N44 H(II)2)
     static const uint32_t OR10=0x1<<10;//Output relay(10): ***NOT USED*** (36p 23 -- N44 H(II)3)
     static const uint32_t OR11=0x1<<11;//Output relay(11): ***NOT USED*** (36p 24 -- N44 H(II)4)
     static const uint32_t VACUUM_PUMP_ON=0x1<<12;//Allow to operate vacuum pump ***EXPERIMENTAL*** (36p 30 -- N45 R(I)1 and N44 H(II)6)
     static const uint32_t FURNACE_ON=0x1<<13;//Switch on the furnace  ***EXPERIMENTAL*** (36p 31 -- N45 R(I)2 and N44 H(II)7)
     static const uint32_t HW_MOTORS_ON=0x1<<14;//Switch on the harwell motors (36p 32 -- N45 R(I)3)
     static const uint32_t OR15=0x1<<15;//Output relay(15): ***NOT USED*** (36p 34 -- broken)
     static const uint32_t MOT_MON_TLT=0x1<<16;//Motor 1 power relay selection: monochromator tilt (36p 1 -- common with 36p 4)
     static const uint32_t MOT2=0x1<<17;//Motor 2 power relay selection: ***NOT USED*** (36p 2)
     static const uint32_t MOT_MON_SHFT=0x1<<18;//Motor 3 power relay selection: monochromator shift (36p 3)
     static const uint32_t MOT4=0x1<<19;//Motor 4 power relay selection: ***NOT USED*** (36p 4 -- common with 36p 1)
     static const uint32_t MOT_MON_ROT=0x1<<20;//Motor 5 power relay selection: monochromator rotate (36p 5)
     static const uint32_t MOT_DIFFM_ARM=0x1<<21;//Motor 6 power relay selection: rotate diffractometer arm (36p 6)
     static const uint32_t MOT7=0x1<<22;//Motor 7 power relay selection: ***NOT USED*** (36p 7)
     static const uint32_t MOT8=0x1<<23;//Motor 8 power relay selection: ***NOT USED*** (36p 8)
     static const uint32_t MOT9=0x1<<24;//Motor 9 power relay selection: ***NOT USED*** (36p 9 -- N45 R(I)4)
     static const uint32_t MOT10=0x1<<25;//Motor 10 power relay selection: ***NOT USED*** (36p 10 -- N45 R(I)5)
     static const uint32_t MOT11=0x1<<26;//Motor 11 power relay selection: ***NOT USED*** (36p 11 -- N45 R(I)6)
     static const uint32_t MOT12=0x1<<27;//Motor 12 power relay selection: ***NOT USED*** (36p 12 -- N45 R(I)7)
     static const uint32_t VLV_CA_R=0x1<<28;//Magnet valve: compressed air reactor side (36p 26)
     static const uint32_t VLV_CA_H=0x1<<29;//Magnet valve: compressed air hall side (36p 27)
     static const uint32_t VLV=0x1<<30;//Magnet valve: ***NOT USED*** (36p 28)
     static const uint32_t MOTOR_ON=0x1<<31;//Motor power on (36p 29)
    private:
     //members
     Camac *cam;//Camac controller
     uint16_t message[2];//First and second output word
     uint8_t station;//Camac station number
     bool jumper;//Switch: 1-true, 2-false
     //static members
     static const uint8_t READ_F=0;//Read (read buffer) function code
     static const uint8_t CLEAR_F=9;//Clears (clear buff + exec chn register) function code
     static const uint8_t WRITE_F=16;//Write (owerwrite buffer + execute if switch is 1) function code
     static const uint8_t EXEC_F=25;//Execute (only if switch is 2) function code - sw=1 thus this is not relevant
     static const uint8_t LO_A=0;//First output word
     static const uint8_t HI_A=1;//Second output word
     static const uint8_t HILO_A=2;//Both register
     //methods
     //bool readbuffer(uint32_t &buffer, const bool first=true){ return cam->getData(first ? LO_A : HI_A, READ_F, station, buffer, Camac::STATUS_Q | Camac::STATUS_X);}//Reads the buffer data for the requested channel -- this method is not present by checking the real device
     bool clearbuffer(const bool first=true){ if(jumper) return overwritebuffer(0x0,first); else return cam->Control(first ? LO_A : HI_A, CLEAR_F, station, Camac::STATUS_Q | Camac::STATUS_X);}//Clears the buffer and gives execute command
     bool overwritebuffer(uint32_t buffer, const bool first=true){ return cam->setData(first ? LO_A : HI_A, WRITE_F, station, buffer, Camac::STATUS_Q | Camac::STATUS_X);}//Owerwrite buffer and execute(if switch is in 1)
     bool execute(uint8_t reg){ return cam->Control(reg, EXEC_F, station, Camac::STATUS_Q | Camac::STATUS_X);}//Gives execute command (if switch is 2)
  };
}
#endif
