#include <iostream>
#include "Camac.h"

#ifndef LOGIC_H
#define LOGIC_H

namespace Logic
{
  class DualGate
  //This class describes operations related to the Harwell 95/7020-1/6 Dual Gate logic module
  //The dual gate creates logic output at A=1*2*3*4 and B=5*6*7*8, where *=.AND., where user can select and specify the polarity of inputs
  //The "input logic=1" equals to 'high' level of input, if direct has been set. If "N" is set, the "input logic=1" appears if the level of input is low.
  {
    public:
      //public methods
      DualGate(Camac &camac, uint8_t station_number=11);//Constructor: selects the camac comminicator and station number. Program retrieve the actual control register 
      ~DualGate(){};//Destructor
      //All of the following string configuration is the following: "1".."8" set as direct AND, "N1"/"n1"..."N8"/"n8" sets as negative input AND
      std::string GetConfiguration(bool direct=false);//Retrieve the actual configuration of the dual gate. direct: read from module(t)/return with the stored value (f)
      bool SetConfiguration(std::string config);//Sets the "config" configuration
      bool SelectiveConfiguration(std::string config, bool set=true);//Selectively sets(t)/resets(f) the configuration (for reset, all configuration will be direct)
    private:
      //private members
      uint16_t actual_control_register;
      Camac *cam;//Camac controller
      uint8_t station;//Camac station number
      //static members
      static const uint8_t READ_F=0;//Read function code
      static const uint8_t READ_A=0;//Read subadress
      static const uint8_t WRITE_F=16;//Write function code
      static const uint8_t WRITE_A_SELECT_1_4=1;//Select 1..4 for A subcode
      static const uint8_t WRITE_A_SELECT_5_8=2;//Select 5..8 for B subcode
      static const uint8_t WRITE_A_NEG_1_4=4;//Negative input 1..4 for A subcode
      static const uint8_t WRITE_A_NEG_5_8=8;//Negative input 1..4 for A subcode
      //private methods
      bool read_control_register();//Reads the actual value of the control register
      bool set_control_register(uint16_t value);//Sets the control register and updates its local representation
      uint16_t str2bit(const std::string input);//Converts string logics into bits
      std::string bit2str(const uint16_t input);//Converts string logics into bits
  };
}

#endif