#include "MTESTControl.h"

std::string MTESTControl::CommandIP(std::string commandline)
//This method does the interpreatation of command, transmits to hardware and returns with the answer
{
  std::stringstream ss(commandline);//Transforming commandline string to stream
  std::string command;//Command
  ss>>command;//Get the command
  //Transmit to the specific interpreter
  if (command=="stop")
  {
    //First: we assume, that motor is moving: trying to stop motor
    if(!motor.empty())
    {
      bool retval=Positioner::Motor::Stop();//Try to stop motor
      while(retval){usleep(10000); retval=!(Positioner::Motor::isAllStopped());}//If stopping process started: waiting for end of the process 
    }
    //TODO Second: we should stop running measurement
    //Return with success
    return std::string("1");
  }
  else if((command[0]=='k')||(command[0]=='K')) return do_camac_operation(command, ss);//CAMAC controller command
  else if((command[0]=='m')||(command[0]=='M')) return do_motor_operation(command, ss);//Motor operation controlling command
  else if((command[0]=='c')||(command[0]=='C')) return do_counter_operation(command, ss);//Counter operation controlling command
  else return "0";//ANSWER: "0(command does not exist)"
}

bool MTESTControl::readCfgFile(std::ifstream &is)
//This method read the configuration file and try to initialize data structure
{
  if(is)
  {
    is.seekg(0,is.beg);//Set the pointer to the first line
    //Create maps
    std::map<std::string,IOOperation::InterruptRegister*> irname;//This map associates the id to the element of the interrupt register array
    std::map<std::string,IOOperation::OutputRegister*> orname;//This map associates the id to the element of the output register array
    std::map<std::string,Positioner::StepMotorController*> smcname;//This map associates the id to the element of the step motor controller array
    while(!is.eof())
    {
      std::string line=getNextRelevantLine(is);//Get the next line
      if((line.find("Section")==std::string::npos)&&(line.size()>0))
      {
	//Skip Section, EndSection and empty lines
	std::stringstream ls(line);//Get the line into stream
	std::string command;//command word
	ls>>command;//Get the command word
	//Camac controllers
	if (command=="PCAM")
	{
	  //PCAM(parallel Camac) ID, port number set at the card in hexa, CAMAC crate number
	  std::string id;//ID of the controller
	  uint32_t port=0;//Start of IO port
	  int crate=0;//Crate number
	  //get the datasets
	  ls>>id>>std::hex>>port>>std::dec>>crate;
#ifdef DEBUG_MTEST_CONTROL
	  std::cout<<"Parallel Camac: id: \""<<id<<"\", port: 0x"<<std::hex<<port<<std::dec<<", crate: "<<crate<<std::endl;
#endif
	  camac.push_back(new ParallelCamac(port,(uint8_t)crate,id));//Create object
	}
	else if (command=="SCAM")
	{
	  //SCAM(serial Camac) ID, port(device): /dev/device, crate number, timeout in seconds [0.5], baud rate [2400], databits [7], parity [O as odd], stopbits [2]
	  std::string id;//ID of the controller
	  std::string port="camac_serial";//port(device): /dev/device
	  int crate=0;//crate number
	  double timeout=0.0;//timeout in seconds
	  int baud=0;//Baud rate
	  int datab=0;//Data bits
	  char parity='N';//Data parity
	  int stopb=0;//Number of stopbits
	  //get the datasets
	  ls>>id>>port>>crate>>timeout>>baud>>datab>>parity>>stopb;
#ifdef DEBUG_MTEST_CONTROL
	  std::cout<<"Serial Camac: id: \""<<id<<"\", port: "<<port<<", crate: "<<crate<<std::endl;
	  std::cout<<"\t timeout: "<<timeout<<"s, baud rate: "<<baud<<", data bits: "<<datab<<", parity: "<<((parity=='O') ? "ODD" : (parity=='E' ? "EVEN" : "NO"))<<", stop bits: "<<stopb<<std::endl;
#endif
	  camac.push_back(new SerialCamac(port,(uint8_t)crate,id,timeout,baud,datab,parity,stopb));//Create object
	}
	//IO devices
	else if (command=="IRREG")
	{
	  //IRREG(interrupt register) ID, CAMAC device ID, station number
	  std::string id;//ID of the interrupt register
	  std::string camid;//Camac id
	  int station;//station number
	  ls>>id>>camid>>station;//Get the datasets
	  //Looking for CAMAC ID
	  int camindx=-1;
	  for(int i=0; (i<(signed)camac.size())&&(camindx==-1); i++) if((camac[i])->getName() ==camid) camindx=i;
	  if(camindx==-1)
	  {
	    std::cerr<<"ERROR: at line IRREG: the \""<<camid<<"\" ID camac is not exist!"<<std::endl;
	    return false;
	  }
#ifdef DEBUG_MTEST_CONTROL
	  std::cout<<"Interrupt register: id: \""<<id<<"\", on camac controller \""<<camac[camindx]->getName()<<"\", station: "<<station<<std::endl;
#endif
	  iregister.push_back(new IOOperation::InterruptRegister(*(camac[camindx]),(uint8_t)station));//Allocate interrupt register
	  irname.insert(std::pair<std::string,IOOperation::InterruptRegister*>(id,iregister.back()));//Bind ID to the present index
	}
	else if (command=="OPREG")
	{
	  //OPREG(output register) ID, CAMAC device ID, station number, command and execute (1) / command, but separate execute (0)
	  std::string id;//ID of the output register
	  std::string camid;//Camac id
	  int station;//station number
	  int cex;//Command and execute (1) or separate (0)
	  ls>>id>>camid>>station>>cex;//Get the datasets
	  //Looking for CAMAC ID
	  int camindx=-1;
	  for(int i=0; (i<(signed)camac.size())&&(camindx==-1); i++) if((camac[i])->getName() ==camid) camindx=i;
	  if(camindx==-1)
	  {
	    std::cerr<<"ERROR: at line OPREG: the \""<<camid<<"\" ID camac is not exist!"<<std::endl;
	    return false;
	  }
#ifdef DEBUG_MTEST_CONTROL
	  std::cout<<"Output register: id: \""<<id<<"\", on camac controller \""<<camac[camindx]->getName()<<"\", station: "<<station<<", operation: "<<(cex? "command&execute" : "command;execute")<<std::endl;
#endif
	  oregister.push_back(new IOOperation::OutputRegister(*(camac[camindx]),(uint8_t)station,(bool)cex));//Allocate output register
	  orname.insert(std::pair<std::string,IOOperation::OutputRegister*>(id,oregister.back()));//Bind ID to the present index
	}
	//Motors and motor controllers
	else if (command=="SMTRCTRL")
	{
	  //SMTRCTRL(step motor controller) ID, CAMAC device ID, station number
	  std::string id;//ID of the step motor controller
	  std::string camid;//Camac id
	  int station;//station number
	  ls>>id>>camid>>station;//Get the datasets
	  //Looking for CAMAC ID
	  int camindx=-1;
	  for(int i=0; (i<(signed)camac.size())&&(camindx==-1); i++) if((camac[i])->getName() ==camid) camindx=i;
	  if(camindx==-1)
	  {
	    std::cerr<<"ERROR: at line SMTRCTRL: the \""<<camid<<"\" ID camac is not exist!"<<std::endl;
	    return false;
	  }
#ifdef DEBUG_MTEST_CONTROL
	  std::cout<<"Step motor controller: id: \""<<id<<"\", on camac controller \""<<camac[camindx]->getName()<<"\", station: "<<station<<std::endl;
#endif
	  smc.push_back(new Positioner::StepMotorController(*(camac[camindx]),station));//Allocate step motor controller
	  smcname.insert(std::pair<std::string,Positioner::StepMotorController*>(id,smc.back()));//Bind ID to the present index
	  
	}
	else if (command=="HWMOTOR")
	{
	  //HWMOTOR(harwell motor) name, CAMAC device ID, station number, low limit in general impulse, high limit in general impulse, output register ID, maximal accepted difference in pulses [0]
	  std::string id;//ID of the harwell motor
	  std::string camid;//Camac id
	  int station;//station number
	  int limits[2];//Low and high limits in general impulse
	  std::string orid;//Associated output register ID
	  int max_acc_diff;//Maximal accepted difference
	  ls>>id>>camid>>station>>limits[0]>>limits[1]>>orid>>max_acc_diff;//Get the datasets
	  //Looking for CAMAC and output register ID
	  int camindx=-1;
	  for(int i=0; (i<(signed)camac.size())&&(camindx==-1); i++) if((camac[i])->getName() ==camid) camindx=i;
	  IOOperation::OutputRegister* ore=orname.find(orid)->second;
	  if((camindx==-1)||(ore==0))
	  {
	    std::cerr<<"ERROR: at line HWMOTOR: the \""<<camid<<"\" ID camac or the \""<<orid<<"\" output register label does not exist!"<<std::endl;
	    return false;
	  }
#ifdef DEBUG_MTEST_CONTROL
	  std::cout<<"Harwell motor: id: \""<<id<<"\", on camac controller \""<<camac[camindx]->getName()<<"\", station: "<<station<<std::endl;
	  std::cout<<"\tAllowed range: "<<limits[0]<<".."<<limits[1]<<" pulses, output register: \""<<orid<<"\", maximal accepted difference: "<<max_acc_diff<<" pulses"<<std::endl;
#endif
	  motor.push_back(new Positioner::HarwellMotor(*(camac[camindx]),id, (uint8_t)station, (uint32_t)limits[1], (uint32_t)limits[0], *ore, max_acc_diff));//Create object
	}
	else if (command=="PMOTOR")
	{
	  //PMOTOR(parallel motor) name, CAMAC device ID, "CAMAC Ablesemodul" station number [2], motor number in "CAMAC Ablesemodul", motor number in electronic unit (output), low limit in general impulse, high limit in general impulse, bit number of the encoder, output register ID, interrupt register ID, step motor controller ID, time to reach high speed from low in seconds, lowest speed of the motor in step/seconds, fastest speed of the motor in step/seconds, maximal accepted difference in pulses [0]
	  std::string id;//ID of the parallel motor
	  std::string camid;//Camac id
	  int station;//Station number of the "CAMAC Ablesemodul"
	  int motor_no_ablese;//motor number in ablese modul
	  int motor_no_electronic;//motor number in electronic unit
	  int limits[2];//Low and high limits in general impulse
	  int bits;//Bit number of the encoder
	  std::string orid;//Associated output register ID
	  std::string irid;//Associated interrupt register ID
	  std::string smcid;//Associated step motor controller ID
	  double ramp_sec;//time to reach hi-speed from low in seconds
	  int speed[2];//Low and high-speed of motor in step/seconds
	  int max_acc_diff;//Maximal accepted difference
	  ls>>id>>camid>>station>>motor_no_ablese>>motor_no_electronic>>limits[0]>>limits[1]>>bits>>orid>>irid>>smcid>>ramp_sec;
	  ls>>speed[0]>>speed[1]>>max_acc_diff;//Get the datasets
	  //Looking for CAMAC, output register, interrupt register and step motor controller ID
	  int camindx=-1;
	  for(int i=0; (i<(signed)camac.size())&&(camindx==-1); i++) if((camac[i])->getName() ==camid) camindx=i;
	  IOOperation::OutputRegister* ore=orname.find(orid)->second;
	  IOOperation::InterruptRegister* ire=irname.find(irid)->second;
	  Positioner::StepMotorController* smco=smcname.find(smcid)->second;
	  if((camindx==-1)||(ore==0)||(ire==0)||(smco==0))
	  {
	    std::cerr<<"ERROR: at line PMOTOR: the \""<<camid<<"\" ID camac or the \""<<orid<<"\" output register,"<<std::endl;
	    std::cerr<<"\tor the \""<<irid<<"\" interrupt register, or the \""<<smcid<<"\" step motor controller label does not exist!"<<std::endl;
	    return false;
	  }
#ifdef DEBUG_MTEST_CONTROL
	  std::cout<<"Parallel motor: id: \""<<id<<"\", on camac controller \""<<camac[camindx]->getName()<<"\", digitizer station: "<<station<<std::endl;
	  std::cout<<"\tmotor number at digitizer: "<<motor_no_ablese<<", motor number in control electronics: "<<motor_no_electronic<<",\n\tbit number of the encoder: "<<bits<<std::endl;
	  std::cout<<"\tAllowed range: "<<limits[0]<<".."<<limits[1]<<" pulses,\n\toutput register: \""<<orid<<"\", interrupt register: \""<<irid<<"\", step motor controller id: \""<<smcid<<"\",\n\ttime to reach high-speed: "<<ramp_sec<<" s, slowest speed: "<<speed[0]<<" step/s, fastest speed: "<<speed[1]<<" step/s,\n\tmaximal accepted difference: "<<max_acc_diff<<" pulses"<<std::endl;
#endif
	  motor.push_back(new Positioner::ParallelMotor(*(camac[camindx]),id, (uint8_t) motor_no_ablese, (uint8_t) motor_no_electronic, (uint32_t)limits[1], (uint32_t)limits[0], *ore, *ire, *smco, ramp_sec, speed[0], speed[1],  (uint8_t)station, max_acc_diff, bits));//Create object
	}
	else if (command=="APMOTOR")
	{
	  //APMOTOR(parallel motor, related to angle) name, CAMAC device ID, "CAMAC Ablesemodul" station number [2], motor number in "CAMAC Ablesemodul", motor number in electronic unit (output), low limit in general impulse, high limit in general impulse, bit number of the encoder, output register ID, interrupt register ID, step motor controller ID, time to reach high speed from low in seconds, lowest speed of the motor in step/seconds, fastest speed of the motor in step/seconds, maximal accepted difference in pulses [0], shift of degree coefficient, slope of degree coefficient, according to degree=slope*(pos-shift) equation
	  std::string id;//ID of the parallel motor
	  std::string camid;//Camac id
	  int station;//Station number of the "CAMAC Ablesemodul"
	  int motor_no_ablese;//motor number in ablese modul
	  int motor_no_electronic;//motor number in electronic unit
	  int limits[2];//Low and high limits in general impulse
	  int bits;//Bit number of the encoder
	  std::string orid;//Associated output register ID
	  std::string irid;//Associated interrupt register ID
	  std::string smcid;//Associated step motor controller ID
	  double ramp_sec;//time to reach hi-speed from low in seconds
	  int speed[2];//Low and high-speed of motor in step/seconds
	  int max_acc_diff;//Maximal accepted difference
	  double shft;//Shift coefficient angle=slope*(pos-shift)
	  double slope;//slope coefficient angle=slope*(pos-shift)
	  ls>>id>>camid>>station>>motor_no_ablese>>motor_no_electronic>>limits[0]>>limits[1]>>bits>>orid>>irid>>smcid>>ramp_sec;
	  ls>>speed[0]>>speed[1]>>max_acc_diff>>shft>>slope;//Get the datasets
	  //Looking for CAMAC, output register, interrupt register and step motor controller ID
	  int camindx=-1;
	  for(int i=0; (i<(signed)camac.size())&&(camindx==-1); i++) if((camac[i])->getName() ==camid) camindx=i;
	  IOOperation::OutputRegister* ore=orname.find(orid)->second;
	  IOOperation::InterruptRegister* ire=irname.find(irid)->second;
	  Positioner::StepMotorController* smco=smcname.find(smcid)->second;
	  if((camindx==-1)||(ore==0)||(ire==0)||(smco==0))
	  {
	    std::cerr<<"ERROR: at line APMOTOR: the \""<<camid<<"\" ID camac or the \""<<orid<<"\" output register,"<<std::endl;
	    std::cerr<<"\tor the \""<<irid<<"\" interrupt register, or the \""<<smcid<<"\" step motor controller label does not exist!"<<std::endl;
	    return false;
	  }
#ifdef DEBUG_MTEST_CONTROL
	  std::cout<<"AngleParallel motor: id: \""<<id<<"\", on camac controller \""<<camac[camindx]->getName()<<"\", digitizer station: "<<station<<std::endl;
	  std::cout<<"\tmotor number at digitizer: "<<motor_no_ablese<<", motor number in control electronics: "<<motor_no_electronic<<",\n\tbit number of the encoder: "<<bits<<std::endl;
	  std::cout<<"\tAllowed range: "<<limits[0]<<".."<<limits[1]<<" pulses,\n\toutput register: \""<<orid<<"\", interrupt register: \""<<irid<<"\", step motor controller id: \""<<smcid<<"\",\n\ttime to reach high-speed: "<<ramp_sec<<" s, slowest speed: "<<speed[0]<<" step/s, fastest speed: "<<speed[1]<<" step/s,\n\tmaximal accepted difference: "<<max_acc_diff<<" pulses, angle="<<slope<<"*(position-"<<shft<<") degrees"<<std::endl;
#endif
	  motor.push_back(new Positioner::AngleParallelMotor(*(camac[camindx]),id, (uint8_t) motor_no_ablese, (uint8_t) motor_no_electronic, (uint32_t)limits[1], (uint32_t)limits[0], *ore, *ire, *smco, ramp_sec, speed[0], speed[1],shft,slope,  (uint8_t)station, max_acc_diff, bits));//Create object
	}
	else if (command=="CAPMOTOR")
	{
	  //CAPMOTOR(parallel motor, related to angle and operated with air-cushion) name, CAMAC device ID, "CAMAC Ablesemodul" station number [2], motor number in "CAMAC Ablesemodul", motor number in electronic unit (output), low limit in general impulse, high limit in general impulse, bit number of the encoder, output register ID, interrupt register ID, step motor controller ID, time to reach high speed from low in seconds, lowest speed of the motor in step/seconds, fastest speed of the motor in step/seconds, maximal accepted difference in pulses [0], shift of degree coefficient, slope of degree coefficient, according to degree=slope*(pos-shift) equation
	  std::string id;//ID of the parallel motor
	  std::string camid;//Camac id
	  int station;//Station number of the "CAMAC Ablesemodul"
	  int motor_no_ablese;//motor number in ablese modul
	  int motor_no_electronic;//motor number in electronic unit
	  int limits[2];//Low and high limits in general impulse
	  int bits;//Bit number of the encoder
	  std::string orid;//Associated output register ID
	  std::string irid;//Associated interrupt register ID
	  std::string smcid;//Associated step motor controller ID
	  double ramp_sec;//time to reach hi-speed from low in seconds
	  int speed[2];//Low and high-speed of motor in step/seconds
	  int max_acc_diff;//Maximal accepted difference
	  double shft;//Shift coefficient angle=slope*(pos-shift)
	  double slope;//slope coefficient angle=slope*(pos-shift)
	  ls>>id>>camid>>station>>motor_no_ablese>>motor_no_electronic>>limits[0]>>limits[1]>>bits>>orid>>irid>>smcid>>ramp_sec;
	  ls>>speed[0]>>speed[1]>>max_acc_diff>>shft>>slope;//Get the datasets
	  //Looking for CAMAC, output register, interrupt register and step motor controller ID
	  int camindx=-1;
	  for(int i=0; (i<(signed)camac.size())&&(camindx==-1); i++) if((camac[i])->getName() ==camid) camindx=i;
	  IOOperation::OutputRegister* ore=orname.find(orid)->second;
	  IOOperation::InterruptRegister* ire=irname.find(irid)->second;
	  Positioner::StepMotorController* smco=smcname.find(smcid)->second;
	  if((camindx==-1)||(ore==0)||(ire==0)||(smco==0))
	  {
	    std::cerr<<"ERROR: at line CAPMOTOR: the \""<<camid<<"\" ID camac or the \""<<orid<<"\" output register,"<<std::endl;
	    std::cerr<<"\tor the \""<<irid<<"\" interrupt register, or the \""<<smcid<<"\" step motor controller label does not exist!"<<std::endl;
	    return false;
	  }
#ifdef DEBUG_MTEST_CONTROL
	  std::cout<<"CushionAP motor: id: \""<<id<<"\", on camac controller \""<<camac[camindx]->getName()<<"\", digitizer station: "<<station<<std::endl;
	  std::cout<<"\tmotor number at digitizer: "<<motor_no_ablese<<", motor number in control electronics: "<<motor_no_electronic<<",\n\tbit number of the encoder: "<<bits<<std::endl;
	  std::cout<<"\tAllowed range: "<<limits[0]<<".."<<limits[1]<<" pulses,\n\toutput register: \""<<orid<<"\", interrupt register: \""<<irid<<"\", step motor controller id: \""<<smcid<<"\",\n\ttime to reach high-speed: "<<ramp_sec<<" s, slowest speed: "<<speed[0]<<" step/s, fastest speed: "<<speed[1]<<" step/s,\n\tmaximal accepted difference: "<<max_acc_diff<<" pulses, angle="<<slope<<"*(position-"<<shft<<") degrees"<<std::endl;
#endif
	  motor.push_back(new Positioner::CushionAPMotor(*(camac[camindx]),id, (uint8_t) motor_no_ablese, (uint8_t) motor_no_electronic, (uint32_t)limits[1], (uint32_t)limits[0], *ore, *ire, *smco, ramp_sec, speed[0], speed[1],shft,slope,  (uint8_t)station, max_acc_diff, bits));//Create object
	}
	//Counters
	else if (command=="PSDCOUNTER")
	{
	  //PSDCOUNTER(TrumpPCI card) name, number of channels, low limit discriminator, high limit discriminator, gate mode: O(ff)/C(oincidence)/A(nti coincidence), deadtime delay in us [1.6], serial number of the card [-1], adc_offset [2048], timeout in ms [0.1]
	  std::string id;//ID of thecounter
	  int no_channels;//No of channels
	  int discriminator[2];//Low and high level of discriminator
	  char gate;//Gate mode indicator
	  double dtdelay;//Delay of deadtime
	  int ser_no;//Serial number of the card
	  int adc_offset;//ADC offset value
	  double timeout_in_ms;//Timeout in ms
	  //Get the datasets
	  ls>>id>>no_channels>>discriminator[0]>>discriminator[1]>>gate>>dtdelay>>ser_no>>adc_offset>>timeout_in_ms;
	  GATE_MODE gm;//Gate mode
	  //Converting input to gate mode
	  if(gate=='O') gm=GATE_OFF;
	  else if(gate=='C') gm=GATE_COINCIDENCE;
	  else if(gate=='A') gm=GATE_ANTICOINCIDENCE;
	  else return false;
#ifdef DEBUG_MTEST_CONTROL
	  std::cout<<"PSD counter: id: \""<<id<<"\""<<std::endl; 
	  std::cout<<"\tnumber of channels: "<<no_channels<<", discriminator levels: "<<discriminator[0]<<" and "<<discriminator[1]<<std::endl;
	  std::cout<<"\t gate mode: "<<((gm==GATE_OFF) ? "OFF" : ((gm==GATE_COINCIDENCE)? "COINCIDENCE" : "ANTICOINCIDENCE"))<<", deadtime delay: "<<dtdelay<<" us, card serial number: "<<ser_no<<", ADC offset: "<<adc_offset<<", card timeout: "<<timeout_in_ms<<" ms"<<std::endl;
#endif	  
	  psd.push_back(new Scaler::PSDCounter(id,0.0,0.0,no_channels,discriminator[0],discriminator[1],gm,dtdelay,ser_no,timeout_in_ms,(uint16_t)adc_offset));//Add item to the array
	}
	else if (command=="MONCOUNTER")
	{
	  //MONCOUNTER (KFKI monitor counter) name, CAMAC device ID, station number, low intensity warning level in cps, how_many_times [0..16]
	  std::string id;//ID of the counter
	  std::string camid;//ID of the Camac module
	  int station_no;//Station number
	  int low_int_warning;//Low intensity warning level in cps
	  int how_many_times;//How many times should appear below the warning level to treat as low intensity
	  //Get the datasets
	  ls>>id>>camid>>station_no>>low_int_warning>>how_many_times;
	  int camindx=-1;
	  for(int i=0; (i<(signed)camac.size())&&(camindx==-1); i++) if((camac[i])->getName() ==camid) camindx=i;
#ifdef DEBUG_MTEST_CONTROL
	  std::cout<<"Monitor counter: id: \""<<id<<"\" is set as station "<<station_no<<" on \""<<camac[camindx]->getName()<<"\""<<std::endl;
	  std::cout<<"\tLow intensity warning level: "<<low_int_warning<<"cps should be appear "<<how_many_times<<" times to qualify as low"<<std::endl;
#endif	 
	  monitor.push_back(new Scaler::MonitorCounter(id, *(camac[camindx]), (uint8_t) station_no, false, (Scaler::MON_CNT) 0L, true, (uint16_t) low_int_warning, (uint8_t) how_many_times, true, true));//Add item to the array
	}
	else if (command=="HWCOUNTER")
	{
	  //HWCOUNTER (Harwell preset counter) name, CAMAC device ID, station number, setup_codeword [A0N(ow)|A1N(ow)|BPU(lse)|A0(ne)X(t)|A1(ne)X(t)|LA(m ne)X(t)|A0C(arry)|A1C(arry)|LA(m )E(nable)]
	  std::string id;//ID of the counter
	  std::string camid;//ID of the Camac module
	  int station_no;//Station number
	  std::string codeword;//Codeword of operation
	  //Get the datasets
	  ls>>id>>camid>>station_no>>codeword;
	  //Camac controller
	  int camindx=-1;
	  for(int i=0; (i<(signed)camac.size())&&(camindx==-1); i++) if((camac[i])->getName() ==camid) camindx=i;
#ifdef DEBUG_MTEST_CONTROL
	  std::cout<<"Harwell counter: id: \""<<id<<"\" is set as station "<<station_no<<" on \""<<camac[camindx]->getName()<<"\""<<std::endl;
	  std::cout<<"\tCodeword: 0x"<<std::hex<<Scaler::HarwellCounter::str2code(codeword)<<std::dec<<std::endl;
#endif	
	  hwcnt.push_back(new Scaler::HarwellCounter(id, *(camac[camindx]),(uint8_t) station_no, (Scaler::HW_CNT) 0L, Scaler::HarwellCounter::str2code(codeword), true));//Add item to the array
	}
	else if (command=="RTCCOUNTER")
	{
	  //RTCCOUNTER (SEN RTC timer) name, CAMAC device ID, station number, frequency division [0(stop)/1/8/64/512/4096/32768/262144]
	  std::string id;//ID of the counter
	  std::string camid;//ID of the Camac module
	  int station_no;//Station number
	  int freq_div;//Frequency division
	  //Get the datasets
	  ls>>id>>camid>>station_no>>freq_div;
	  //Camac controller
	  int camindx=-1;
	  for(int i=0; (i<(signed)camac.size())&&(camindx==-1); i++) if((camac[i])->getName() ==camid) camindx=i;
#ifdef DEBUG_MTEST_CONTROL
	  std::cout<<"RTC counter/timer: id: \""<<id<<"\" is set as station "<<station_no<<" on \""<<camac[camindx]->getName()<<"\""<<std::endl;
	  std::cout<<"\tFrequency divison: "<<(int)freq_div<<std::endl;
#endif	
	  rtccnt.push_back(new Scaler::RTCCounter(id, *(camac[camindx]),(uint8_t) station_no, (Scaler::RTC_CNT) 0L, Scaler::RTCCounter::num2div(freq_div), true));//Add item to the array
	}
	else return false;
      }
    }
    return true;
  }
  else return false;//The stream has not been opened yet
}


bool MTESTControl::writeCfgFile(std::ofstream &os)
//This method writes the configuration file, according to the present setup
{
  if(os)
  {
    //The file is already open -- write some important information out
    os<<"#This is the control file of MTEST controller"<<std::endl;
    os<<"# the line starting with \"#\" sign is interpreted as comment."<<std::endl<<"#"<<std::endl;
    //CAMAC
    os<<"#The section CAMAC contains the initialisation of CAMAC modules"<<std::endl;
    os<<"# -PCAM(parallel Camac) ID, port number set at the card in hexa, CAMAC crate number"<<std::endl;
    os<<"# -SCAM(serial Camac) ID, port (device): /dev/{device}, crate number, timeout in seconds [0.5], baud rate [2400], databits [7], parity [O as odd], stopbits [2]"<<std::endl;
    os<<"Section CAMAC"<<std::endl;
    if(!camac.empty())
    {
      //Writing out the configuration
      for(int i=0; i<(int)camac.size(); i++)
      {
	//Determine the real type
	ParallelCamac *pc=dynamic_cast<ParallelCamac*> (camac[i]);
	SerialCamac *sc=dynamic_cast<SerialCamac*> (camac[i]);
	if(pc!=0)
	{
	  os<<"\tPCAM "<<pc->getName()<<" 0x"<<std::hex<<pc->getStartPortNo()<<std::dec<<" "<<(int)(pc->getCrate())<<std::endl;
	}
	else if (sc!=0)
	{
	  int baud;//Baud rate
	  int databits;//Number of data bits
	  int stopbits;//Number of stopbits
	  char parity;//Parity
	  sc->getCommParameters(&baud,&databits,&parity,&stopbits);//Get the setup parameters
	  os<<"\tSCAM "<<sc->getName()<<" "<<sc->getPortName()<<" "<<(int)(sc->getCrate())<<" "<<sc->getTimeout()<<" "<<baud<<" "<<databits<<" "<<parity<<" "<<stopbits<<std::endl;
	}
	else return false;//Non-specified case
      }
    }
    os<<"EndSection CAMAC"<<std::endl;
    //IO devices
    os<<"#IO devices:"<<std::endl;
    os<<"# -IRREG(interrupt register) ID, CAMAC device ID, station number"<<std::endl;
    os<<"# -OPREG(output register) ID, CAMAC device ID, station number, command and execute (1) / command, but separate execute (0)"<<std::endl;
    os<<"Section IODEV"<<std::endl;
    if(!iregister.empty()) for(int i=0; i<(int)iregister.size(); i++) os<<"\tIRREG ireg"<<i+1<<" "<<iregister[i]->getCAMName()<<" "<<(int)(iregister[i]->getStation())<<std::endl;//Interrupt registers
    if(!oregister.empty()) for(int i=0; i<(int)oregister.size(); i++) os<<"\tOPREG oreg"<<i+1<<" "<<oregister[i]->getCAMName()<<" "<<(int)(oregister[i]->getStation())<<((oregister[i]->getExecuteJumper()) ? " 1" : " 0")<<std::endl;//Output registers
    os<<"EndSection IODEV"<<std::endl;
    //Motor controllers and motors
    os<<"#Motors and motor controllers"<<std::endl;
    os<<"# -SMTRCTRL(step motor controller) ID, CAMAC device ID, station number"<<std::endl;
    os<<"# -HWMOTOR(harwell motor) name, CAMAC device ID, station number, low limit in general impulse, high limit in general impulse, output register ID, maximal accepted difference in pulses [0]"<<std::endl;
    os<<"# -PMOTOR(parallel motor) name, CAMAC device ID, \"CAMAC Ablesemodul\" station number [2], motor number in \"CAMAC Ablesemodul\", motor number in electronic unit (output), low limit in general impulse, high limit in general impulse, bit number of the encoder, output register ID, interrupt register ID, step motor controller ID, time to reach high speed from low in seconds, lowest speed of the motor in step/seconds, fastest speed of the motor in step/seconds, maximal accepted difference in pulses [0]"<<std::endl;
    os<<"# -APMOTOR(parallel motor, related to angle) name, CAMAC device ID, \"CAMAC Ablesemodul\" station number [2], motor number in \"CAMAC Ablesemodul\", motor number in electronic unit (output), low limit in general impulse, high limit in general impulse, bit number of the encoder, output register ID, interrupt register ID, step motor controller ID, time to reach high speed from low in seconds, lowest speed of the motor in step/seconds, fastest speed of the motor in step/seconds, maximal accepted difference in pulses [0], shift of degree coefficient, slope of degree coefficient, according to degree=slope*(pos-shift) equation"<<std::endl;
    os<<"# -CAPMOTOR(parallel motor, related to angle and operated with air-cushion) name, CAMAC device ID, \"CAMAC Ablesemodul\" station number [2], motor number in \"CAMAC Ablesemodul\", motor number in electronic unit (output), low limit in general impulse, high limit in general impulse, bit number of the encoder, output register ID, interrupt register ID, step motor controller ID, time to reach high speed from low in seconds, lowest speed of the motor in step/seconds, fastest speed of the motor in step/seconds, maximal accepted difference in pulses [0], shift of degree coefficient, slope of degree coefficient, according to degree=slope*(pos-shift) equation"<<std::endl;
    os<<"Section MOTOR"<<std::endl;
    if(!smc.empty()) for(int i=0; i<(int)smc.size(); i++) os<<"\tSMTRCTRL smc"<<i+1<<" "<<smc[i]->getCAMName()<<" "<<(int)(smc[i]->getStation())<<std::endl;//Step motor controller
    if(!motor.empty())
    {
      //Writing out the configuration
      for(int i=0; i<(int)motor.size(); i++)
      {
	//Determine the real type
	Positioner::HarwellMotor *hwm=dynamic_cast<Positioner::HarwellMotor*> (motor[i]);
	Positioner::ParallelMotor *pm=dynamic_cast<Positioner::ParallelMotor*> (motor[i]);
	Positioner::AngleParallelMotor *apm=dynamic_cast<Positioner::AngleParallelMotor*> (motor[i]);
	Positioner::CushionAPMotor *capm=dynamic_cast<Positioner::CushionAPMotor*> (motor[i]);
	if(hwm!=0)
	{
	  int orindx=-1;
	  for(int i=0; i<(oregister.size()&&(orindx==-1)); i++) if(oregister[i]==(hwm->getOutReg())) orindx=i; 
	  os<<"\tHWMOTOR "<<hwm->getName()<<" "<<hwm->getCAMName()<<" "<<(int)(hwm->getAddress())<<" "<<(int)(hwm->getLimit(false))<<" "<<(int)(hwm->getLimit(true))<<" oreg"<<orindx+1<<" "<<hwm->getAcceptedDifference()<<std::endl;
	}
	else if (capm!=0)
	{
	  int orindx=-1;
	  for(int i=0; i<(oregister.size()&&(orindx==-1)); i++) if(oregister[i]==(capm->getOutReg())) orindx=i;
	  int irindx=-1;
	  for(int i=0; i<(iregister.size()&&(irindx==-1)); i++) if(iregister[i]==(capm->getIrReg())) irindx=i;
	  int smcindx=-1;
	  for(int i=0; i<(smc.size()&&(smcindx==-1)); i++) if(smc[i]==(capm->getSMC())) smcindx=i;
	  os<<"\tCAPMOTOR "<<capm->getName()<<" "<<capm->getCAMName()<<" "<<(int)(capm->getDisplayStation())<<" "<<(int)(capm->getAddress())<<" "<<(int)(capm->getMotorOutputNo())<<" "<<(int)(capm->getLimit(false))<<" "<<(int)(capm->getLimit(true))<<" "<<capm->getEncoderBits()<<" oreg"<<orindx+1<<" ireg"<<irindx+1<<" smc"<<smcindx+1<<" "<<capm->getRamp()<<" "<<capm->getSpeed(false)<<" "<<capm->getSpeed(true)<<" "<<capm->getAcceptedDifference()<<" "<<capm->getConversion(true)<<" "<<capm->getConversion(false)<<std::endl;
	}
	else if (apm!=0)
	{
	  int orindx=-1;
	  for(int i=0; i<(oregister.size()&&(orindx==-1)); i++) if(oregister[i]==(apm->getOutReg())) orindx=i;
	  int irindx=-1;
	  for(int i=0; i<(iregister.size()&&(irindx==-1)); i++) if(iregister[i]==(apm->getIrReg())) irindx=i;
	  int smcindx=-1;
	  for(int i=0; i<(smc.size()&&(smcindx==-1)); i++) if(smc[i]==(apm->getSMC())) smcindx=i;
	  os<<"\tAPMOTOR "<<apm->getName()<<" "<<apm->getCAMName()<<" "<<(int)(apm->getDisplayStation())<<" "<<(int)(apm->getAddress())<<" "<<(int)(apm->getMotorOutputNo())<<" "<<(int)(apm->getLimit(false))<<" "<<(int)(apm->getLimit(true))<<" "<<apm->getEncoderBits()<<" oreg"<<orindx+1<<" ireg"<<irindx+1<<" smc"<<smcindx+1<<" "<<apm->getRamp()<<" "<<apm->getSpeed(false)<<" "<<apm->getSpeed(true)<<" "<<apm->getAcceptedDifference()<<" "<<apm->getConversion(true)<<" "<<apm->getConversion(false)<<std::endl;
	}
	else if (pm!=0)
	{
	  int orindx=-1;
	  for(int i=0; i<(oregister.size()&&(orindx==-1)); i++) if(oregister[i]==(pm->getOutReg())) orindx=i;
	  int irindx=-1;
	  for(int i=0; i<(iregister.size()&&(irindx==-1)); i++) if(iregister[i]==(pm->getIrReg())) irindx=i;
	  int smcindx=-1;
	  for(int i=0; i<(smc.size()&&(smcindx==-1)); i++) if(smc[i]==(pm->getSMC())) smcindx=i;
	  os<<"\tPMOTOR "<<pm->getName()<<" "<<pm->getCAMName()<<" "<<(int)(pm->getDisplayStation())<<" "<<(int)(pm->getAddress())<<" "<<(int)(pm->getMotorOutputNo())<<" "<<(int)(pm->getLimit(false))<<" "<<(int)(pm->getLimit(true))<<" "<<pm->getEncoderBits()<<" oreg"<<orindx+1<<" ireg"<<irindx+1<<" smc"<<smcindx+1<<" "<<pm->getRamp()<<" "<<pm->getSpeed(false)<<" "<<pm->getSpeed(true)<<" "<<pm->getAcceptedDifference()<<std::endl;
	}
	else return false;//Non-specified case
      }
    }
    os<<"EndSection MOTOR"<<std::endl;
    //Counters
    os<<"#Counters: detectors and timers"<<std::endl;
    os<<"# -PSDCOUNTER(TrumpPCI card) name, number of channels, low limit discriminator, high limit discriminator, gate mode: OFF/C(oincidence)/A(nti)C(oincidence), deadtime delay in us [1.6], serial number of the card [-1], adc_offset [2048], timeout in ms [0.1]"<<std::endl;
    os<<"# -MONCOUNTER (KFKI monitor counter) name, CAMAC device ID, station number, low intensity warning level in cps, how_many_times [0..16]"<<std::endl;
    os<<"# -HWCOUNTER (Harwell preset counter) name, CAMAC device ID, station number, setup_codeword [A0N(ow)|A1N(ow)|BPU(lse)|A0(ne)X(t)|A1(ne)X(t)|LA(m ne)X(t)|A0C(arry)|A1C(arry)|LA(m )E(nable)]"<<std::endl;
    os<<"# -RTCCOUNTER (SEN RTC timer) name, CAMAC device ID, station number, frequency division [0(stop)/1/8/64/512/4096/32768/262144]"<<std::endl;
    os<<"Section SCALER"<<std::endl;
    if(!psd.empty())
    {
      //Writing out the configuration
      for(int i=0; i<(int)psd.size(); i++)
      {
        //Get channel related setups
        int chn=0;//Number of channels
        int discriminator[2];//Discriminator levels
        if(!(psd[i]->GetChannel(&chn, discriminator, discriminator+1))) return false;
        //Get the actual gate setup
        char gate;//Gate setup
        GATE_MODE gm=psd[i]->GetGate();
        if(gm==GATE_OFF) gate='O';
        else if(gm==GATE_COINCIDENCE) gate='C';
        else if(gm==GATE_ANTICOINCIDENCE) gate='A';
        else return false;
        //Get the serial number of the card
        int sno;//serial number
        std::string firmware;//Firmware
        
        if(!(psd[i]->About(&sno,&firmware))) return false;
        os<<"\tPSDCOUNTER";
        std::vector<std::string> names=psd[i]->getNames();
        std::for_each(names.begin(), names.end(), [&os](std::string n){os<<" "<<n;});
        os<<" "<<chn<<" "<<discriminator[0]<<" "<<discriminator[1]<<" "<<gate<<" "<<psd[i]->getDTimeDelay()<<" "<<sno<<" "<<psd[i]->getOffset()<<" "<<psd[i]->getTimeout()<<std::endl;
      }
    }
    if(!monitor.empty())
    {
      //Writing out the configuration
      for(int i=0; i<(int)monitor.size(); i++)
      {
        uint16_t niv;//Poor level
        uint8_t consec;//Consecutive niv to qualify signal as poor
        monitor[i]->GetIntensityParameters(&niv, &consec);//Get the levels
        os<<"\tMONCOUNTER ";
        std::vector<std::string> names=monitor[i]->getNames();
        std::for_each(names.begin(), names.end(), [&os](std::string n){os<<" "<<n;});
        os<<" "<<monitor[i]->getCAMName()<<" "<<(int)(monitor[i]->getSlot())<<" "<<(int)niv<<" "<<(int)consec<<std::endl;
      }
    }
    if(!hwcnt.empty())
    {
      //Writing out the configuration
      for(int i=0; i<(int)hwcnt.size(); i++)
      {
          os<<"\tHWCOUNTER ";
          std::vector<std::string> names=hwcnt[i]->getNames();
          std::for_each(names.begin(), names.end(), [&os](std::string n){os<<" "<<n;});
          os<<" "<<hwcnt[i]->getCAMName()<<" "<<(int)(hwcnt[i]->getSlot())<<" "<<Scaler::HarwellCounter::code2str(hwcnt[i]->GetControlRegister())<<std::endl;
      }
    }
    if(!rtccnt.empty())
    {
      //Writing out the configuration
      for(int i=0; i<(int)rtccnt.size(); i++)
      {
          os<<"\tRTCCOUNTER ";
          std::vector<std::string> names=rtccnt[i]->getNames();
          std::for_each(names.begin(), names.end(), [&os](std::string n){os<<" "<<n;});
          os<<" "<<rtccnt[i]->getCAMName()<<" "<<(int)(rtccnt[i]->getSlot())<<" "<<Scaler::RTCCounter::div2num(rtccnt[i]->GetFrequencyDivisor())<<std::endl;//frequency division [0(stop)/1/8/64/512/4096/32768/262144]
      }
    }
    os<<"EndSection SCALER"<<std::endl;
    return true;
  }
  else return false;
}

std::string MTESTControl::do_camac_operation(std::string command, std::stringstream &ss)
//This program interpret camac related commands, performs action and returns with an appropriate message. Input is the command, then ss contains the following parameters
{
  std::transform(command.begin(), command.end(), command.begin(),::tolower);//Transforming the command string to lowercase
  std::stringstream ans;//Answer
  if(command=="klist")
  {
    //It should list the name of the available CAMAC counters: "klist"
    ans<<"1 "<<camac.size()<<" ";//Set accepted command number of CAMAC controllers
    for(int i=0; i<(int)camac.size(); i++) ans<<(camac[i])->getName()<<" ";
    //ANSWER: 1(command OK) number_of_constrollers name1 name2...
  }
  else if(command=="kcn")
  {
    //It should change the name of the CAMAC unit: "kcn old_name new_name"
    std::string actual_name;
    std::string name_to_change;
    ss>>actual_name;//Get the actual name
    ss>>name_to_change;//New name
    std::vector<Camac*>::const_iterator i=std::find_if(camac.begin(),camac.end(),compare_name<Camac>(actual_name));//Looking for the name
    if((i!=camac.end())&&(name_to_change.size()!=0)) (**i).setName(name_to_change);//Setting the requested name
    ans<<(((i!=camac.end())&&(name_to_change.size()!=0)) ? "1 1" : "1 0");
    //ANSWER: 1(COMMAND OK) 1(found and changed)/0(not found/second argument is missing)
  }
  else if(command=="kr")
  {
    //Camac reset:"kr controller_name"
    //TODO: check, does it possible to do this operation, or not? -- we assume, it can be done
    std::string name;
    ss>>name;//Get the controller name
    std::vector<Camac*>::const_iterator i=std::find_if(camac.begin(),camac.end(),compare_name<Camac>(name));//Looking for the name
    if(i!=camac.end()) ans<<"1 1 "<<((**i).Reset() ? "1" : "0");//Reset the requested controller
    else ans<<"1 0";
    //ANSWER: 1(COMMAND OK) 1(found and attepted to reset it)/0(not found) 1(reset OK)/0(error on reset)
  }
  else ans<<"0";//ANSWER: 0(COMMAND does not exist)
  //Returning with the answer
  return ans.str();
}

std::string MTESTControl::do_motor_operation(std::string command, std::stringstream &ss)
//This program interpret motor related commands, performs action and returns with an appropriate message. Input is the command, then ss contains the following parameters
{
  std::transform(command.begin(), command.end(), command.begin(),::tolower);//Transforming the command string to lowercase
  std::stringstream ans;//Answer
  if(command=="mlist")
  {
    //It should list the name of the available motors : "mlist"
    ans<<"1 "<<motor.size()<<" ";//Set accepted command number and number of motors
    for(int i=0; i<(int)motor.size(); i++)
    {
      ans<<(motor[i])->getName()<<" ";
      if((dynamic_cast<Positioner::HarwellMotor*> (motor[i]))!=0) ans<<"HWMOTOR ";
      else if((dynamic_cast<Positioner::CushionAPMotor*> (motor[i]))!=0) ans<<"CAPMOTOR ";
      else if((dynamic_cast<Positioner::AngleParallelMotor*> (motor[i]))!=0) ans<<"APMOTOR ";
      else if((dynamic_cast<Positioner::ParallelMotor*> (motor[i]))!=0) ans<<"PMOTOR ";
      else ans<<"ERROR ";
    }
    //ANSWER: 1(command OK) number_of_motors name1 TYPE1 name2 TYPE2...
  }
  else if(command=="mcn")
  {
    //It should change the name of the motor: "mcn old_name new_name"
    std::string actual_name;
    std::string name_to_change;
    ss>>actual_name;//Get the actual name
    ss>>name_to_change;//New name
    std::vector<Positioner::Motor*>::const_iterator i=std::find_if(motor.begin(),motor.end(),compare_name<Positioner::Motor>(actual_name));//Looking for the name
    if((i!=motor.end())&&(name_to_change.size()!=0)) (**i).setName(name_to_change);//Setting the requested name
    ans<<(((i!=motor.end())&&(name_to_change.size()!=0)) ? "1 1" : "1 0");
    //ANSWER: 1(COMMAND OK) 1(found and changed)/0(not found/second argument is missing)
  }
  else if ((command=="mpp")||(command=="mpa"))
  {
    //Get the actual/last checked position in general pulses: "mpp name" or in angles: in angles: "mpa name[HW/APM/CAPM]"
    std::string name;
    ss>>name;//get the name
    std::vector<Positioner::Motor*>::const_iterator i=std::find_if(motor.begin(),motor.end(),compare_name<Positioner::Motor>(name));//Looking for the name
    if(i!=motor.end())
    {
      //In pulses
      if(command=="mpp") ans<<"1 1 "<<(long) ((**i).getPosition());//ANSWER: 1(COMMAND OK) 1(motor found) last_position
      else
      {
	//In angles
	Positioner::AngleMotor* ap=dynamic_cast<Positioner::AngleMotor*> (*i);//Try to convert to angle motor type
	if(ap!=0) ans<<"1 1 "<<ap->getAPosition();//ANSWER: 1(COMMAND OK) 1(motor found and register angles) last_position_in_angles
	else ans<<"1 -1";//ANSWER: 1(COMMAND OK) -1(motor found, but type does not allow the operation)
      }
    }
    else ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found)
  }
  else if ((command=="mgllp")||(command=="mghlp")||(command=="mglla")||(command=="mghla"))
  {
    //Get the low/high level limit of the motor in pulses or in angles: "mg[l/h]l[p/a] name(for angle only [HW/APM/CAPM])"
    std::string name;
    ss>>name;//get the name
    std::vector<Positioner::Motor*>::const_iterator i=std::find_if(motor.begin(),motor.end(),compare_name<Positioner::Motor>(name));//Looking for the name
    if(i!=motor.end())
    {
      //Case of pulses
      if((command=="mgllp")||(command=="mghlp")) ans<<"1 1 "<<(long) ((**i).getLimit((command=="mghlp")));//Get the requested limit in general pulses//ANSWER: 1(COMMAND OK) 1(motor found) limit_value
      else
      {
	//In angles
	Positioner::AngleMotor* ap=dynamic_cast<Positioner::AngleMotor*> (*i);//Try to convert to angle motor type
	if(ap!=0) ans<<"1 1 "<<ap->getALimit((command=="mghla"));//ANSWER: 1(COMMAND OK) 1(motor found) limit_in angles
	else ans<<"1 -1";//ANSWER: 1(COMMAND OK) -1(motor found, but type does not allow the operation)
      }
    }
    else ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found)
  }
  else if (command=="mea")
  {
    //Evacuate air from compressed air system of cushion AP motor: "mea name[CAPM]"
    std::string name;
    ss>>name;//get the name
    //TODO: check the conditions which should allow this kind of operation (calling the function check only, that moving of motors are not in progress)
    std::vector<Positioner::Motor*>::const_iterator i=std::find_if(motor.begin(),motor.end(),compare_name<Positioner::Motor>(name));//Looking for the name
    if(i!=motor.end())
    {
      Positioner::CushionAPMotor* ap=dynamic_cast<Positioner::CushionAPMotor*> (*i);//Try to convert to cushion AP motor type
      if(ap!=0) ans<<"1 1 "<<(ap->EvacuateAir() ? "1" : "0");//ANSWER: 1(COMMAND OK) 1(motor found) 1(did)/0(error during the execution)
      else ans<<"1 -1";//ANSWER: 1(COMMAND OK) -1(motor found, but type does not allow the operation)
    }
    else ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found)
  }
  else if ((command=="msllp")||(command=="mshlp")||(command=="mslla")||(command=="mshla"))
  {
    //Set the low/high level limit of the motor in pulses or in angles: "mg[l/h]l[p/a] name(for angle only [HW/APM/CAPM]) pulse/angle"
    //TODO: it should be check, does it possible to do, or not
    std::string name;
    ss>>name;//get the name
    std::vector<Positioner::Motor*>::const_iterator i=std::find_if(motor.begin(),motor.end(),compare_name<Positioner::Motor>(name));//Looking for the name
    if(i!=motor.end())
    {
      //Case of pulses
      if((command=="msllp")||(command=="mshlp"))
      {
	long position=0;
	ss>>position;
	ans<<"1 1 "<<((**i).setLimit((uint32_t) position,command=="mshlp") ? "1" : "0");//ANSWER: 1(COMMAND OK) 1(motor found) limit_value 1(OK)/0(error)
      }
      else
      {
	//In angles
	Positioner::AngleMotor* ap=dynamic_cast<Positioner::AngleMotor*> (*i);//Try to convert to angle motor type
	if(ap!=0)
	{
	  double position=0.0;
	  ss>>position;
	  ans<<"1 1 "<<(ap->setALimit(position,(command=="mshla")) ? "1" : "0");//ANSWER: 1(COMMAND OK) 1(motor found) limit_value 1(OK)/0(error)
	}
	else ans<<"1 -1";//ANSWER: 1(COMMAND OK) -1(motor found, but type does not allow the operation)
      }
    }
    else ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found)
  }
  else if ((command=="mmp")||(command=="mma"))
  {
    //Start to move a motor to a given position in pulses or in angles: "mm[p/a] name(for angle only [HW/APM/CAPM]) pulse/angle"
    //TODO: it should be check, does it possible to do, or not
    std::string name;
    ss>>name;//get the name
    std::vector<Positioner::Motor*>::const_iterator i=std::find_if(motor.begin(),motor.end(),compare_name<Positioner::Motor>(name));//Looking for the name
    if(i!=motor.end())
    {
      //Case of pulses
      if(command=="mmp")
      {
	long position=0;
	ss>>position;
	//TODO: check relative positions OK
	ans<<"1 1 "<<((**i).moveTo((uint32_t) position) ? "1" : "0" );//ANSWER: 1(COMMAND OK) 1(motor found) limit_value 1(OK)/0(error)
      }
      else
      {
	//In angles
	Positioner::AngleMotor* ap=dynamic_cast<Positioner::AngleMotor*> (*i);//Try to convert to angle motor type
	if(ap!=0)
	{
	  double position=0.0;
	  ss>>position;
	  //TODO: check relative positions OK
	  ans<<"1 1 "<<(ap->moveATo(position) ? "1" : "0");//ANSWER: 1(COMMAND OK) 1(motor found) limit_value 1(OK)/0(error)
	}
	else ans<<"1 -1";//ANSWER: 1(COMMAND OK) -1(motor found, but type does not allow the operation)
      }
    }
    else ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found)
  }
  else if (command=="mias")
  {
    //Is all motors stopped?: "mea"
    if(!motor.empty()) ans<<"1 "<<(Positioner::Motor::isAllStopped() ? "1" : "0");//ANSWER: 1(COMMAND OK) 1(all stopped)/0(one motor moving)
    else ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found: the system does not contain any motors)
  }
  else if (command=="mgs")
  {
    //Motor get status: "mgs name[HWM]"
    std::string name;
    ss>>name;//get the name
    std::vector<Positioner::Motor*>::const_iterator i=std::find_if(motor.begin(),motor.end(),compare_name<Positioner::Motor>(name));//Looking for the name
    if(i!=motor.end())
    {
      Positioner::HarwellMotor* ap=dynamic_cast<Positioner::HarwellMotor*> (*i);//Try to convert to HWmotor type
      if(ap!=0) ans<<"1 1 0x"<<std::hex<<(int)(ap->getStatus())<<std::dec;//ANSWER: 1(COMMAND OK) 1(motor found) status_in_hexa)
      else ans<<"1 -1";//ANSWER: 1(COMMAND OK) -1(motor found, but type does not allow the operation)
    }
    else ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found)
  }
  else if (command=="msza")
  {
    //Motor set zero position in angles: "msza name[HWM/APM/CAPM] actual_angle_where_zero_should_be"
    std::string name;
    ss>>name;//get the name
    double angle;//the angle in the actual scale, where zero position is
    ss>>angle;
    //TODO: check the conditions which should allow this kind of operation (calling the function check only, that moving of motors are not in progress)
    std::vector<Positioner::Motor*>::const_iterator i=std::find_if(motor.begin(),motor.end(),compare_name<Positioner::Motor>(name));//Looking for the name
    if(i!=motor.end())
    {
      Positioner::AngleMotor* ap=dynamic_cast<Positioner::AngleMotor*> (*i);//Try to convert to Anglemotor type
      if(ap!=0)
      {
	ap->setZero(angle);//Set the zero angle
	ans<<"1 1";//ANSWER: 1(COMMAND OK) 1(motor found)
      }
      else ans<<"1 -1";//ANSWER: 1(COMMAND OK) -1(motor found, but type does not allow the operation)
    }
    else ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found)
  }
  else ans<<"0";//ANSWER: 0(COMMAND does not exist)
  //Returning with the answer
  return ans.str();  
}

std::string MTESTControl::do_counter_operation(std::string command, std::stringstream &ss)
//This method interpret counting related commands, performs action and returns with an appropriate message. Input is the command, then ss contains the following parameters
{
  std::transform(command.begin(), command.end(), command.begin(),::tolower);//Transforming the command string to lowercase
  std::stringstream ans;//Answer
  if(command=="clist")
  {
    //It should list the name of the availabl counters : "mlist"
    ans<<"1 "<<psd.size()+monitor.size()+hwcnt.size()+rtccnt.size()<<" ";//Set accepted command number and number of counters
    for(int i=0; i<(int)psd.size(); i++)
    {
        std::vector<std::string> names=psd[i]->getNames();
        ans<<names.size()<<" ";
        std::for_each(names.begin(), names.end(), [&ans](std::string n){ans<<n<<" ";});
        ans<<" PSDCOUNTER ";
    }
    for(int i=0; i<(int)monitor.size(); i++)
    {
        std::vector<std::string> names=monitor[i]->getNames();
        ans<<names.size()<<" ";
        std::for_each(names.begin(), names.end(), [&ans](std::string n){ans<<n<<" ";});
        ans<<" MONCOUNTER ";
    }
    for(int i=0; i<(int)hwcnt.size(); i++)
    {
        std::vector<std::string> names=hwcnt[i]->getNames();
        ans<<names.size()<<" ";
        std::for_each(names.begin(), names.end(), [&ans](std::string n){ans<<n<<" ";});
        ans<<" HWCOUNTER ";
    }
    for(int i=0; i<(int)rtccnt.size(); i++)
    {
        std::vector<std::string> names=rtccnt[i]->getNames();
        ans<<names.size()<<" ";
        std::for_each(names.begin(), names.end(), [&ans](std::string n){ans<<n<<" ";});
        ans<<" RTCCOUNTER ";
    }
    //ANSWER: 1(command OK) number_of_counters #names1 name1 TYPE1 #names2 name2 TYPE2...
  }
  else if(command=="ccn")
  {
    //It should change the name of the counter: "ccn old_name new_name"
    std::string actual_name;
    std::string name_to_change;
    ss>>actual_name;//Get the actual name
    ss>>name_to_change;//New name
    if(name_to_change.size()==0) ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found/second argument is missing)
    else
    {
      std::vector<Scaler::PSDCounter*>::const_iterator i=std::find_if(psd.begin(),psd.end(),compare_with_names<Scaler::PSDCounter>(actual_name));//Looking for the name among PSD counters
      if(i!=psd.end())
      {
          std::vector<std::string> name=(**i).getNames();
          std::replace(name.begin(), name.end(), actual_name, name_to_change);
          (**i).setNames(name); ans<<"1 1";
      }//ANSWER: 1(COMMAND OK) 1(found and changed)
      else
      {
        std::vector<Scaler::MonitorCounter*>::const_iterator j=std::find_if(monitor.begin(),monitor.end(),compare_with_names<Scaler::MonitorCounter>(actual_name));//Looking for the name among monitor counters
        if(j!=monitor.end())
        {
          std::vector<std::string> name=(**j).getNames();
          std::replace(name.begin(), name.end(), actual_name, name_to_change);
          (**j).setNames(name); ans<<"1 1";
        }//ANSWER: 1(COMMAND OK) 1(found and changed)
        else
        {
            std::vector<Scaler::HarwellCounter*>::const_iterator k=std::find_if(hwcnt.begin(),hwcnt.end(),compare_with_names<Scaler::HarwellCounter>(actual_name));//Looking for the name among harwell counters
            if(k!=hwcnt.end()) 
            {
                std::vector<std::string> name=(**k).getNames();
                std::replace(name.begin(), name.end(), actual_name, name_to_change);
                (**k).setNames(name); ans<<"1 1";
            }//ANSWER: 1(COMMAND OK) 1(found and changed)
            else
            {
                std::vector<Scaler::RTCCounter*>::const_iterator l=std::find_if(rtccnt.begin(),rtccnt.end(),compare_with_names<Scaler::RTCCounter>(actual_name));//Looking for the name among RTC counters
                if(l!=rtccnt.end())
                {
                    std::vector<std::string> name=(**l).getNames();
                    std::replace(name.begin(), name.end(), actual_name, name_to_change);
                    (**l).setNames(name); ans<<"1 1";
                }//ANSWER: 1(COMMAND OK) 1(found and changed)
                else ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found/second argument is missing)
            }
        }
      }
    }
  }
  else if(command=="cgp")
  {
    //Provides the start preset value(s): "cgp name"
    std::string name;
    ss>>name;//Get the actual name
    std::vector<Scaler::PSDCounter*>::const_iterator i=std::find_if(psd.begin(),psd.end(),compare_with_names<Scaler::PSDCounter>(name));//Looking for the name among PSD counters
    if(i!=psd.end())
    {
      double real, live;//Presets
      ans<<"1 "<<((**i).GetPresets(&real,&live) ? "1 " : "0 ");
      ans<<real<<" "<<live;//ANSWER: 1(COMMAND OK) 1(found and changed)/0(not found/error on read) real_preset live_preset
    }
    else
    {
      std::vector<Scaler::MonitorCounter*>::const_iterator j=std::find_if(monitor.begin(),monitor.end(),compare_with_names<Scaler::MonitorCounter>(name));//Looking for the name among monitor counters
      if(j!=monitor.end()) ans<<"1 1 "<<(**j).GetPreset();//ANSWER: 1(COMMAND OK) 1(found) preset
      else
      {
	std::vector<Scaler::HarwellCounter*>::const_iterator k=std::find_if(hwcnt.begin(),hwcnt.end(),compare_with_names<Scaler::HarwellCounter>(name));//Looking for the name among harwell counters
	if(k!=hwcnt.end()) ans<<"1 1 "<<(**k).GetPreset();//ANSWER: 1(COMMAND OK) 1(found) preset
	else
	{
	  std::vector<Scaler::RTCCounter*>::const_iterator l=std::find_if(rtccnt.begin(),rtccnt.end(),compare_with_names<Scaler::RTCCounter>(name));//Looking for the name among RTC counters
	  if(l!=rtccnt.end()) ans<<"1 1 "<<(**l).GetPreset();//ANSWER: 1(COMMAND OK) 1(found) preset
	  else ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found)
	}
      }
    }
  }
  else if(command=="csp")
  {
    //Set presets (s): "csp counter_name [real_preset live_presets (PSD multiple of 0.05, 0.0 means: no pereset)][preset (others)]"
    std::string name;
    ss>>name;//Get the actual name
    std::vector<Scaler::PSDCounter*>::const_iterator i=std::find_if(psd.begin(),psd.end(),compare_with_names<Scaler::PSDCounter>(name));//Looking for the name among PSD counters
    if(i!=psd.end())
    {
      double real=0.0,live=0.0;//Presets
      ss>>real>>live;
      ans<<"1 "<<((**i).ChangePresets(real,live) ? "1 " : "0 ");//ANSWER: 1(COMMAND OK) 1(found and changed)/0(not found/error on read)
    }
    else
    {
      long preset=0;//Preset
      ss>>preset;
      std::vector<Scaler::MonitorCounter*>::const_iterator j=std::find_if(monitor.begin(),monitor.end(),compare_with_names<Scaler::MonitorCounter>(name));//Looking for the name among monitor counters
      if(j!=monitor.end()) ans<<"1 "<<(**j).SetPreset((Scaler::MON_CNT)preset);//ANSWER: 1(COMMAND OK) 1(found and changed)/0(not found/error)
      else
      {
	std::vector<Scaler::HarwellCounter*>::const_iterator k=std::find_if(hwcnt.begin(),hwcnt.end(),compare_with_names<Scaler::HarwellCounter>(name));//Looking for the name among harwell counters
	if(k!=hwcnt.end()) ans<<"1 "<<(**k).SetPreset((Scaler::HW_CNT)preset);//ANSWER: 1(COMMAND OK) 1(found and changed)/0(not found/error)
	else
	{
	  std::vector<Scaler::RTCCounter*>::const_iterator l=std::find_if(rtccnt.begin(),rtccnt.end(),compare_with_names<Scaler::RTCCounter>(name));//Looking for the name among RTC counters
	  if(l!=rtccnt.end()) ans<<"1 "<<(**l).SetPreset((Scaler::RTC_CNT)preset);//ANSWER: 1(COMMAND OK) 1(found and changed)/0(not found/error)
	  else ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found)
	}
      }
    }
  }
  else if(command=="ccsp")
  {
    //Change setup parameters:
    //"ccsp name[PSD] [O(ff gate)/C(oincidence gate)/A(nti coincidence gate)] channel_no low_discriminator hi_discriminator"
    //"ccsp name[Monitor] [T(imer)/C(counter)] low_intensity_level_cps consecutive_times"
    //"ccsp name[HW] \"codeword\""
    //"ccsp name[RTC] divisor"
    std::string name;//Name of the counter
    ss>>name;
    std::vector<Scaler::PSDCounter*>::const_iterator i=std::find_if(psd.begin(),psd.end(),compare_with_names<Scaler::PSDCounter>(name));//Looking for the name among PSD counters
    if(i!=psd.end())
    {
      char gate;//Gate mode
      GATE_MODE gm;//Gate mode to be transformed
      int channel_no, ch_lims[2];//Numberof channels, limits of channels
      ss>>gate>>channel_no>>ch_lims[0]>>ch_lims[1];//Get the datasets
      switch(gate)
      {
	case 'O': gm=GATE_OFF; break;
	case 'C': gm=GATE_COINCIDENCE; break;
	case 'A': gm=GATE_ANTICOINCIDENCE; break;
	default: gm=GATE_ERROR;
      }
      if (gm==GATE_ERROR) ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found/error on read)
      else ans<<"1 "<<((**i).ChangeChannel(channel_no,ch_lims[0],ch_lims[1]) && (**i).ChangeGate(gm));//ANSWER: 1(COMMAND OK) 1(found and changed)/0(not found/error on read)
    }
    else
    {
      std::vector<Scaler::MonitorCounter*>::const_iterator j=std::find_if(monitor.begin(),monitor.end(),compare_with_names<Scaler::MonitorCounter>(name));//Looking for the name among monitor counters
      if(j!=monitor.end())
      {
	char mode;//Mode: T/C
	int intensity;//Low level of intensity to be taken as low
	int times;//Consecutively
	ss>>mode>>intensity>>times;//Get the datasets
	ans<<"1 "<<((**j).ChangeOpMode(mode=='C') && (**j).SetIntensityParameters(true,(uint16_t) intensity, (uint8_t) times, true));//ANSWER: 1(COMMAND OK) 1(found and changed)/0(not found/error)
      }
      else
      {
	std::vector<Scaler::HarwellCounter*>::const_iterator k=std::find_if(hwcnt.begin(),hwcnt.end(),compare_with_names<Scaler::HarwellCounter>(name));//Looking for the name among harwell counters
	if(k!=hwcnt.end())
	{
	  std::string codestr;//Codeword for HW and divider for RTC
	  ss>>codestr;//Get the code
	  ans<<"1 "<<(**k).SetControlRegister(Scaler::HarwellCounter::str2code(codestr));//ANSWER: 1(COMMAND OK) 1(found and changed)/0(not found/error)
	}
	else
	{
	  std::vector<Scaler::RTCCounter*>::const_iterator l=std::find_if(rtccnt.begin(),rtccnt.end(),compare_with_names<Scaler::RTCCounter>(name));//Looking for the name among RTC counters
	  if(l!=rtccnt.end())
	  {
	    int fdiv;//Codeword for HW and divider for RTC
	    ss>>fdiv;//Get the code
	    (**l).SetFrequencyDivisor(Scaler::RTCCounter::num2div(fdiv));//Set frequency division
	    ans<<"1 1";//ANSWER: 1(COMMAND OK) 1(found and changed)
	  }
	  else ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found)
	}
      }
    }
  }
  else if(command=="cgsp")
  {
    //Get setup parameters:
    //"cgsp name"
    std::string name;//Name of the counter
    ss>>name;
    std::vector<Scaler::PSDCounter*>::const_iterator i=std::find_if(psd.begin(),psd.end(),compare_with_names<Scaler::PSDCounter>(name));//Looking for the name among PSD counters
    if(i!=psd.end())
    {
      GATE_MODE gm=GATE_ERROR;//Gate mode to be transformed
      int channel_no, ch_lims[2];//Numberof channels, limits of channels
      ans<<"1 "<<(((**i).GetChannel(&channel_no,ch_lims,ch_lims+1) && (((gm=(**i).GetGate()))!=GATE_ERROR)) ? "1 " : "0 ");//ANSWER: 1(COMMAND OK) 1(found and changed)/0(not found/error on read) [O(ff gate)/C(oincidence gate)/A(nti coincidence gate)/!(error)] channel_no low_discriminator hi_discriminator
 
      char gate=' ';//Gate mode
      switch(gm)
      {
	case GATE_OFF: gate='O'; break;
	case GATE_COINCIDENCE: gate='C'; break;
	case GATE_ANTICOINCIDENCE: gate='A'; break;
	default: gate='!';
      }
      ans<<gate<<" "<<channel_no<<" "<<ch_lims[0]<<" "<<ch_lims[1];//ANSWER continues
    }
    else
    {
      std::vector<Scaler::MonitorCounter*>::const_iterator j=std::find_if(monitor.begin(),monitor.end(),compare_with_names<Scaler::MonitorCounter>(name));//Looking for the name among monitor counters
      if(j!=monitor.end())
      {
	char mode=((**j).GetOpMode() ? 'C' : 'T');//Mode: T/C
	uint16_t intensity;//Low level of intensity to be taken as low
	uint8_t times;//Consecutively
	(**j).GetIntensityParameters(&intensity, &times);//Get intensity parameters
	ans<<"1 1 "<<mode<<" "<<(int)intensity<<" "<<(int)times;//ANSWER: 1(COMMAND OK) 1(found and changed)/0(not found/error) [T(imer)/C(counter)] low_intensity_level_cps consecutive_times
      }
      else
      {
	std::vector<Scaler::HarwellCounter*>::const_iterator k=std::find_if(hwcnt.begin(),hwcnt.end(),compare_with_names<Scaler::HarwellCounter>(name));//Looking for the name among harwell counters
	if(k!=hwcnt.end()) ans<<"1 1 \""<<Scaler::HarwellCounter::code2str((**k).GetControlRegister())<<"\"";//ANSWER: 1(COMMAND OK) 1(found and changed) "code"
	else
	{
	  std::vector<Scaler::RTCCounter*>::const_iterator l=std::find_if(rtccnt.begin(),rtccnt.end(),compare_with_names<Scaler::RTCCounter>(name));//Looking for the name among RTC counters
	  if(l!=rtccnt.end()) ans<<"1 1 "<<Scaler::RTCCounter::div2num((**l).GetFrequencyDivisor());//ANSWER: 1(COMMAND OK) 1(found and changed)
	  else ans<<"1 0";//ANSWER: 1(COMMAND OK) 0(not found)
	}
      }
    }
  }
  else ans<<"0";//ANSWER: 0(COMMAND does not exist)
  //Returning with the answer
  return ans.str();
}

std::string MTESTControl::getNextRelevantLine(std::ifstream &is)
//This function returns with the next relevant line
{
  std::string line;
  do std::getline(is,line); while ((line.find("#")==0)&& !is.eof());
  return line;
}
