#include <algorithm>
#include <fstream>
#include <map>
#include <sstream>
#include <string>
#include <vector>

//Camac controller
#include "Camac.h"
#include "ParallelCamac.h"
#include "SerialCamac.h"

//IO registers
#include "IOOperation.h"

//Motors
#include "Motor.h"

//Scalers
#include "Scaler.h"

#ifndef MTEST_CONTROL_H
#define MTEST_CONTROL_H

class MTESTControl
//This class deals with the control of the whole system
{
  public:
    //methods
    MTESTControl(){};//Constructor
    ~MTESTControl(){};//Destructor
    std::string CommandIP(std::string commandline);//This method does the interpreatation of command, transmits to hardware and returns with the answer
    bool readCfgFile(std::ifstream &is);//This method read the configuration file and try to initialize data structure
    bool writeCfgFile(std::ofstream &os);//This method writes the configuration file, according to the present setup
  private:
    std::string do_camac_operation(std::string command, std::stringstream &ss);//This method interpret camac related commands, performs action and returns with an appropriate message. Input is the command, then ss contains the following parameters
    std::string do_motor_operation(std::string command, std::stringstream &ss);//This method interpret motor related commands, performs action and returns with an appropriate message. Input is the command, then ss contains the following parameters
    std::string do_counter_operation(std::string command, std::stringstream &ss);//This method interpret counting related commands, performs action and returns with an appropriate message. Input is the command, then ss contains the following parameters
    std::string getNextRelevantLine(std::ifstream &is);//This function returns with the next relevant line
    //members
    std::vector<Camac*> camac;//Array to handle camac connections
    std::vector<IOOperation::InterruptRegister*> iregister;//Array of interrupt registers
    std::vector<IOOperation::OutputRegister*> oregister;//Array of output registers
    std::vector<Positioner::StepMotorController*> smc;//Array of step motor controller
    std::vector<Positioner::Motor*> motor;//Array of motors
    std::vector<Scaler::PSDCounter*> psd;//Array of trump counters
    std::vector<Scaler::MonitorCounter*> monitor;//Array of monitor counters
    std::vector<Scaler::HarwellCounter*> hwcnt;//Array of harwell counters
    std::vector<Scaler::RTCCounter*> rtccnt;//Array of SEN RTC counters
};

template <class T> struct compare_name
{
  std::string name_looking_for;//This is the name, which we are looking for
  compare_name(std::string &name_look_for):name_looking_for(name_look_for){};//Constructor to initialize name_looking_for
  bool operator() (T* const obj) const {return obj->getName()==name_looking_for;}//Does the compare operation
};

template <class T> struct compare_with_names
{
  std::string name_looking_for;//This is the name, which we are looking for
  compare_with_names(std::string &name_look_for):name_looking_for(name_look_for){};//Constructor to initialize name_looking_for
  bool operator() (T* const obj) const
  {
    std::vector<std::string> name=obj->getNames();
    return std::any_of(name.begin(),name.end(),[&](std::string n) {return n==name_looking_for;});
  }//Does the compare operation
};


#endif
