#include "MTESTServer.h"

bool MTESTServer::Run(MTESTControl &mtc)
//This method runs the socket, which communicates with (returns with success or fail)
{
  int descriptor=initsocket(path,pending_no);//Initialize: in the case of error, it returns with 0<0
  if(descriptor>=0) 
  {
    accept_and_close_loop(descriptor,mtc);//Accepting incoming connections
    closesocket(descriptor);//Closing the server
    return true;
  }
  else return false;
}

int MTESTServer::initsocket(const std::string &s_path, int pend_list_size)
//This method initializes a socket, whose path is s_path and the size of pending connections is pend_list_size. It returns with the file descriptor or error codes
{
  struct sockaddr_un address;//Defines unix domain socket address structure
  int sock;//File descriptor of the socket
  
  //Create socket
  if((sock=socket(PF_LOCAL, SOCK_STREAM,0))<0)
  {
    perror("Initialisattion of socket");
    return -1;
  }
  
  unlink(s_path.c_str());//Remove the existing socket
  
  address.sun_family=AF_UNIX;//Unix domain socket
  strcpy(address.sun_path,s_path.c_str());//Define path
  size_t addr_length=sizeof(address.sun_family)+strlen(address.sun_path);//Determine size
  
  //Attach socket to the address
  if(bind(sock,(struct sockaddr *) &address, (socklen_t) addr_length))
  {
    perror("Bind has not been succeeded!");
    return -2;
  }
  
  //Setting the access
  fchmod(sock,0777);//Modifying the access
  
  //Switch on waiting for connection
  if(listen(sock,pend_list_size)) //Only pending list size number of connections are accepted
  {
    perror("Trouble at listen!");
    return -3;
  }
  
  close_connection=false;
  return sock;//Returns with the descriptor of the socket
}

bool MTESTServer::accept_and_close_loop(int descriptor_socket, MTESTControl &mtc)
//This method accepts the incoming requests, communicates and closes the socket inputs are the socket descriptor and the MTESTControl object. Returns with success or error
{
  //Accepting connections
  int client;//Client descriptor
  struct sockaddr_un address;//Defines the client address structure
  size_t addr_length;//Size of the address
  bool retval=true;
  while(!close_connection && retval && ((client=accept(descriptor_socket,(struct sockaddr *) &address,(socklen_t *) &addr_length))>=0))//Waiting for incoming connections
  {
    retval=handle_client(client,mtc);//Do read and write from/to client
    close(client);//Closes client
  }
  return retval;
}

bool MTESTServer::handle_client(const int client, MTESTControl &mtc)
//This member handles the accepted client connection: transfers the received message to control, then transmit the answer. Returns with success or fail
{
  //Transfering the incoming command to the commandline string
  std::string commandline;
  int datacounts=0;//Number of counts for dataset
  char buffer[2];//Buffer for command writing
  while((commandline.find("\n")==std::string::npos)&&((datacounts=read( client, buffer, sizeof(buffer)))>0)) commandline.append(buffer,datacounts); //Getting the datasets until read finishes as \n appears int the message
  
  if(datacounts<0)
  {
    perror("Trouble at reading from socket!");
    return false;
  }
  
  std::string answer;//Answer
  if(commandline.find("CLOSE")==1) close_connection=true;
  else answer=mtc.CommandIP(commandline);//Processing command, get the answer
  //writing the answer
  answer+="\n";
  write(client,answer.c_str(), answer.size());
  return true;
}