#include "Serial.h"

#ifndef MODBUS_H
#define MODBUS_H

//debug: DEBUG_MODBUS

class Modbus
{
  public:
    Modbus(const std::string nPort="haga_controller", const int nBaud=9600, const int nDataBits=8, const char nParity='N', const int nStopbits=1, double timeout=0.5)
    {
      //Opening Serial communication line
      pS=new Serial(nPort,nBaud,nDataBits,nParity,nStopbits);
      //Checking opened status
      if(pS->isOpened())
      {
#ifdef DEBUG_MODBUS
	std::cout<<"(Modbus::Modbus) Successful opening of the communication line\n";
#endif
	//setting the timeout
	pS->setTimeout(timeout);
      }
      else
      {
	std::cerr<<"ERROR(Modbus::Modbus): Unsuccessful opening of the serial communication line\n";
      }
    };
    ~Modbus() {delete pS;};
    bool getValue(const uint8_t slavenumber, const uint16_t start_register, const uint16_t numberofregisters, uint16_t *result, const bool readonly=true);
    //This method asks the content of the slave's start_register~start_register+numberofregisters field and sets its content into result buffer.
    //It asks read-only registers by default. Returns with .....
    bool setValue(const uint8_t slavenumber, const uint16_t registernumber, const uint16_t value);//This method writes the slave's registernumber. Returns with succ (true) or not
  private:
    //static members
    static const uint8_t CRCHitable[256];//Stores CRC high bit table necessary for check
    static const uint8_t CRCLotable[256];//Stores CRC low bit table necessary for check
    static const int WAIT_TIME=2000;//2ms
    //members
    Serial *pS;//Serial descriptor
    //methods
    bool crccheck(uint8_t *msgbuffer, const int datalength, bool sendmode=true);
    //This method if send is true, then sets the crc bytes into the msgbuffer after datalength-2 and returns with true in any cases.
    //If send is false, then check crc hi (msgbuffer[datalength-2]) and low (msgbuffer[datalength-1]) values and returns with the result
    bool communicate(uint8_t *sendbuffer, const int slen, uint8_t *replybuffer, const int rlen);
};
#endif
