//TESTED:15.08.2013 by Laszlo Temleitner

#include <algorithm>
#include <cstdint>
#include <iostream>
#include <iomanip>
#include <sstream>
#include "Serial.h"
#include "Thermometer.h"

#ifndef OHM_H
#define OHM_H

class OHM : public Thermometer
//This class describes the Universal Pyrometer Protocol
{
  public:
    //methods
    OHM(const std::string nPort="thermometer_ohm", const double timeout=OHM::TIMEOUT);//Constructor: serial port name, Celsius(t)/Fahrenheit(f), baud rate, timeout
    ~OHM(){delete communicator;};//Destructor
    bool ResetCommunication(const std::string nPort="thermometer_ohm", const double timeout=0.1);//Reset the communication parameters: serial port, transmission rate, timeout
    //ACTUAL TEMPERATURE
    bool GetTemperature(bool &chAOK, bool &chBOK, double &chA, double &chB);//Returns with the successful communication(t). The first 2 variables indicates validity of the datasets, which are stored in the position of the last 2 variables
    std::string GetTemperatureUnit(bool chnA=true);//Returns with the unit of temperature of the selected (A:t, B:f) channel
    void getTempHW(double *result);//Returns with the temperatures: , should be implemented in child class
    //LOGGING
    bool DataLog(bool start=false);//Start(t)/stop(f) data logging
    bool ResetLog();//Reset logged data
    std::string GetLoggedData(){return getbulky('L','D');};//Get the logged datasets
    int GetFreeLogPages();//Returns with the number of free log pages
    bool SetTimeInterval(int interval);//Set print/log interval in s
    int	 GetTimeInterval();//Returns with the print/log interval in s
    //PRINTING
    void ImmediatePrinting(){std::cout<<"This function: K1 does not implemented yet"<<std::endl; return;};//Immediate printout mode
    bool StopPrint() { char command[2]={'K','0'}; printing=(!setvalue(command,0, NULL)); return !printing; };//Stops the immediate print
    //SETTINGS
    bool	SetUserCode(std::string user);//Sets the user code, returns with success(t) or fail(f)
    std::string GetUserCode(){return getabout('G','B',8);};//Returns with the actually set user code
    bool SetModes(const bool sleep_mode_log_on=false, const bool auto_power_off=false, const bool relative_temperature=false);//Set sleep mode logging, auto power off and relative temperature measurement on (t) or off(f)
    bool LockULKeyboard(bool lock=false);//Locks (t) or unlock (f) the keyboard
    //ABOUT
    void About( std::string &model, std::string &model_desc, std::string &inst_sn, std::string &firm_ver, std::string &firm_date, std::string &cal_date, std::string &probe, std::string &probe_sn, std::string &probe_cal, double &batterylevel);//This function returns with general information about instrument: model number, description, instrument serial number, firmware version and date, calibration date, probe set, probe serial number and calibration date, battery level in volts - locked
    std::string GetAbout(){return getbulky('G','C');};//Returns with head information - locked
  private:
    //methods
    //Set-Unset
    bool SleepModeLogOnOff(bool on=false);//Set on sleep mode log(t) or off
    bool AutoPowerOff(bool enable=false);//Enable(t)/disable(f) auto poweroff function
    bool RelativeTemp(bool enable=false);//Enable(t)/disable(f) relative temperature mode
    //About
    std::string GetModel() {return getabout('G','0',6);};//Returns the model number
    std::string GetModelDesc(){return getabout('G','1',2);};//Returns with the model description
    std::string GetInstSN(){return getabout('G','2',3);};//Returns with the serial number of the instrument
    std::string GetFirmVer(){return getabout('G','3',10);};//Returns with firmware version
    std::string GetFirmDate(){return getabout('G','4',10);};//Returns with firmware date
    std::string GetCalDate(){return getabout('G','5',4);};//Returns with calibration date and time
    std::string GetProbe(){return getabout('G','6',6);};//Returns with the type of the probe set
    std::string GetProbeSN(){return getabout('G','7',9);};//Returns with the serial number of the probe
    std::string GetPCalDate(){return getabout('G','8',11);};//Returns with the date of calibration of the probe
    double GetBatteryLevel();//Returns with the battery level in volts
    //medium levels
    std::string getabout(char clett1, char clett2, int cut);//Skeleton to ask about instrument settings - unlocked
    std::string getbulky(char clett1, char clett2);//Skeleton to ask bulky data locked
    //med-low level
    bool setvalue(char *command, const int length, uint8_t *message); //Send the command [2 letters] with the length long message
    int getvalue(char *command, const int cletters, const int length, uint8_t *message);
    //Requesting a given value by command ["cletters" letters]. The available buffer for receiving messages is "message" which is "length" long. If command letters are 0, it will perform read only task!
    //low level
    bool communicate(uint8_t *sendbuffer, const int slen, uint8_t *replybuffer, int *rlen, const bool onlyreceive=false);//Communicate with serial interface: it contains the data to send, its length, the reply message and its expected length. If onlyreply true, then it read message only.
    std::string tostr(uint8_t* buffer, const int length);//Converting the digits containing the buffer into string
    //static members
    static const uint8_t CR=0x0d;//ASCII Carriage return
    static const int TEMP_PARAMETERS=2;//Number of temperature parameters
    static const double TIMEOUT;//Default timeout time in s
    static const double WAITSUCCESS;//Default time to wait between successfull receive and the next command in s
    //members
    Serial *communicator;//Serial communication object
    bool printing;//The actual status of the instrument
};

#endif
