#include "ParallelCamac.h"

bool ParallelCamac::Reset()
//Reset the present communication port and allocate new ones
{
  //Close, if it allocated before - we are assuming, that it need to change  
  if (allocated) 
  {
    if((getdata(CEASEINHIB_A, GENERATE_F, INITCEASE_N, 0, true) &STATUS_X)==STATUS_X) allocated=!openclose(portNo, false);//Generate Clear signal and close port
  }
  //Opening the new communication line
  if (!allocated) 
  {
    //Open new port
    if(openclose(portNo, true))
    {
      allocated=true;//Set signal to allocated
      if (connect_crate()) return ((getdata(INIT_A, GENERATE_F, INITCEASE_N, 0, true) &STATUS_X)==STATUS_X) ? true : false;//Set the crate address, then send initialiZe message
      else return false;
    }
    else return false;
  }
  else return false;
}

//Camac communication routines
bool ParallelCamac::connect_crate()
//Tries to connect with the crate controller card
{
  write(P_CRATE_ADDR,getCrate());//Send crate number
  while((read(P_CRATE_CNTRL_ADDR)&P_ST_BUSY)==P_ST_BUSY);//Reading status information: the device should be in "idle" or "not busy" state (P_ST_BUSY=0) instead of "busy" or "not used" to finish this part
  return true;//Regular end
}

uint32_t ParallelCamac::getdata(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, uint8_t *response, const bool only_status)
//Get dataset from the station using function code and subaddress. If the only_status bit is false, then it returns with the dataset and the response code. If it were true, it returns only with the status byte information, regardless the returning value
{
  //Set station, subaddress, function code and waiting for the response and/or status information
  write(P_STATION_NUM,station);
  write(P_SUB_ADDR,subaddress);
  write(P_FUNC,functioncode);
  uint8_t resp=P_ST_BUSY;//Initializing the response
  while((resp&(P_ST_BUSY | P_ST_DONE))==P_ST_BUSY) { resp=read(P_STATUS);}//Get the status byte information (it should not be BUSY)
#ifdef DEBUG
  std::cout<<"ParallelCamac::getdata: "<<(int)resp<<std::endl;
#endif
  if(response) *response=transform2u(resp);//Set answer, if response is requested by pointer 
  if(only_status) return (uint32_t) transform2u(resp);//Returns with the status byte
  else return (read(P_READ_LOW) | (read(P_READ_MID)<<8) | (read(P_READ_UPP)<<16));//Returns with the requested data
}

uint8_t ParallelCamac::setdata(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, const uint32_t data)
//Set dataset of the station's using function code and subaddress. It returns with proper response. Otherwise: with error codes 00
{
  //Partitioning the dataset to write
  write(P_WRITE_LOW,(uint8_t)(data&0xff));
  write(P_WRITE_MID,(uint8_t)((data>>8)&0xff));
  write(P_WRITE_UPP,(uint8_t)((data>>16)&0xff));
  //writing the remaining parts
  return getdata(subaddress, functioncode, station, 0, true)&0xff;
}

uint8_t ParallelCamac::transform2u(const uint8_t inp)
//Transforms status byte to uniform and device specific format
{
    uint8_t ans=0x0;//Initialize answer
    if(inp&P_ST_X) ans+=Camac::STATUS_X;//Generals
    if(inp&P_ST_Q) ans+=Camac::STATUS_Q;
    if(inp&P_ST_DONE) ans+=this->STATUS_P_DONE;//Device specific
    if(inp&P_ST_BUSY) ans+=this->STATUS_P_BUSY;
    if(inp&P_ST_I) ans+=this->STATUS_P_INHIBIT;
    return ans;
}

//Low level routines
bool ParallelCamac::openclose(uint32_t port_number, bool bopen)
//Open/close the present communication port
{
  //We are intend to use port_number between 0 to max_offset
  seteuid(0);//Get root access
  int ecode=ioperm((unsigned long)port_number, (unsigned long)MAX_OFFSET, (bopen ? 1 : 0));
  seteuid(getuid());//Revert access to single user
  if (ecode!=0)
  {
    std::cerr<<"ERROR: (ParalelCCard::<<"<<(bopen ? "open)" : "close)")<<std::endl;
    switch(errno)
    {
      case EINVAL: std::cerr<<"Invalid values of port number ("<<std::hex<<port_number<<std::dec<<") or maximal offset ("<<MAX_OFFSET<<")"<<std::endl; break;
      case EIO: std::cerr<<"This call is not supported!"<<std::endl; break;
      case ENOMEM: std::cerr<<"Out of memory!"<<std::endl; break;
      case EPERM: std::cerr<<"Your privilege is insufficient!"<<std::endl; break;
      default: std::cerr<<"Other Error"<<std::endl;
    }
    return false;
  }
  return true;
}

uint8_t ParallelCamac::read(uint8_t offset)
//This function performs reading of one byte, which is shifted by offset bytes from the starting port. Before reading it waits wait_in_mus microseconds
{
  return (uint8_t) inb((unsigned)(portNo+offset));
}

void ParallelCamac::write(uint8_t offset, uint8_t data)
//This function performs writing one byte, which is shifted by offset bytes from the starting port.
{
  outb((unsigned char) data,(unsigned)(portNo+offset));
  return;
}
