#include <cstdint>
#include <iostream>
#include <sys/io.h>
#include <unistd.h>
#include "Camac.h"

#ifndef PARALLEL_CAMAC_H
#define PARALLEL_CAMAC_H

class ParallelCamac: public Camac
//This class describes an ISA Camac Parallel Controller card 
{
  public:
    //Constructor-destructor
    ParallelCamac(uint32_t port_number_start=0x230, uint8_t crateNo=7, const std::string nm=""):Camac(crateNo,nm){allocated=false; portNo=port_number_start; Reset();};//Constructor: crate number, the start of the memory range, name of the controller
    ~ParallelCamac(){if(allocated){while((getdata(CEASEINHIB_A, GENERATE_F, INITCEASE_N, 0, true)&STATUS_X)!=STATUS_X) openclose(portNo,false);}};//Closes the port
    bool WriteSTNumRegister(uint8_t crate_no){return setData(INIT_A, WRITE_F, CONTROL_N, (uint32_t) crate_no, STATUS_X | STATUS_Q);};//This method writes the station number register, allowing to give the next command for two stations
    uint32_t getStartPortNo(){return portNo;};//Returns with the start port number
    //implementation of virtuals
    bool Reset();//Reset the present communication port and allocate new ones
    bool setCrate(const uint8_t cn){if (getCrate()!=cn) {Camac::setCrate(cn); return connect_crate();} else return true;};//Sets the crate address (redefine it)
    bool SetResetInhibit(bool reset){if(allocated) return Control(CEASEINHIB_A, reset ? DISABLE_F : GENERATE_F, CONTROL_N,STATUS_X); else return false;};//Set(f)/clear(t) dataway inhibit. Returns with communication success or error
    bool GetInhibit(bool test){ if (test) return Control(CEASEINHIB_A, TEST_F, CONTROL_N, Camac::STATUS_X| Camac::STATUS_Q); else return ((read(P_STATUS)&P_ST_I)==P_ST_I);};//Get the dataway inhibit value (t: set, f: unset). If test is true, it returns the setting of the controller (it might be other sources of dataway inhibit: test is tests only the controller, but general the dataway directly).
    //static members - converted status registers
    static const uint8_t STATUS_P_DONE=0x4;//Done bit: 1: completionof the camac op, 0: beginning of camac op
    static const uint8_t STATUS_P_BUSY=0x8;//Busy bit: 1: beginning of the camac op, 0: completion of operation
    static const uint8_t STATUS_P_INHIBIT=0x10;//Inhibit signal
  private:
    //static members
     static const uint8_t INITCEASE_N=28;//InitialiZe/Clear station code
     static const uint8_t CONTROL_N=30;//Control station code
     static const uint8_t INIT_A=8;//Initialize and set station number sub address
     static const uint8_t CEASEINHIB_A=9;//Clear and inhibit sub address
     static const uint8_t WRITE_F=16;//Write function code
     static const uint8_t DISABLE_F=24;//Remove Dataway Inhibit, disable demand function code
     static const uint8_t GENERATE_F=26;//Generate Initialize/Clear, Dataway Inhibit, enable demand function code
     static const uint8_t TEST_F=27;//Test function code
     //registers-offsts
     static const uint8_t MAX_OFFSET=12;//Maximal allowed offset of the data
     static const uint8_t P_CRATE_ADDR=0;//Crate address register
     static const uint8_t P_CRATE_CNTRL_ADDR=1;//Crate controller status register for crate address
     static const uint8_t P_WRITE_LOW=2;//Write dtat buffer register - lower byte
     static const uint8_t P_WRITE_MID=3;//Write dtat buffer register - middle byte
     static const uint8_t P_WRITE_UPP=4;//Write dtat buffer register - upper byte
     static const uint8_t P_STATION_NUM=5;//Station number register
     static const uint8_t P_SUB_ADDR=6;//Sub address register
     static const uint8_t P_FUNC=7;//Function register
     static const uint8_t P_STATUS=8;//Crate controller status register for read/write
     static const uint8_t P_READ_LOW=9;//Write dtat buffer register - lower byte
     static const uint8_t P_READ_MID=10;//Write dtat buffer register - middle byte
     static const uint8_t P_READ_UPP=11;//Write dtat buffer register - upper byte
     //registers-status register bits
     static const uint8_t P_ST_DONE=0x2;//Done bit: 1: completionof the camac op, 0: beginning of camac op
     static const uint8_t P_ST_BUSY=0x4;//Busy bit: 1: beginning of the camac op, 0: completion of operation
     static const uint8_t P_ST_I=0x8;//Inhibit signal
     static const uint8_t P_ST_X=0x10;//X bistable
     static const uint8_t P_ST_Q=0x80;//Q bistable
    //members
     uint32_t portNo;//Start of the port number
     bool allocated;//Allocated port range
    //The parent's virtual methods, should be implemented
     uint32_t getdata(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, uint8_t *response, const bool only_status=false);//Get dataset from the station using function code and subaddress. If the only_status bit is false, then it returns with the dataset and the response code. If it were true, it returns only with the status byte information, regardless the returning value
     uint8_t setdata(const uint8_t subaddress, const uint8_t functioncode, const uint8_t station, const uint32_t data);//Set dataset of the station's using function code and subaddress. It returns with proper response. Otherwise: with error codes 00
    //medium level routines
     bool connect_crate();//Tries to connect with the crate controller card
     uint8_t transform2u(const uint8_t inp);//Transforms status byte to uniform format and append the device specific ones, as well
    //low-level open/close/read/write methods
     bool openclose(uint32_t port_number=0x230, bool bopen=false);//Open/close the present communication port
     uint8_t read(uint8_t offset);//This function performs reading of one byte, which is shifted by offset bytes from the starting port.
     void write(uint8_t offset, uint8_t data);//This function performs writing one byte, which is shifted by offset bytes from the starting port.
};

#endif
