#include <cstdint>
#include <fcntl.h>
#include <iostream>
#include <queue>
#include <strings.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <termios.h>
#include <unistd.h>

#ifndef SERIAL_H
#define SERIAL_H

//debug information: DEBUG_SERIAL


class Serial
{
  public:
    Serial(const std::string nPort="ttyUSB0", const int nBaud=9600, const int nDataBits=8, const char nParity='N', const int nStopbits=1, const bool rawcanonical=true, const bool softhand=false, const bool hardhand=false)
    {
      fd=0;
      timeout=NULL;
      Open(nPort, nBaud, nDataBits, nParity, nStopbits, rawcanonical, softhand, hardhand);
    };//Constructor calls the open method
    ~Serial(){Close();}//Destructor calls the Close method
    bool Open(const std::string nPort, const int nBaud, const int nDatabits, const char nParity, const int nStopbits, const bool rawcanonical=true, const bool softhand=false, const bool hardhand=false);
    //This function opens a serial communication line: port name (device), baud rate, number of databits, parity: 'N'-none,'E'-even,'O'-odd, number of stopbits, raw (t) or canonical input(f), software handshake (Xon/Xoff), hardwarehandshake --- if everything is OK, retuns with true
    bool Close();//If everything is OK, it returns with true
    int Send(const uint8_t *buffer, const int length);//This method send the buffer content to the serial line. Buffer (or the length of the message) is consits of length number of bytes. Returns with the number of sent bytes
    int Receive(uint8_t *buffer, const int length, bool flush=true); //This method read length bytes and stores them into the buffer.
      //If flush is true, then reading will start from the line. If it is false, it will try to read from the buffer, then from the port.
      //It returns with the number of available bytes at the present situation (returned+buffer) if ret>=length; 
      //ret<length: timeout error, only this number of datasets are received and put into the array
      //ret<0: other errors
    bool setTimeout(double second);//Sets the timeout into the requested time
    bool isOpened(){return fd>0;};//Returns with opened status
    std::string getPortName();//Returns with the port name
    void getCommParameters(int *baud, int *databits, char *parity, int *stopbits);//This function returns with the currently set communication parameters as baud rate, data bits, parity and number of stopbits
    double getTimeout();//Returns with the current value of timeout
  private:
    //members
    static const int LOCAL_BUFFER_SIZE=2048;//Size of the local receiving buffer
    std::string device;//It stores the name of the device
    int fd;//Port descriptor: if it is less than zero it indicates not allocated status
    termios oldstruct, newstruct;//Old and new structures. Former for restore, later for the new
    bool rc;//Raw mode (true) or canonic mode switch
    struct timeval *timeout;//Timeout function important in data receiving
    std::queue<uint8_t> temp;//Temporary buffer to store received, but not used bytes
    
    //methods
};

#endif
