#include <cstring>
#include <iostream>
#include <map>
#include <string>
#include <vector>

#include <arpa/inet.h>
#include <errno.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#ifndef UDP_H
#define UDP_H

//Comparison operators for the sockaddr in structure
//bool operator== (const struct sockaddr_in &a, const struct sockaddr_in &b);// { return (a.sin_port==b.sin_port)&&(a.sin_addr.s_addr==b.sin_addr.s_addr);}
bool operator!= (const struct sockaddr_in &a, const struct sockaddr_in &b);//{ return (a.sin_addr.s_addr!=b.sin_addr.s_addr)||(a.sin_port!=b.sin_port);}
bool operator< (const struct sockaddr_in &a, const struct sockaddr_in &b);//{ return (a.sin_addr.s_addr<b.sin_addr.s_addr)||((a.sin_addr.s_addr==b.sin_addr.s_addr)&&(a.sin_port<b.sin_port));}

class UDP
{
  public:
    UDP(const int local_port_no, const std::string local_ip, std::vector<std::string> &far_ips, std::vector<int> &far_ports, std::vector<int> &far_ids);//Constructor by setting the client/server to the local port number acting as server (local ip should be="") or client(local_ip should be specified) and set the IP/port/module_id structures up
    ~UDP();//Destructor
    //ID-ip-port related methods
    std::string GetFarIP(int id) const;//Returns with the far IP address belonging to id
    int GetFarPort(int id) const;//Returns with the far port number belonging to id
    bool SetFar(std::string ip, int port_no, int module_id);//Add an item to the IP/port/module_id structure
    void DeleteFar(int module_id);//Delete the module_id from the list
    bool ModifyFar(int prev_id,int new_id);//Modify the prev_id to new_id
    bool ModifyFar(int module_id, std::string new_ip, int new_port_no);//Modifies the current setup of module_id by setting the new_ip and new_port_no
    //Local port settings and initialization
    std::string GetLocalIP() const;//Returns with the local IP address
    int GetLocalPort() const;//Returns with the local port number
    bool SetLocal(const int local_port_no, const std::string &local_ip);//Configures the server(local_ip="")/client(specific local_ip) to use the local_port_no. If a socket already in operation, it reset to the new values
    std::vector<int> GetFarIDs();//Returns with the far id's 
  public:
    //Communication
    bool SendMessage(const char *msgtosend, const int &send_length, const int &destination_id);//This method send the send_length long msgtosend to destination id. Returns with successfull delivery report
    int ReceiveMessage(char *buffer, const int &buffer_size, bool &tolerate_other_sender, const int &expected_sender_id);//This method writes the received message to the buffer_size long buffer and returns with the length of the message. If error happens, it returns with negative value. If tolerate_other_sender is set to false the id of the sender should be the same as expected_sender_id 
  
  protected:
    //members
    int Socket_fd;//File descriptor of the socket  
    
  private:
    //members
    struct sockaddr_in Local;//Local IPv4 address
    struct sockaddr_in Partner;//IPv4 address of the partner used at ReceiveMessage
    size_t Partnersize;//Size of the Partner
    std::map<int,struct sockaddr_in> Far_id;//Set of far IP and  port setups identified by ID
    std::map<struct sockaddr_in,int> Far_sock;//Set of ID identified by IP and port structure
};

class UDPClient: public UDP
//Realizes a send and receive kind of interactions
{
  public:
    UDPClient(const int local_port_no, const std::string local_ip, std::vector<std::string> &far_ips, std::vector<int> &far_ports, std::vector<int> &far_ids):UDP(local_port_no,local_ip, far_ips,far_ports,far_ids){};//Constructor by setting to the local port number acting as client and set the IP/port/module_id structures up
    int SendAndReceive(const char *msgtosend, const int send_length, char *answer, const int &answer_maxsize, const int &destination_id, bool only_receive=false, bool tolerate_other_sender=false);//If onlyrecive is false it sends a send_length long msgtosend to the destination_id address and receive the answer to an answer_maxsize long answer buffer. If only_receive is true, it waits for the reply only. If tolerate_other_sender is false, it returns with error if other than the computer with destination_id is answering. Returns with the valid length of the answer or <0 in the case of error, and puts the answer to answer_length array
};

class UDPServer: public UDP
//Realizes an UDP server interactions
{
  public:
    UDPServer(const int local_port_no, std::vector<std::string> &far_ips, std::vector<int> &far_ports, std::vector<int> &far_ids):UDP(local_port_no,"",far_ips,far_ports,far_ids){};//Constructor by setting to the local port number acting as server and set the IP/port/module_id structures up
};



class UDPServerTimeOut: public UDP
//Realizes an UDP server interactions
{
  private:
    struct timeval Tv;//Timeval structure to store timeout data

  public:
    UDPServerTimeOut(const int local_port_no, std::vector<std::string> &far_ips, std::vector<int> &far_ports, std::vector<int> &far_ids,int timeout_milliseconds);//Constructor by setting to the local port number acting as server and set the IP/port/module_id structures up, also timeout is set in milliseconds
    int GetTimeoutMS() const {return Tv.tv_sec*1000+Tv.tv_usec/1000;};//Returns with the timeout in milliseconds
};
#endif

