//TESTED:04.10.2013 by Laszlo Temleitner

#include <cstdint>
#include <iostream>
#include <iomanip>
#include <sstream>
#include "Serial.h"
#include "Thermometer.h"

#ifndef UPP_H
#define UPP_H

class UPP: public Thermometer
//This class describes the Universal Pyrometer Protocol
{
  public:
    //methods
    UPP(const int address, const std::string nPort="ttyUSB0", const bool cf=true, const int nBaud=19200, const double timeout=UPP::TIMEOUT);//Constructor: device address, serial port name, Celsius(t)/Fahrenheit(f), baud rate, timeout
    ~UPP(){delete communicator;};//Destructor
    bool ResetCommunication(const std::string nPort="ttyUSB0", const int nBaud=19200, const double timeout=0.05);//Reset the communication parameters: serial port name, transmission rate, timeout
   //READ ONLY
    bool GetTemperature(double &onechannel, double &ratio);//Get the one channel temperature in the measurement unit and the ratio temperature
    double GetMeasuredValue();//Returns with the measured value, -1: communication error, -2: overflow error
    int GetSignalStrength();//Returns with the strength of the signal: 0..1500
    void getTempHW(double *result){double r[TEMP_PARAMETERS]; if(GetTemperature(r[0], r[1])) for(int i=0; i<TEMP_PARAMETERS; i++) result[i]=r[i]; else  for(int i=0; i<TEMP_PARAMETERS; i++) result[i]=-5000.0; return;};//Returns with the temperatures: , should be implemented in child class
   //SET ONLY 
    bool SetOperationMode(const bool ratioonech=true);//Sets the operation mode to 2channel (ratio, true) or 1 channel (false)
    bool ClearMaxStorage();//Clears the maximum storage (software simulation of external clearance)
    //Pairs
    //subrange
    bool GetSubRange(int &lowlimit, int &upperlimit);//Returns the lower and upper limits of the subrange in the requested unit
    bool SetSubRange(const int lowlimit, const int upperlimit);//Sets the lower and upper limits of the subrange in the requested unit, if lowlimit=0: reset
    //response time
    bool SetResponseTime(const int time=0);//Sets the response time: from 0.000 to 10.000s (with discrate steps:0=min; 1=0.01; 2=0.05; 3=0.25; 4=1.0; 5=3.0; 6=10.0)
    int GetResponseTime();//Returns with the response time: from 0.000 to 10.000s (with discrate steps: -1=error, 0=min; 1=0.01; 2=0.05; 3=0.25; 4=1.0; 5=3.0; 6=10.0)
    //peak memory time
    bool ClearPeakMemoryTime(const int time=0);//Clear peak memory time. It changes by values (0=OFF; 1=0.01s; 2=0.05s; 3=0.25s; 4=1.0s; 5=5.0s; 6=25.0s; 7=EXTERN; 8=AUTO; 9=HOLD)
    int GetPeakMemoryTime();//Gets the peak memory time. It changes by values (-1=error, 0=OFF; 1=0.01s; 2=0.05s; 3=0.25s; 4=1.0s; 5=5.0s; 6=25.0s; 7=EXTERN; 8=AUTO; 9=HOLD)
    //not available for this device
      bool GetLaserStatus();//Returns with the status of laser: on(t) / off (f) / error, not allowed (f)
      bool SwitchLaser(const bool on=false);//Switch on(t)/off(f) laser
    //Pyrometer not frequent set and get parameters
    void getRuralSetParameters(bool *temp_unit, int *curr_internal_temp, int* max_internal_temp, int *minint_switchoff_level, double *emissivity, double *window_trans, double *emiss_ratio, bool bulky=false);//Gives information about the not so frequently set parameters: temperature unit(C-t, F-f), current and maximal internal temperature, minimum intensity switch-off level, emissivity in onechannel mode, window transmittance, emissivity ratio (epsilon1/epsilon2). 
    bool setRuralSetParameters(bool temp_unit=true, int min_int_switch_off_level=10, double emissivity=1.0, double window_trans=1.0, double emiss_ratio=1.0, int dirty_window_warn=0);//Set the values of not frequently set parameters: temperature unit(C-t, F-f), minimum intensity switch-off level, emissivity in onechannel mode, window transmittance, emissivity ratio (epsilon1/epsilon2), "Dirty window" warning level: from 0 to 99%.
    //Get and set communication parameters
    bool getCommunication(int *address, int *baud, bool *analog_output, bool bulky=true);//Get communication related parameters: address, baud rate, analog output setting (t: 0..20mA, f:4..20mA), bulky reading, returns with true if communication succeeded
    bool setCommunication(int address=0, int baud_rate=19200, bool analog_output=true);//Changes the communication related parameters: device address (00 to 97 (regular); 98, 99 (global address with response)), baud rate (1200 to 115200 baud) and analog output (t: 0..20mA, f:4..20mA)
    //About
    void About(std::string *ref_num, std::string *dev_type, std::string *ser_num, std::string *soft_ver, std::string *comm_det, std::string *soft_det, int *basic_range_lo, int *basic_range_hi);
    //Gives general information about the device: reference number, device type, detector serial number, device type and software version, communication deatails, software details, and finally with basic range limits    
  private:
    //middle level methods
    //collective read
    bool ReadParams(double &emissivity, int &response_time, int &peak_memory_time, bool &analog_output, int &internal_temp, int &device_addr, int &baud, double &emissivity_ratio);//Read status information on: emissivity, response time, peak memory time, analog output, internal temperature, device address, baud rate, emissivity ratio
    //non-frequent get and set related
    bool GetTemperatureUnit(){return cfunit;};//It returns with the current unit of temperature (C:t, F:f)
    int  GetInternalTemperature(const bool currmax=true);//Returns with the internal temperature, which can be the current one (true) or maximum temperature (false). It is changes between 0..98C or 32..210F; -1: error
    int  GetMinimumIntensitySwitchOffLevel();//Gets the minimum intensity switch-off level, -1: error
    double GetWindowTrans();//Returns with the current setting for the transmittance of the window from 0.05 to 1.0; -1.0: error
    double GetEmissivity();//Returns with the current setting of emissivity for one channel temperature from 0.05 to 1.0; -1.0: error
    double GetEmissivityRatio();//Returns with the current setting for the emissivity ratio (epsilon1/epsilon2): from 0.8 to 1.2; -1.0: error
    bool SetTemperatureUnit(const bool cf=true);//Sets the display unit in C (true)/F (false)
    bool SetMinimumIntensitySwitchOffLevel(const int level=10);//It sets the minimum intensity switch-off level: between 2 and 50
    bool SetEmissivity(const double epsilon=1.0);//Sets the emissivity for one channel temperature: from 0.05 to 1.0
    bool SetWindowTrans(const double tau=1.0);//Sets the window transmittance: from 0.05 to 1.0
    bool SetEmissivityRatio(const double ratio=1.0);//Sets the emissivity ratio(epsilon1/epsilon2): from 0.8 to 1.2
    bool SetDirtyWindowWarn(const int level=0);//Set the "Dirty window" warning level: from 0 to 99%
    //Communication related
    int GetAddress(){return (add[0]-48)*10+add[1]-48;};//Returns with the actual address of the device
    bool SetAnalogOutput(const bool set=true);//Sets the analog output: true 0..20mA, false: 4..20mA
    bool SetBaudRate(const int rate=19200);//Sets the requested baudrate to communicate: from 1200 to 115200 baud
    bool SetDeviceAddress(const int address=0);//Sets the new device address: 00 to 97 (regular); 98, 99 (global address with response)
    void SetAddress(const int address){add[0]=48+(address%100)/10; add[1]=48+address%10;};//Sets the address of the device, which communicates
    //About related
    std::string GetReferenceNumber();//Gets the reference number, Error: "ERROR"
    bool GetBasicRange(int &lowlimit, int &upperlimit);//Returns the lower and upper limits of the basic range in the requested unit
    std::string GetDeviceType();//Returns with the device type or with error message
    std::string GetSerialNumber();//Returns the serial number of the detector
    std::string GetDeviceTypeSoftwareVersion();//Returns with the device type (25) and the version of the software (MMYY)
    std::string GetCommunicationDetails();//Returns with the details of the software: DD.MM.YY SOFT.VERSION
    std::string GetSoftwareDetails();//Returns with the details of the software: DD.MM.YY SOFT.VERSION
    //low-level methods
    bool setvalue(char *command, const int length, uint8_t *message, const bool onlysend=false, const bool wait_extra=false);//Send the command [2 letters] with the length long message, if onlysend true, it does not wait for response. If wait_extra is true, it waits extra amount of time to the response
    bool getvalue(char *command, const int length, uint8_t *message, bool limits=false);//Get the actual value of command [2 letters]. Its message is length long message. If limits are true, the limits are the question
    bool communicate(uint8_t *sendbuffer, const int slen, uint8_t *replybuffer, const int rlen, const bool onlysend=false, const bool wait_extra=false);//Communicate with serial interface: it contains the data to send, its length, the reply message and its expected length. If onlysend true, then it send the message only. If wait_extra is true, it waits extra amount of time to the response
    void to(const int number, uint8_t *buffer, const int blength, const bool dechex=true);//Converting int (number) to decimal(t)/hexa(f) digits (into blength long buffer)
    int from(uint8_t* buffer, const int blength, const bool dechex=true);//Converting the decimal(t)/hexa(f) digits containing in blength long buffer to integer
    std::string tostr(uint8_t* buffer, const int length);//Converting the digits containing the buffer into string
    //static members
    static const int TEMP_PARAMETERS=2;//Number of temperature parameters to register
    static const uint8_t CR=0xd;//ASCII Carriage return
    static const double TIMEOUT;//Default timeout time in s
    static const double WAITSUCCESS;//Default time to wait between successfull receive and the next command in s
    //members
    Serial *communicator;//Serial communication object
    uint8_t add[2];//Address
    bool cfunit;//Celsius(t)/Fahrenheit(f) units
};

#endif