#include <numeric>
#include <iostream>
#include "Modbus.h"
#include "MTESTControl.h"
#include "Camac.h"
#include "SerialCamac.h"
#include "ParallelCamac.h"
#include "CamacTest.h"
#include "Motor.h"
#include "Scaler.h"
#include "kd48p.h"
#include "UPP.h"
#include "OHM.h"
#include "Thermometer.h"
#include "IOOperation.h"
#include "MTESTRawData.h"
#include "Logic.h"
#include <unistd.h>
#include <chrono>
//#include <sys/types.h>


void testThermometer (Thermometer *therm);

bool update_default_file(std::string motor_name, int position, bool adata, double angle, uint32_t low_limit, uint32_t high_limit);
//This function updates the default file with the provided position belonging to motor_name. If adata is present, the angular data should be provided. Also low and high limit are essentials 

template <typename T> int sign (T value) {return (T(0) < value)-(T(0) > value);}

int main(int argc, char **argv)
{
  //Thermometers and temperature controllers
  bool save_data=false;
  bool ohm2=false;
  bool kd48=false; 
  bool pyrometer=false;
  //Motors
  bool harwell_motor=false;
  bool m14motor=false;

  //Detectors
  bool monitor=false;
  bool psd=false;
  bool hw_counter=false;
  bool rtc_counter=false;
  bool psdmonitorcombined=false;
  
  //IO
  //bool io=false;
  //bool io_discover=false;
  //bool i_discover=false;
  
  bool commandcontrol=false;
  
  
  bool inhib_trial=false;
  bool measurement_run=true;
  bool run_break_low_intensity=false;
  bool monochromator_scan=false;
  bool transform2normalized=false;
  bool check_monitor=false;
  bool check_monitor2=false;

  bool prof_test=false;
  bool prof_test2=false;
  
  bool dual_gate_test=false;
  
  if(dual_gate_test)
  {
    std::cout<<"This program tests the efficienty of the communication"<<std::endl;
    SerialCamac sc("camac_serial",1,"",0.5,9600);
    Logic::DualGate dg(sc,16);
    std::cout<<"Actual value of the setup (should be 0): "<<dg.GetConfiguration(true)<<std::endl;
    std::cout<<"\"A\" follows 1: "<<(dg.SetConfiguration(std::string("1")) ? "OK" : "ERROR" )<<std::endl;
    std::cout<<"Actual value of the setup (should be 1): "<<dg.GetConfiguration(false)<<std::endl;
    std::cout<<"Actual value of the setup (should be 1): "<<dg.GetConfiguration(true)<<std::endl;//Check it by 2 ways
    std::cout<<"Resets it: "<<(dg.SelectiveConfiguration(std::string("1"),false)  ? "OK" : "ERROR" )<<std::endl;
    std::cout<<"Actual value of the setup (should be 0): "<<dg.GetConfiguration(true)<<std::endl;
    std::cout<<"\"A\" follows 4: "<<(dg.SelectiveConfiguration(std::string("4"),true) ? "OK" : "ERROR" )<<std::endl;
    std::cout<<"Actual value of the setup (should be 4): "<<dg.GetConfiguration(true)<<std::endl;
    std::cout<<"Waits until the end of the electric test... to continue type any number"<<std::endl;
    int number;
    std::cin>>number;
    std::cout<<"Resets it by SetConfiguration: "<<(dg.SetConfiguration(std::string("0")) ? "OK" : "ERROR" )<<std::endl;
    std::cout<<"Actual value of the setup (should be 0): "<<dg.GetConfiguration(true)<<std::endl;
    std::cout<<"\"A\" follows N1: "<<(dg.SetConfiguration(std::string("N1")) ? "OK" : "ERROR" )<<std::endl;
    std::cout<<"Actual value of the setup (should be N1): "<<dg.GetConfiguration(false)<<std::endl;
    std::cout<<"Actual value of the setup (should be N1): "<<dg.GetConfiguration(true)<<std::endl;//Check it by 2 ways
    std::cout<<"Resets it by \"1\": "<<(dg.SelectiveConfiguration(std::string("1"),false) ? "OK" : "ERROR" )<<std::endl;
    std::cout<<"Actual value of the setup (should be 0): "<<dg.GetConfiguration(true)<<std::endl;
    std::cout<<"\"A\" follows N1: "<<(dg.SelectiveConfiguration(std::string("N1"),true) ? "OK" : "ERROR" )<<std::endl;
    std::cout<<"Actual value of the setup (should be N1): "<<dg.GetConfiguration(true)<<std::endl;
    std::cout<<"Resets it by \"N1\": "<<(dg.SelectiveConfiguration(std::string("N1"),false) ? "OK" : "ERROR" )<<std::endl;
    std::cout<<"Actual value of the setup (should be 0): "<<dg.GetConfiguration(true)<<std::endl;
    std::cout<<"\"A\" follows N4: "<<(dg.SelectiveConfiguration(std::string("N4"),true) ? "OK" : "ERROR" )<<std::endl;
    std::cout<<"Actual value of the setup (should be N4): "<<dg.GetConfiguration(true)<<std::endl;
    std::cout<<"Waits until the end of the electric test... to continue type any number"<<std::endl;
    std::cin>>number;
    std::cout<<"Resets it by \"4\": "<<(dg.SelectiveConfiguration(std::string("4"),false) ? "OK" : "ERROR" )<<std::endl;
    std::cout<<"Actual value of the setup (should be 0): "<<dg.GetConfiguration(true)<<std::endl;
    std::cout <<"Program finished\n";
  }

  if(prof_test2)
  {
    std::cout<<"This program test the efficienty of the communication"<<std::endl;
    SerialCamac sc("camac_serial",1,"",0.5,9600);
  }
  
  if(prof_test)
  {
    std::cout<<"This program test the efficienty of the communication"<<std::endl;
    SerialCamac sc("camac_serial",1,"",0.5,9600);
    IOOperation::OutputRegister oreg(sc,14);
    Positioner:: HarwellMotor tth(sc,"2Theta", 17, 33868, 47268, oreg, 0);

    std::chrono::high_resolution_clock::time_point t1 = std::chrono::high_resolution_clock::now();
    for(int i=0; i<100; i++) tth.getPosition();
    std::chrono::high_resolution_clock::time_point t2 = std::chrono::high_resolution_clock::now();
    std::cout <<"Elapsed time: "<<std::chrono::duration_cast<std::chrono::microseconds>( t2 - t1 ).count()<<" ms\n";
  }
  
  if(check_monitor2)
  {
    if(argc!=2)
    {
      std::cerr<<"Usage: mtest_check_snapshots filename_without_dat_and_snapshot_number"<<std::endl;
      return 0;
    }
    
    bool file_found=true;
    int i=0;
    int last_relevant_data=0;
    std::vector<double> real_time;
    std::vector<long> cumul_monitor;
    std::vector<long> cumul_psd;
    double p_prev[2];
    double dp_prev[2];
    double p[2];
    double dp[2];
    bool previous=false;
    do
    {
      i++;
      std::string filename(argv[1]);
      filename+="_"+std::to_string(i)+".dat";
      std::ifstream is(filename.c_str());
      file_found=is.is_open();
      
      if(file_found)
      {
        MTESTRawData out;
        out.read(is);
        is.close();
	//Get some relevant data
	real_time.push_back((double) out.counters[0].count_data[1]);
	cumul_monitor.push_back((long) out.counters[0].count_data[0]);
	cumul_psd.push_back((long) std::accumulate(out.counters[1].count_data.begin(),out.counters[1].count_data.end(),0));
	//Analysis starts here
	//1.) Check sudden drop of psd/monitor ratio
	if(i>2)
	{
	  //double ratio_actual=((double)(cumul_psd[i-1]-cumul_psd[i-2]))/(cumul_monitor[i-1]-cumul_monitor[i-2]);
	  //double ratio_previous=((double)(cumul_psd[i-2]-cumul_psd[i-3]))/(cumul_monitor[i-2]-cumul_monitor[i-3]);
	  //double ratio_uncertainty=ratio_previous*sqrt(1.0/(cumul_psd[i-2]-cumul_psd[i-3])+1.0/(cumul_monitor[i-2]-cumul_monitor[i-3]));
	  if((cumul_psd[i-1]==cumul_psd[i-2])&&(cumul_monitor[i-1]==cumul_monitor[i-2]))
	  {
	    //Probably, end of measurement has been found at the previous entry (this also helps to avoid error of bad time)
	    last_relevant_data=i-1;
	  }
	  else
	  {
	    last_relevant_data=i;
	    if(i>11)
	    {
	      long difference_prev=cumul_monitor[i-2]-cumul_monitor[i-12];
	      double dt_prev=real_time[i-2]-real_time[i-12];
	      long difference=cumul_monitor[i-1]-cumul_monitor[i-2];
	      double dt=real_time[i-1]-real_time[i-2];
	      p_prev[0]=p_prev[1];
	      dp_prev[0]=dp_prev[1];
	      p[0]=p[1];
	      dp[0]=dp[1];
	      p_prev[1]=difference_prev/dt_prev;
	      dp_prev[1]=p_prev[1]*sqrt(1.0/difference_prev+1.0/dt_prev/dt_prev);
	    
	      p[1]=difference/dt;
	      dp[1]=p[1]*sqrt(1.0/difference+1.0/dt/dt);
	      if(i!=12)
	      {
		
		if (std::abs((p[1]-p_prev[1])/dp[1])>0.5)
		{
		  if ((std::abs((p[0]-p_prev[0])/dp[0])>0.5)&&(sign((p[0]-p_prev[0])/dp[0])*sign((p[1]-p_prev[1])/dp[1])==1))
		  {
		    if(!previous) std::cout<<i-1<<" "<<p[0]<<" "<<dp[0]<<" "<<p_prev[0]<<" "<<dp_prev[0]<<" "<<(p[0]-p_prev[0])/dp[0]*100.0<<std::endl;
		    std::cout<<i<<" "<<p[1]<<" "<<dp[1]<<" "<<p_prev[1]<<" "<<dp_prev[1]<<" "<<(p[1]-p_prev[1])/dp[1]*100.0<<std::endl;
		    previous=true;
		  }
		}
		else previous=false;
	      }
	    }
	  }
	  //if(std::abs((ratio_actual-ratio_previous)/ratio_uncertainty)>3) std::cout<<i<<" "<<ratio_actual<<" "<<ratio_previous<<" "<<(ratio_actual-ratio_previous)/ratio_uncertainty<<std::endl;
	}
      }
    } while(file_found);
    
    std::cout<<argv[1]<<"_"<<last_relevant_data<<".dat"<<std::endl;
    return 0;
  }



  if(check_monitor)
  {
    if(argc!=2)
    {
      std::cerr<<"Usage: mtest_check_snapshots filename_without_dat_and_snapshot_number"<<std::endl;
      return 0;
    }
    
    bool file_found=true;
    int i=0;
    std::ofstream os("out.dtxt");
    os<<"#Snapshot, Real time, time difference, cumulative counts, difference counts, relative monitor cps, cumulative psd counts, difference psd, relative psd cps, relative psd/monitor, cumulative psd/monitor"<<std::endl;
    double prev_time=0.0;
    long prev_monitor=0;
    long prev_psd=0; 
    bool first=true;
    do
    {
      i++;
      std::string filename(argv[1]);
      filename+="_"+std::to_string(i)+".dat";
      std::ifstream is(filename.c_str());
      file_found=is.is_open();
      
      if(file_found)
      {
        MTESTRawData out;
        out.read(is);
        is.close();
        double difference_t=out.counters[0].count_data[1]-prev_time;
        double difference_m=out.counters[0].count_data[0]-prev_monitor;
        long sum=std::accumulate(out.counters[1].count_data.begin(),out.counters[1].count_data.end(),0);
        double difference_p=sum-prev_psd;
        if(first) first=false;
        else
        {
    	    os<<i<<" "<<out.counters[0].count_data[1]<<" "<<difference_t<<" "<<out.counters[0].count_data[0]<<" "<<difference_m<<" ";
    	    if(difference_t>0.0) os<<difference_m/difference_t<<" ";
    	    else os<<" 0 ";
    	    os<<sum<<" "<<difference_p<<" ";
    	    if(difference_t>0.0) os<<difference_p/difference_t<<" ";
    	    else os<<" 0 ";
    	    if(difference_m>0.0) os<<difference_p/difference_m;
    	    else os<<" 0";
    	    if(out.counters[0].count_data[0]>0) os<<" "<<((double)sum)/out.counters[0].count_data[0];
    	    else os<<" 0";
    	    os<<std::endl;
    	}
        prev_time=out.counters[0].count_data[1];
        prev_monitor=out.counters[0].count_data[0];
        prev_psd=sum;
      }
    } while(file_found);
    std::cout<<argv[1]<<"_"<<i-1<<".dat"<<std::endl;
    os.close();
    return 0;
  }
  
  if(transform2normalized)
  {
    if(argc!=2)
    {
      std::cerr<<"Usage: mtest_normalize filename_without_dat"<<std::endl;
      return 0;
    }
    
    std::string filename(argv[1]);
    filename+=".dat";
    std::ifstream is(filename.c_str());
    MTESTRawData out;
    out.read(is);
    is.close();
    filename=argv[1];
    filename+="_norm.dat";
    std::ofstream os(filename.c_str());
    os<<"#Real time: "<<out.counters[0].count_data[1]<<"s, live time: "<<out.counters[0].count_data[1]-out.counters[1].additionals[0]+out.counters[1].additionals[1]<<"s, Deadtime:"<<
    ((out.counters[0].count_data[1]>0) ? ((double)(out.counters[1].additionals[0]-out.counters[1].additionals[1])*100.0)/out.counters[0].count_data[1] : 0.0)<<"%"<<std::endl;
    long sum=std::accumulate(out.counters[1].count_data.begin(),out.counters[1].count_data.end(),0);
    
    os<<"#Monitor counter: "<<out.counters[0].count_data[0];
    if(out.counters[0].count_data[1]!=0) os<<", psd cps: "<<((double) sum)/out.counters[0].count_data[1]<<"+/-"<<sqrt((double)sum)/out.counters[0].count_data[1]<<", monitor cps: "<<((double)out.counters[0].count_data[0])/out.counters[0].count_data[1]<<"+/-"<<sqrt((double)out.counters[0].count_data[0])/out.counters[0].count_data[1]<<std::endl;
    else os<<std::endl;
    os<<"# "<<out.motors.back().pos_angles[0]<<" degrees, "<<out.comment_sample<<std::endl;
    os<<"#Channel, normalised intensity, uncertainty"<<std::endl;
    if(out.counters[0].count_data[1]!=0) for(int i=0; i<out.counters[1].no_channels; i++) os<<out.counters[1].caption[i]<<" "<<((double)out.counters[1].count_data[i])/out.counters[0].count_data[1]<<" "<<((out.counters[1].count_data[i] > 0) ?
      ((double)out.counters[1].count_data[i])/out.counters[0].count_data[1]*sqrt(1.0/out.counters[1].count_data[i]+1.0/out.counters[0].count_data[1]) : 0)<<std::endl;
    else for(int i=0; i<out.counters[1].no_channels; i++) os<<out.counters[1].caption[i]<<" 0 0"<<std::endl;
    os.close();
    return 0;
  }
  
  if(monochromator_scan)
  {
    std::cout<<"This program helps to set up monochromator crystal correct"<<std::endl;
    if(argc!=2)
    {
        
	std::cerr<<"mtest_monochromator_scan output_file"<<std::endl;
        return 0;
    }
    //Try to open the external datafile
    std::string filename(argv[1]);


    //Initializing the camac controllers
    SerialCamac sc("camac_serial",1,"",0.5,9600);
    sc.SetResetInhibit(true);
    
    //Initialize IO
    IOOperation::OutputRegister oreg(sc,14);
    IOOperation::InterruptRegister ireg(sc,13);
    
    //Initialize counters
    Scaler::MonitorCounter monc(std::string("Monitor"),sc, 8, false, (Scaler::MON_CNT) time, true, Scaler::MonitorCounter::DEFAULT_INTENSITY_NIVEAU, Scaler::MonitorCounter::DEFAULT_CONSEC, true,true);
    
    //Initialize Motors
    std::vector<Positioner::Motor*> motor;
    Positioner::StepMotorController smc(sc,4);
    motor.push_back(new Positioner::ParallelMotor(sc,"MTilt",1,1,(uint16_t) 1000, (uint16_t) 4150, oreg, ireg, smc, 1.0, 144, 2445, 2, 0,13));
    motor.push_back(new Positioner::ParallelMotor(sc,"MShift",2,3,(uint16_t) 200, (uint16_t) 7000, oreg, ireg, smc, 1.0, 144, 2445, 2, 0,13));
    motor.push_back(new Positioner::AngleParallelMotor(sc,"MRotate",3,5, (uint16_t) 23568, (uint16_t) 39051, oreg, ireg, smc, 2.0, 228, 1590, 36445.0, -1.0/180.0, 2, 0,16));
    motor.push_back(new Positioner::CushionAPMotor(sc,"DArmRotate",4, 6, (uint16_t) 0, (uint16_t) 4289, oreg, ireg, smc, 2.0, 228, 1590, 7074.0, -0.01, 2, 0, 16));

    //Save head information to the storage place
    MTESTRawData out;
    int checktime;
    out.comment_sample="Monochromator setup";
    std::ifstream is("default");
    std::getline(is,out.comment_ms_collimator);
    std::cout<<"Monochromator-sample collimator: "<<out.comment_ms_collimator<<std::endl;
    std::getline(is,out.comment_ms_slits);
    std::cout<<"Monochromator-sample slits: "<<out.comment_ms_slits<<std::endl;
    std::getline(is,out.comment_s_holder);
    std::cout<<"Sample holder: "<<out.comment_s_holder<<std::endl;
    std::getline(is,out.comment_s_environment);
    std::cout<<"Sample environment: "<<out.comment_s_environment<<std::endl;
    std::getline(is,out.comment_sd_slits);
    std::cout<<"Sample-detector slits: "<<out.comment_sd_slits<<std::endl;
    std::getline(is,out.comment_d_position);
    std::cout<<"Detector position: "<<out.comment_d_position<<std::endl;
    is>>checktime;
    std::cout<<"Snapshot will be taken each "<<checktime<<"th seconds during the measurement"<<std::endl;
    std::vector<std::string> name=monc.getNames();
    if(monc.getCounterDimensions().size()!=1)
    {
        std::cout<<"ERROR: (commod at monochromator_scan): The monitor counter dimension is supposed to be 1 instead of "<<monc.getCounterDimensions().size()<<"!"<<std::endl;
        return -1;
    }
    std::vector<std::string> mon_captions;
    mon_captions.push_back("Counts");
    mon_captions.push_back("Time-sec");
    out.addCounterHead((monc.getNames())[0], monc.getCounterNo(), 0, mon_captions);

    //Get the positions
    
    bool first=true;
    bool no_out=true;
    int pos[4];//positions of the motors
    for(int i=0; i<4; i++) 
    {
      pos[i]=(int) motor[i]->getPosition();
      out.addMotor(motor[i]->getName(), pos[i], false, 0);
      Positioner::AngleParallelMotor *am=dynamic_cast<Positioner::AngleParallelMotor*> (motor[i]);
      update_default_file(motor[i]->getName(), pos[i], am!=0, ((am==0) ? 0.0 : am->getAPosition()), motor[i]->getLimit(false), motor[i]->getLimit(true));
    }
    
    while(no_out)
    {
      //Writing out information
      std::cout<<"Motor MTilt(1) position: "<<pos[0]<<std::endl;
      std::cout<<"Motor MShift(2) position: "<<pos[1]<<std::endl;
      std::cout<<"Motor MRotate(3) position: "<<pos[2]<<std::endl;
      std::cout<<"Motor DArmRotate(4) position: "<<pos[3]<<std::endl;
      std::cout<<"Switch on compressed air(5)"<<std::endl;
      std::cout<<"Finish (6)"<<std::endl;
      int ans;
      std::cout<<"Which one would you like to move/leave the program?";
      std::cin>>ans;
      if(ans<5)
      {
        //Scanning task
        no_out=true;
        ans--;
        std::cout<<"Please specify the initial_position starting position end_position step time_s:"<<std::endl;
        int from, to, step, time, init;
        std::cin>>init>>from>>to>>step>>time;
        std::cout<<"Scanning will be performed from "<<from<<" to "<<to<<" with "<<step<<" steps measuring "<<time<<" s at each positions "<<std::endl;
        std::cout<<"Before scan, motor will be moved to "<<init<<" position"<<std::endl;
	int no_steps=(to-from)/step;
        if(motor[ans]->getPosition()!=(uint32_t)init)
        {
	  if(!motor[ans]->moveTo((uint32_t) init))
	  {
	    std::cerr<<"ERROR: at motor move"<<std::endl;
	  }
          else std::cout<<"The move of the motor started to "<<init<<" position"<<std::endl;
        }
	while(!motor[ans]->isAllStopped()) 
        {
	  std::cout<<"\tActual position: "<<(long)motor[ans]->getPosition()<<" pulses"<<std::endl;
          sleep(1);
        }
        sleep(1);
        pos[ans]=motor[ans]->getPosition();
	std::cout<<"\tFinal position: "<<(long)pos[ans]<<" pulses"<<std::endl;
        Positioner::AngleParallelMotor *am=dynamic_cast<Positioner::AngleParallelMotor*> (motor[ans]);
        update_default_file(motor[ans]->getName(), pos[ans], am!=0, ((am==0) ? 0.0 : am->getAPosition()), motor[ans]->getLimit(false), motor[ans]->getLimit(true));
        bool stop=false;
        for(int i=0; (i<no_steps+1)&&(!stop); i++)
	{
	  //Move the motor into the requested position
	  if(motor[ans]->getPosition()!=(uint32_t)(from+i*step))
	  {
	    if(!motor[ans]->moveTo((uint32_t) (from+i*step)))
	    {
	      std::cerr<<"ERROR: at motor move"<<std::endl;
	    }
            else std::cout<<"The move of the motor started to "<<from+i*step<<" position"<<std::endl;
	  }
	  while(!motor[ans]->isAllStopped()) 
	  {
	    std::cout<<"\tActual position: "<<(long)motor[ans]->getPosition()<<" pulses"<<std::endl;
	    sleep(1);
	  }
	  //Motor stopped
	  sleep(1);
	  pos[ans]=motor[ans]->getPosition();
	  std::cout<<"\tFinal position: "<<(long)pos[ans]<<" pulses"<<std::endl;
          Positioner::AngleParallelMotor *am=dynamic_cast<Positioner::AngleParallelMotor*> (motor[ans]);
      update_default_file(motor[ans]->getName(), pos[ans], am!=0, ((am==0) ? 0.0 : am->getAPosition()), motor[ans]->getLimit(false), motor[ans]->getLimit(true));
	  std::cout<<"Start programming the detectors up..."<<std::endl;
	
	  //Setup counters
	  if(!monc.SetPreset((Scaler::MON_CNT) time)) std::cout<<"Error at setpreset of the monitor counter"<<std::endl;
	  if(!monc.Configure()) std::cout<<"Error at monitor configuration"<<std::endl;
	
	  //Start
	  std::cout<<"Start detector"<<std::endl;
	  time_t start_time=std::time(NULL);
	  if(!monc.Start()) std::cout<<"Error at monitor start"<<std::endl;
	  std::cout<<"Start time: "<<Write_Time(start_time)<<std::endl;
	
	  double remaining=3;
	  bool isfin=false;
	  int cnt=0;
	  Scaler::MON_CNT md[2];
	  if(no_out)
	  {
	    do
	    {
	      if(cnt!=0)sleep((checktime>remaining) ? remaining :checktime);
	      if(!monc.Read(md,remaining,isfin)) std::cout<<"Something wrong.."<<std::endl;
	    }while(!isfin);
	  }
	  //Stop
	  time_t stop_time=std::time(NULL);
	  monc.Stop(md);
	  
	  //Writing the dataset into the file
	  out.setTimes(start_time, stop_time);
	  std::cout<<"Final counts: "<<(long)md[0]<<" "<<(long)md[1]<<std::endl;
	  
	  std::vector<long> ch;
	  for(int j=0; j<2; j++) ch.push_back((long)md[j]);
	  out.addCounts((monc.getNames())[0],ch,0);
  
	  //Writing everything out
	  for(int i=0; i<4; i++) out.addMotor(motor[i]->getName(), pos[i], false, 0);
	  if(first) out.WriteNewSample(filename,true,1);
	  else out.WriteNewItem(true);
	  if(std::ifstream("stop"))
	  {
	    stop=true;
	    std::remove("stop");
	    std::cout<<"Scan stopped by user request"<<std::endl;
	  }
	  first=false;
	}
      }
      else if (ans==5)
      {
	oreg.Set(IOOperation::OutputRegister::VLV_CA_R | IOOperation::OutputRegister::VLV_CA_H);
	std::cout<<"Write any character and type ENTER to switch compressed air off: ";
	char smth;
	std::cin>>smth;
	oreg.Reset(IOOperation::OutputRegister::VLV_CA_R | IOOperation::OutputRegister::VLV_CA_H);
	no_out=true;
      }
      else no_out=false;
    }
    std::cout<<"Program finished"<<std::endl;


    return 0;
  }
  
  if(measurement_run)
  {
    //Interpreting the received task
    if((argc!=11)&&(argc!=3)&&(argc!=2))
    {
      std::cerr<<"This program start the measurement on MTEST and saves the results to the requested file"<<std::endl;
      std::cerr<<"Usage: ./mtest filename_wo_extension motor_name no_runs start_run_number initial from to step time_s sample_comment"<<std::endl;
      std::cerr<<"\t or move the motor into the requested position ./mtest motor_name move_to_angles"<<std::endl;
      std::cerr<<"\t or returns the current position of the motor ./mtest motor_name"<<std::endl;
      return 0;
    }
    if(argc==11)
    {
      int checktime=5;
      std::stringstream ss;
      std::string filename,sample_comment,motorname;
      int times,start_run_no,time;
      double first_angle, from, to, step;
      ss.str(argv[1]);
      ss>>filename;
      ss.clear();
      ss.str(argv[2]);
      ss>>motorname;
      ss.clear();
      ss.str(argv[3]);
      ss>>times;
      ss.clear();ss.str(argv[4]);
      ss>>start_run_no;
      ss.clear();ss.str(argv[5]);
      ss>>first_angle;
      ss.clear();ss.str(argv[6]);
      ss>>from;
      ss.clear();ss.str(argv[7]);
      ss>>to;
      ss.clear();ss.str(argv[8]);
      ss>>step;
      ss.clear();ss.str(argv[9]);
      ss>>time;
      ss.clear();ss.str(argv[10]);
      ss>>sample_comment;
      
      std::cout<<"Main filename: \""<<filename<<"\""<<std::endl;
      std::cout<<"Scan type: "<<motorname<<std::endl;
      std::cout<<times<<" number of runs will be performed, starting from "<<start_run_no<<std::endl;
      std::cout<<"Initial degree for each run is "<<first_angle<<"degrees"<<std::endl;
      std::cout<<"Each run consist of measurement recorded from "<<from<<" to "<<to<<" with "<<step<<" degrees steps in "<<motorname<<":"<<std::endl;
      int no_steps=int((to-from)/step);
      if (fabs(from+step*no_steps-from)>fabs(from+step*(no_steps+1)-from)) no_steps++;
      for(int i=0; i<no_steps+1; i++) std::cout<<from+step*i<<" ";
      std::cout<<std::endl;
      std::cout<<"Each measurement takes "<<time<<" seconds long"<<std::endl;
      std::cout<<"Sample comment: "<<sample_comment<<std::endl;
      
      //Initializing the camac controllers
      SerialCamac sc("camac_serial",1,"",0.5,9600);
      sc.SetResetInhibit(true);
      
      //Initialize IO
      IOOperation::OutputRegister oreg(sc,14);
      //IOOperation::InterruptRegister ireg(sc,13);
      
      //Initialize Motors
      //Positioner::StepMotorController smc(sc,4);
      uint32_t lo=0,hi=32768;
      int8_t module;
      if(motorname=="2Theta") module=17;//lo=33668;hi=47268;
      else if(motorname=="Omega") module=18;//lo=35268;hi=59968;
      else if(motorname=="Chi") module=19;//lo=32268;hi=51268;
      else if(motorname=="Phi") module=20;//lo=1268;hi=64268;
      else if(motorname=="X") module=20;//lo=0;//2768;hi=65534;//62768;
      else if(motorname=="Y") module=19;//lo=0;//5768;hi=65534;//59768;
      else
      {
        std::cerr<<"The requested motor does not exist!"<<std::endl;
        return -1;
      }
      //Save head information to the storage place
      MTESTRawData out;
      out.comment_sample=sample_comment;
      std::ifstream is("default");
      std::getline(is,out.comment_ms_collimator);
      std::cout<<"Monochromator-sample collimator: "<<out.comment_ms_collimator<<std::endl;
      std::getline(is,out.comment_ms_slits);
      std::cout<<"Monochromator-sample slits: "<<out.comment_ms_slits<<std::endl;
      std::getline(is,out.comment_s_holder);
      std::cout<<"Sample holder: "<<out.comment_s_holder<<std::endl;
      std::getline(is,out.comment_s_environment);
      std::cout<<"Sample environment: "<<out.comment_s_environment<<std::endl;
      std::getline(is,out.comment_sd_slits);
      std::cout<<"Sample-detector slits: "<<out.comment_sd_slits<<std::endl;
      std::getline(is,out.comment_d_position);
      std::cout<<"Detector position: "<<out.comment_d_position<<std::endl;
      is>>checktime;
      std::cout<<"Snapshot will be taken each "<<checktime<<"th seconds during the measurement"<<std::endl;
      std::string line;
      std::getline(is,line);
      std::getline(is,line);
      ss.clear(); ss.str(line);
      int no_motors;
      ss>>no_motors;
      std::cout<<"Number of standing motors: "<<no_motors-1<<" values reading from file"<<std::endl;
      bool moving_found=false;//Moving motor has been found
      for(int i=0; i<no_motors; i++)
      {
        std::getline(is,line);
        ss.clear(); ss.str(line);
        std::string line_name;
        int position;
        uint32_t hi_mot, lo_mot;
        ss>>line_name>>position>>lo_mot>>hi_mot;
        if(line_name!=motorname)
        {
            std::cout<<"Motor #"<<i+(moving_found ? 0 : 1)<<": "<<line<<std::endl;
            if(ss.rdbuf()->in_avail() ==0) out.addMotor(line_name,position, true, 0);
            else
            {
                double degrees;
                ss>>degrees;
                out.addMotor(line_name,position, true, &degrees);
            }
        }
        else
        {
            moving_found=true;
            lo=lo_mot;
            hi=hi_mot;
        }
      }
      if(!moving_found)
      {
        std::cout<<"The name of the motor \""<<motorname<<"\" is not present in the \"default\" file!"<<std::endl;
        return -1;
      }
      int no_modules, no_tubes;
      std::getline(is,line);
      ss.clear(); ss.str(line);
      ss>>no_modules>>no_tubes;
      std::vector<uint8_t> hard_lld(no_modules);//Hard low level discriminator belonging to each used modules
      std::cout<<"Number of Mesytec modules: "<<no_modules<<", no tubes: "<<no_tubes<<std::endl;
      for(int i=0; i<no_modules; i++)
      {
          int dat;
          ss>>dat;
          hard_lld[i]=static_cast<uint8_t>(dat);
          std::cout<<"Hard low level discriminator for module no "<<i<<": "<<static_cast<int>(hard_lld[i])<<std::endl;
      }
      std::vector<std::string> tube_names(no_tubes);//Name of the tubes
      std::array<std::array<bool,Mesy::NTUBE>,Mesy::NMPSD8> active;//Active tubes array
      std::vector<uint8_t> gain(no_tubes);//Gain belonging to each tubes
      std::vector<std::array<int,2>> discriminator(no_tubes);//Gain belonging to each tubes
      std::vector<uint8_t> hard_lld_each_tubes(no_tubes);//Tube version of hard lld
      for(int i=0; i<Mesy::NMPSD8; i++) (active[i]).fill(false);
      for(int i=0; i<no_tubes; i++)
      {
          std::getline(is,line);
          ss.clear(); ss.str(line);
          int module_no=0, tube_no=0;
          int g=0;
          ss>>module_no>>tube_no>>tube_names[i]>>g>>discriminator[i][0]>>discriminator[i][1];
          gain[i]=static_cast<uint8_t>(g);
          active[module_no][tube_no]=true;
          hard_lld_each_tubes[i] = hard_lld[module_no];
          std::cout<<"Tube #"<<i+1<<": module "<<module_no<<", tube "<<tube_no<<", name: \""<<tube_names[i]<<"\", gain: "<<static_cast<int>(gain[i])<<", software range: "<<discriminator[i][0]<<"..."<<discriminator[i][1]<<std::endl;
      }
      
      is.close();
      
      Positioner:: HarwellMotor tth(sc, motorname, module, hi, lo, oreg, 0);
      
      double hilim=tth.getALimit(true);
      double lolim=tth.getALimit(false);
      std::cout<<"The valid angular range for "<<motorname<<" is "<<lolim<<"..."<<hilim<<" degrees"<<std::endl;
      if((from<lolim)||(hilim<to))
      {
        std::cerr<<"ERROR: the requested range is out from the valid range!"<<std::endl;
        return 0;
      }
      
      //Initialize counters
      Scaler::MonitorCounter monc(std::string("Monitor"),sc, 8, false, (Scaler::MON_CNT) time, run_break_low_intensity, Scaler::MonitorCounter::DEFAULT_INTENSITY_NIVEAU, Scaler::MonitorCounter::DEFAULT_CONSEC, true,true);
      Scaler::PSDCounter psdc(std::string("ORDELA"),0.0,0.0,1024, 10, 1020,GATE_ANTICOINCIDENCE,1.6,634, 0.2, (uint16_t)2048);
      
      std::vector<std::string> ip={"192.168.168.121"};
      std::vector<int>         port={54321};
      std::vector<int>         port_data{56789};
      std::vector<int>         id={0};
      
      UDPClient up(54321,"192.168.168.1",ip,port,id);
      UDPServerTimeOut datasink(56789,ip,port_data,id,10);
      
      //Mesy::MesyDAQMTESTPowderCatchAll mdaq({{active}}, discriminator, filename, 0);
      Mesy::MesyDAQMTESTPowderGatedByMonitorPulse mdaq({{active}}, discriminator, Mesy::PAS_DIGITAL_1, 0, filename, true, 0);//Constructor: active_tubes (t: active, f:passive) for each possible [mcpd:0..hiindex][mpsd:0..<NMPSD][tube:0..<NTUBE] indices, amplitude discriminators (LOW,HIGH) for each active tubes, input line of the monitor start/stop signal, the mcpd module id, the parameter/event setup, finally the highest mcpd id
      Scaler::MesyCounter mesyc(tube_names, up, datasink, mdaq, hard_lld, gain, 0);
      
      
      
      double pos;
      if(monc.getCounterDimensions().size()!=1)
      {
        std::cout<<"ERROR: (commod at measurement_run): The monitor counter dimension is supposed to be 1 instead of "<<monc.getCounterDimensions().size()<<"!"<<std::endl;
        return -1;
      }
      if(psdc.getCounterDimensions().size()!=1)
      {
        std::cout<<"ERROR: (commod at measurement_run): The ORDELA psdcounter dimension is supposed to be 1 instead of "<<psdc.getCounterDimensions().size()<<"!"<<std::endl;
        return -1;
      }
      std::vector<std::string> mon_captions;
      mon_captions.push_back("Counts");
      mon_captions.push_back("Time-sec");
      std::vector<std::string> psd_captions;
      for(int i=0; i<psdc.getCounterNo(); i++) psd_captions.push_back(std::to_string(i+1));
      psd_captions.push_back("RealTime-s");
      psd_captions.push_back("LiveTime-s");
      out.addCounterHead((monc.getNames())[0], monc.getCounterNo(), 0, mon_captions);
      out.addCounterHead((psdc.getNames())[0], psdc.getCounterNo(), 2, psd_captions);
      std::vector<int> mesdims=mesyc.getCounterDimensions();
      std::vector<std::string> mesy_captions;
      for(int i=0; i<mesdims[2]; i++) mesy_captions.push_back(std::to_string(i+1));
      mesy_captions.push_back("Tube_gain");
      mesy_captions.push_back("Module_LLD");
      mesy_captions.push_back("Software_LLD");
      mesy_captions.push_back("Software_HLD");
      for(int i=0; i< static_cast<int>(mesyc.getNames().size()); i++) out.addCounterHead((mesyc.getNames())[i]+"_position", mesdims[2], 4, mesy_captions);
      for(int i=0; i< static_cast<int>(mesyc.getNames().size()); i++) out.addCounterHead((mesyc.getNames())[i]+"_amplitude", mesdims[2], 4, mesy_captions);
      
      Scaler::MON_CNT md[2];
      std::vector<long> ch;
      std::vector<double> add;
      double t[2];
      Scaler::PSD_CNT pd[1024];
      std::vector<Mesy::MESY_CNT> mes_data(mesyc.getCounterNo());
      Mesy::MESY_CNT *mesdata=&(mes_data[0]);
      
      bool new_set=true;
      for(int run=start_run_no; run<times+start_run_no; run++)
      {
        bool new_run=true;
        bool regular_move=false;
        for(int i=0; i<no_steps+1; i++)
        {
        bool new_item=true;
        bool stop=false;
        std::cout<<"------------------------------------------------------------"<<std::endl;
        //First task: move 2theta and update its position
        std::cout<<"Run number: "<<run<<", position number: "<<i+1<<std::endl;
        do
        {
        std::cout<<"Moving "<<motorname<<" to "<<(regular_move ? from+i*step : first_angle)<<" degrees"<<std::endl;
        if(tth.getAPosition()!=(regular_move ? from+i*step : first_angle))
        {
            if(!tth.moveATo(regular_move ? from+i*step : first_angle))
            {
                std::cerr<<"ERROR: at motor move"<<std::endl;
                return 0;
            }
        }
        while(!tth.isAllStopped()) 
        {
            std::cout<<"\tActual position: "<<tth.getAPosition()<<" degrees"<<std::endl;
            sleep(1);
            if(std::ifstream("stop"))
            {
            tth.Stop();
            std::remove("stop");
            std::cout<<"Program and motor stopped by user request"<<std::endl;
            
            stop=true;
            }
        }
        sleep(1);
        pos=tth.getAPosition();
        int posg=(int) tth.getPosition();
        out.addMotor(tth.getName(),posg, false, &pos);
        std::cout<<"\tFinal position: "<<pos<<" degrees"<<std::endl;
        update_default_file(tth.getName(), posg, true, pos, tth.getLimit(false), tth.getLimit(true));
        if(stop) return 0;
        }while(!regular_move++);
        std::cout<<"Start programming the detectors up..."<<std::endl;
        
        //Setup counters
        if(!monc.SetPreset((Scaler::MON_CNT) time)) std::cout<<"Error at setpreset of the monitor counter"<<std::endl;
        if(!psdc.Configure()) std::cout<<"Error at ORDELA configuration"<<std::endl;
        if(!mesyc.Configure()) std::cout<<"Error at Mesytec configuration"<<std::endl;
        if(!monc.Configure()) std::cout<<"Error at monitor configuration"<<std::endl;
        
        //Start
        std::cout<<"Start detectors"<<std::endl;
        if(!psdc.Start()) std::cout<<"Error at ORDELA start"<<std::endl;
        if(!mesyc.Start()) std::cout<<"Error at Mesytec start"<<std::endl;
        time_t start_time=std::time(NULL);
        if(!monc.Start()) std::cout<<"Error at monitor start"<<std::endl;
        std::cout<<"Start time: "<<Write_Time(start_time)<<std::endl;
        
        double remaining=checktime;
        double remaining2, remaining3=0.3;
        bool ispsd,ismes;
        bool isfin=false;
        int cnt=0;
        do
        {
            if(cnt!=0)sleep((checktime>remaining) ? remaining :checktime);
            if(std::ifstream("stop"))
            {
                stop=true;
                std::remove("stop");
                std::cout<<"Program stopped by user request"<<std::endl;
            }
            
            if(!stop)
            {
                if(monc.Read(md,remaining,isfin) && psdc.ReadPSD(pd,t,t+1,remaining2,ispsd) && mesyc.Read(mesdata,remaining3, ismes))
                {
                    time_t snap_time=std::time(NULL);
                    out.setTimes(start_time, snap_time);
                    ch.clear();
                    for(int j=0; j<monc.getCounterNo(); j++) ch.push_back((long)md[j]);
                    out.addCounts((monc.getNames())[0],ch,0);
                    ch.clear(); add.clear();
                    for(int j=0; j<psdc.getCounterNo(); j++) ch.push_back((long)pd[j]);
                    for(int j=0; j<2; j++) add.push_back(t[j]);
                    out.addCounts((psdc.getNames())[0],ch,&add);
                    for(int i=0; i<mesdims[0]*mesdims[1]; i++)
                    {
                        ch.clear(); add.clear();
                        for(int j=0; j<mesdims[2]; j++) ch.push_back((long)mesdata[i*mesdims[2]+j]);
                        add.push_back(static_cast<double>(gain[i%mesdims[0]]));
                        add.push_back(static_cast<double>(hard_lld_each_tubes[i%mesdims[0]]));
                        add.push_back(discriminator[i%mesdims[0]][0]);
                        add.push_back(discriminator[i%mesdims[0]][1]);
                        out.addCounts(std::string(mesyc.getNames()[i%mesdims[0]]+ ((i<mesdims[0]) ? "_position" : "_amplitude")), ch, &add);
                    }
                    if(new_set) out.WriteNewSample(filename,true,start_run_no);
                    else if (new_run) out.WriteNewRun(true);
                    else if (new_item) out.WriteNewItem(true);
                    else out.WriteNewSnapshot(true);
                    new_set=new_run=new_item=false;
                    pos=tth.getAPosition();
                    out.addMotor(tth.getName(),(int) tth.getPosition(), false, &pos);
                    std::cout<<"Run: "<<run<<", item: "<<i+1<<", snapshot "<<++cnt<<" is created at "<<Write_Time(snap_time)<<std::endl;
                }
                else std::cout<<"Something wrong.."<<std::endl;
                if(std::ifstream("stop"))
                {
                    stop=true;
                    std::remove("stop");
                    std::cout<<"Program stopped by user request"<<std::endl;
                }
            }
        }while(!(isfin||stop));
        //Stop
        time_t stop_time=std::time(NULL);
        monc.Stop(md);
        psdc.Stop(pd);
        mesyc.Stop(mesdata);
        
        //Writing the dataset into the file
        out.setTimes(start_time, stop_time);
        
        ch.clear();
        for(int j=0; j<monc.getCounterNo(); j++) ch.push_back((long)md[j]);
        out.addCounts((monc.getNames())[0],ch,0);
        ch.clear(); add.clear();
        for(int j=0; j<psdc.getCounterNo(); j++) ch.push_back((long)pd[j]);
        psdc.GetTimes(t,t+1);
        for(int j=0; j<2; j++) add.push_back(t[j]);
        out.addCounts((psdc.getNames())[0],ch,&add);
        for(int i=0; i<mesdims[0]*mesdims[1]; i++)
        {
            ch.clear(); add.clear();
            for(int j=0; j<mesdims[2]; j++) ch.push_back((long)mesdata[j+i*mesdims[2]]);
            add.push_back(static_cast<double>(gain[i%mesdims[0]]));
            add.push_back(static_cast<double>(hard_lld_each_tubes[i%mesdims[0]]));
            add.push_back(discriminator[i%mesdims[0]][0]);
            add.push_back(discriminator[i%mesdims[0]][1]);
            out.addCounts(std::string(mesyc.getNames()[i%mesdims[0]]+ ((i<mesdims[0]) ? "_position" : "_amplitude")), ch, &add);
        }
        if(new_set) out.WriteNewSample(filename,true,start_run_no);
        else if (new_run) out.WriteNewRun(true);
        else if (new_item) out.WriteNewItem(true);
        else out.WriteNewSnapshot(true);
        mesyc.WriteRaws(filename+"_"+std::to_string(run)+"_"+std::to_string(i+1));
        new_set=new_run=new_item=false;
        std::cout<<"Run: "<<run<<", item: "<<i+1<<", final "<<++cnt<<" is created at "<<Write_Time(stop_time)<<std::endl;
        if(std::ifstream("stop"))
        {
            stop=true;
            std::remove("stop");
            std::cout<<"Program stopped by user request"<<std::endl;
        }
        if(stop) return 0;
        }
      }
    }
    else
    {
      //Only get a position of one motor or move the motor
      std::stringstream ss;
      std::string motorname;
      ss.str(argv[1]);
      ss>>motorname;
      
      //Initializing the camac controllers
      SerialCamac sc("camac_serial",1,"",0.5,9600);
      
      //Initialize IO
      IOOperation::OutputRegister oreg(sc,14);
      
      //Initialize Motors
      uint32_t lo=0,hi=32768;
      int8_t module;
      if(motorname=="2Theta") module=17;//lo=33668;hi=47268;
      else if(motorname=="Omega") module=18;//lo=35268;hi=49968;
      else if(motorname=="Chi") module=19;//lo=32268;hi=51268;
      else if(motorname=="Phi") module=20;//lo=1268;hi=64268;
      else if(motorname=="X") module=20;//lo=0;//2768;hi=65534;//62768;
      else if(motorname=="Y") module=19;//lo=0;//5768;hi=65534;//59768;
      else
      {
        std::cerr<<"The requested motor does not exist!"<<std::endl;
        return -1;
      }
      
      //Get the limits written in the default file
      std::ifstream is("default");
      std::string line;
      for(int i=0; i<8; i++) std::getline(is,line);
      ss.clear(); ss.str(line);
      int no_motors;
      ss>>no_motors;
      for(int i=0; i<no_motors; i++)
      {
        std::getline(is,line);
        ss.clear(); ss.str(line);
        std::string line_name;
        int position;
        uint32_t hi_mot, lo_mot;
        ss>>line_name>>position>>lo_mot>>hi_mot;
        if(line_name==motorname)
        {
            lo=lo_mot;
            hi=hi_mot;
        }
      }
      is.close();

      Positioner:: HarwellMotor tth(sc, motorname, module, hi, lo, oreg, 0);
      
      double hilim=tth.getALimit(true);
      double lolim=tth.getALimit(false);
      std::cout<<"The valid angular range for "<<motorname<<" is "<<lolim<<"..."<<hilim<<" degrees"<<std::endl;
      std::cout<<"Actual position: "<<tth.getAPosition()<<" degrees"<<std::endl;
      if (argc==2) update_default_file(tth.getName(), tth.getPosition(), true, tth.getAPosition(), tth.getLimit(false), tth.getLimit(true));
      else
      {
        //Move the motor into a given position
        double to;
        ss.str(argv[1]);
        ss.clear();ss.str(argv[2]);
        ss>>to;

        if((to<lolim)||(hilim<to))
        {
            std::cerr<<"ERROR: the requested value is out from the valid range!"<<std::endl;
            return 0;
        }
        
        std::cout<<"------------------------------------------------------------"<<std::endl;
        //First task: move 2theta and update its position
        std::cout<<"Moving "<<motorname<<" to "<<to<<" degrees"<<std::endl;
        if(tth.getAPosition()!=to)
        {
            if(!tth.moveATo(to))
            {
                std::cerr<<"ERROR: at motor move"<<std::endl;
                return 0;
            }
        }
        bool stop=false;
        while(!tth.isAllStopped()) 
        {
            std::cout<<"\tActual position: "<<tth.getAPosition()<<" degrees"<<std::endl;
            sleep(1);
            if(std::ifstream("stop"))
            {
            tth.Stop();
            std::remove("stop");
            std::cout<<"Program and motor stopped by user request"<<std::endl;
            stop=true;
            }
        }
        sleep(1);
        double pos=tth.getAPosition();
        int posg=(int) tth.getPosition();
        update_default_file(tth.getName(), posg, true, pos, tth.getLimit(false), tth.getLimit(true));
        if(stop) return 0;
        std::cout<<"\tFinal position: "<<pos<<" degrees"<<std::endl;
      }
      
    }
    std::cout<<"Program normal end"<<std::endl;
    return 0;
  }
  
  if(save_data)
  {
   MTESTRawData data;
   data.comment_sample="E";//Comments about the sample
   data.comment_ms_collimator="Z";//Comment on collimator setup applied between the monochromator and sample
   data.comment_ms_slits=" ";//Comment on the setup of slits applied between the monochromator and sample
   data.comment_s_holder="E";//Comment on the sample holder
   data.comment_s_environment="G";//Comment on the applied sample environment
   data.comment_sd_slits="Y";//Comment on the setup of slits applied between the sample and the detector
   data.comment_d_position=".";//Comment on the position of the detector
   double value=0.56;
   data.addMotor("2Theta",568,false,&value);
   value=1.56;
   data.addMotor("Phi",1568,true,&value);
   data.addMotor("DArm",1568,true,0);
   data.addMotor("MTilt",1568,false,0);
   std::vector<std::string> names,names2;
   names.push_back("Time");
   names.push_back("Preset");
   for(int i=0;i<3; i++) names2.push_back(std::to_string(i+1));
   names2.push_back("Real_time");
   names2.push_back("Live_time");
   data.addCounterHead("MonitorCounter", 2, 0, names);
   data.addCounterHead("ORDELACounter", 3, 2, names2);
   std::vector<long> dataset_m;
   dataset_m.push_back(203);
   dataset_m.push_back(1489);
   data.addCounts("MonitorCounter", dataset_m,0);
   std::vector<long> dataset_psd;
   dataset_psd.push_back(120);
   dataset_psd.push_back(140);
   dataset_psd.push_back(150);
   std::vector<double> addit={100.1, 100.0};
   data.addCounts("ORDELACounter", dataset_psd,&addit);
   time_t vals[2];
   vals[0]=std::time(NULL);
   sleep(2);
   vals[1]=std::time(NULL);
   data.setTimes(vals[0], vals[1]);
   
   data.WriteNewSample("ki",true);
 
    value=1.56;
    data.addMotor("2Theta",1568,false,&value);
    data.addMotor("MTilt",2568,false,0);
    dataset_psd.clear();
    dataset_m.clear();
    dataset_m.push_back(204);
    dataset_m.push_back(1490);
    dataset_psd.push_back(121);
    dataset_psd.push_back(141);
    dataset_psd.push_back(151);
    addit[0]=50.0;
    addit[1]=45.0;
    data.addCounts("MonitorCounter", dataset_m,0);
    data.addCounts("ORDELACounter", dataset_psd,&addit);
    vals[0]=std::time(NULL);
    sleep(2);
    vals[1]=std::time(NULL);
    data.setTimes(vals[0], vals[1]);
    data.WriteNewRun(true);
  
   data.clear_comments();
   data.clear_motors(false);
   data.clear_counters(false);
   data.clear_time();
   
   std::ifstream is("ki.dat");
   data.read(is);
   is.close();
   
   std::cout<<"sample: \""<<data.comment_sample<<"\", msc: \""<<data.comment_ms_collimator<<"\", mss: \""<<data.comment_ms_slits<<"\", s_h: \""<<data.comment_s_holder<<"\", se: \""<<data.comment_s_environment<<"\", sds: \""<<data.comment_sd_slits<<"\", dpos:\""<<data.comment_d_position<<"\""<<std::endl;
   
   for(int i=0; i<(int)data.motors.size(); i++)
   {
     std::cout<<"Motor name: "<<(data.motors[i]).name<<std::endl;
     std::cout<<"\t"<<(((data.motors[i]).standing) ? "STATIONER" : "SCANNING")<<std::endl;
     std::cout<<"\t"<<(((data.motors[i]).only_pulses) ? "ONLY PULSES" : "PULSES & ANGLES")<<std::endl;
     std::cout<<"\t Number of entries: "<<(int)((data.motors[i]).pos_pulses.size())<<" "<<(int)((data.motors[i]).pos_angles.size())<<std::endl;
     for(int j=0; j<(int)((data.motors[i]).pos_pulses.size()); j++)
     {
       std::cout<<"\t"<<(data.motors[i]).pos_pulses[j];
       if(!(data.motors[i]).only_pulses) std::cout<<" "<<(data.motors[i]).pos_angles[j];
       std::cout<<std::endl;
     }
   }
   
   for(int i=0; i<(int)data.counters.size(); i++)
   {
     std::cout<<"Counter name: "<<(data.counters[i]).name<<std::endl;
     std::cout<<"\t Number of channels: "<<(data.counters[i]).no_channels<<", additionals "<<(data.counters[i]).no_additionals<<std::endl;
     std::cout<<"\t Number of entries: "<<(int)((data.counters[i]).count_data.size())<<" "<<(int)((data.counters[i]).additionals.size())<<" "<<(int)((data.counters[i]).caption.size())<<std::endl;
     for(int j=0; j<(int)((data.counters[i]).count_data.size())/(data.counters[i]).no_channels; j++)
     {
       std::cout<<"\tDataset: "<<j+1<<std::endl;
       for(int k=0; k<(data.counters[i]).no_channels; k++) std::cout<<"\t"<<(data.counters[i]).caption[k]<<" "<<(data.counters[i]).count_data[k+j*(data.counters[i]).no_channels]<<std::endl;
       for(int k=0; k<(data.counters[i]).no_additionals; k++) std::cout<<"\t"<<(data.counters[i]).caption[k+(data.counters[i]).no_channels]<<" "<<(data.counters[i]).additionals[k+j*(data.counters[i]).no_additionals]<<std::endl;
     }
   }
   
   for(int i=0; i<(int)data.start_time.size(); i++)
   {
     std::cout<<"Start: "<<Write_Time(data.start_time[i])<<" end: "<<Write_Time(data.end_time[i])<<std::endl;
   }
   
   return 0;
  }
  
  if(commandcontrol)
  {
    MTESTControl mt;
    std::ifstream ifile("config_file.dat");
    std::cout<<"Read cfg file: "<<(mt.readCfgFile(ifile) ? "OK" : "Fail")<<std::endl;
    std::ofstream ofile("config_file_out.dat");
    std::cout<<"Write cfg file: "<<(mt.writeCfgFile(ofile) ? "OK" : "Fail")<<std::endl;
  }
  
  if(psdmonitorcombined)
  {
    //This is a test for combined working
    
    SerialCamac sc("camac_serial",1,"",0.5,9600);
    std::cout<<"Assert inhibit: "<<(sc.SetResetInhibit(false)? "OK" : "Error")<<std::endl;
    int up=233;
    int down=228;
    std::cout<<"Please, choose gate setup:\n\t0=GATE_OFF\n\t1=GATE_COINCIDENCE\n\t2=GATE_ANTICOINCIDENCE"<<std::endl;
    int ans;
    std::cin>>ans;
    GATE_MODE gm;
    if(ans==0) gm=GATE_OFF;
    else gm=((ans==1) ? GATE_COINCIDENCE : GATE_ANTICOINCIDENCE);
    Scaler::MonitorCounter monc(std::string("Monitor"),sc, 8, false, (Scaler::MON_CNT) 10, false, Scaler::MonitorCounter::DEFAULT_INTENSITY_NIVEAU, Scaler::MonitorCounter::DEFAULT_CONSEC, false,true);
    Scaler::PSDCounter psdc(std::string("ORDELA"),0.0,0.0,256, 5, 254,gm,1.6,-1, .1, 2048);
    std::cout<<"Counters are initialized, configure devices: "<<psdc.Configure()<<" "<<monc.Configure()<<std::endl;
    Scaler::PSD_CNT* pcnt = new  Scaler::PSD_CNT [psdc.getCounterNo()];
    Scaler::MON_CNT* mcnt = new  Scaler::MON_CNT [monc.getCounterNo()];
    double rem=0.0;
    bool isfin=false;
    std::cout<<"Actual gate mode: "<<psdc.GetGate()<<std::endl;
    std::cout<<"Try to read from ORDELA: "<<psdc.Read(pcnt, rem, isfin);
    std::cout<<"Remaining: "<<rem<<", finished?: "<<isfin<<std::endl;
    std::cout<<"Content of ORDELA: "<<std::endl;
    for(int i=down; i<up; i++) std::cout<<"["<<i<<"]: "<<pcnt[i]<<std::endl;
    std::cout<<"Try to read from monitor: "<<monc.Read(mcnt, rem, isfin);
    std::cout<<"Remaining: "<<rem<<", finished?: "<<isfin<<", counts: "<<mcnt[0]<<" "<<mcnt[1]<<std::endl;
    std::cout<<"Start ORDELA: "<<psdc.Start();
    std::cout<<"Wait 5 secs... "<<std::endl;
    sleep(5);
    std::cout<<"Try to read from ORDELA: "<<psdc.Read(pcnt, rem, isfin);
    std::cout<<"Remaining: "<<rem<<", finished?: "<<isfin<<std::endl;
    std::cout<<"Content of ORDELA: "<<std::endl;
    for(int i=down; i<up; i++) std::cout<<"["<<i<<"]: "<<pcnt[i]<<std::endl;
    std::cout<<"Start monitor: "<<monc.Start();
    std::cout<<"Release inhibit: "<<(sc.SetResetInhibit(true)? "OK" : "Error")<<std::endl;
    std::cout<<"Wait 5 secs... "<<std::endl;
    sleep(5);
    std::cout<<"Try to read from monitor: "<<monc.Read(mcnt, rem, isfin);
    std::cout<<"Remaining: "<<rem<<", finished?: "<<isfin<<", counts: "<<mcnt[0]<<" "<<mcnt[1]<<std::endl;
    std::cout<<"Try to read from ORDELA: "<<psdc.Read(pcnt, rem, isfin);
    std::cout<<"Remaining: "<<rem<<", finished?: "<<isfin<<std::endl;
    std::cout<<"Content of ORDELA: "<<std::endl;
    for(int i=down; i<up; i++) std::cout<<"["<<i<<"]: "<<pcnt[i]<<std::endl;
    std::cout<<"Wait 5 secs... "<<std::endl;
    sleep(5);
    std::cout<<"Assert inhibit: "<<(sc.SetResetInhibit(false)? "OK" : "Error")<<std::endl;
    std::cout<<"Stop ORDELA: "<<psdc.Stop(pcnt)<<", STOP monitor: "<<monc.Stop(mcnt)<<std::endl;
    std::cout<<"Monitor counts: "<<mcnt[0]<<" "<<mcnt[1]<<std::endl;
    std::cout<<"Content of ORDELA: "<<std::endl;
    for(int i=down; i<up; i++) std::cout<<"["<<i<<"]: "<<pcnt[i]<<std::endl;
    std::cout<<"Wait 1 sec... "<<std::endl;
    sleep(1);
    std::cout<<"Try to read from monitor: "<<monc.Read(mcnt, rem, isfin);
    std::cout<<"Remaining: "<<rem<<", finished?: "<<isfin<<", counts: "<<mcnt[0]<<" "<<mcnt[1]<<std::endl;
    std::cout<<"Try to read from ORDELA: "<<psdc.Read(pcnt, rem, isfin);
    std::cout<<"Remaining: "<<rem<<", finished?: "<<isfin<<std::endl;
    std::cout<<"Content of ORDELA: "<<std::endl;
    for(int i=down; i<up; i++) std::cout<<"["<<i<<"]: "<<pcnt[i]<<std::endl;
    delete [] mcnt;
    delete [] pcnt;
  }
  

  if(rtc_counter)
  {
    SerialCamac sc("camac_serial",1,"",0.5,9600);
    bool isfinished=false;
    
    Scaler::RTC_CNT initial_preset=0;//Scaler::RTCCounter::MAX_PRESET;
    Scaler::RTCCounter::DIVISOR freq_div=Scaler::RTCCounter::DIV_1;
    bool lam_in_use=true;
    Scaler::RTC_CNT counts=0;
    double remaining=-1.0;
    Scaler::RTCCounter rtc(std::string("Timer"), sc, 16, initial_preset, freq_div, lam_in_use);
    
    std::cout<<"Configure: "<<rtc.Configure()<<std::endl;
    //std::cout<<"Read: "<<hc.Read(&counts,remaining,isfinished)<<std::endl;
    //std::cout<<"Counts: "<<counts<<", remaining: "<<remaining<<", "<<(isfinished ? "OFF" : "ON")<<std::endl;
    //std::cout<<"Getting the Busy status: "<<rtc.GetBusyStatus()<<std::endl;
    //sleep(10);
    std::cout<<"Start: "<<rtc.Start()<<std::endl;
    std::cout<<"Read: "<<rtc.Read(&counts,remaining,isfinished)<<std::endl;
    std::cout<<"Counts: "<<counts<<", remaining: "<<remaining<<", "<<(isfinished ? "OFF" : "ON")<<std::endl;
    sleep(10);
    //std::cout<<"Getting the Busy status: "<<rtc.GetBusyStatus()<<std::endl;
    std::cout<<"Read: "<<rtc.Read(&counts,remaining,isfinished)<<std::endl;
    std::cout<<"Counts: "<<counts<<", remaining: "<<remaining<<", "<<(isfinished ? "OFF" : "ON")<<std::endl;
    //std::cout<<"Decrease number of counts by 1: "<<hc.StepDownPreset()<<std::endl;
    sleep(10);
    //std::cout<<"Decrease number of counts by 1: "<<hc.StepDownPreset()<<std::endl;
    std::cout<<"Stop: "<<rtc.Stop(&counts)<<std::endl;
    std::cout<<"Counts: "<<counts<<std::endl;
    std::cout<<"Read: "<<rtc.Read(&counts,remaining,isfinished)<<std::endl;
    std::cout<<"Counts: "<<counts<<", remaining: "<<remaining<<", "<<(isfinished ? "OFF" : "ON")<<std::endl;
    sleep(5);
    rtc.SetFrequencyDivisor(freq_div);
    std::cout<<"Restart with different preset : "<<rtc.SetPreset(20)<<", and program: "<<", and using different lam: "<<rtc.SetLAMUse(!lam_in_use)<<std::endl;
    std::cout<<"Configure: "<<rtc.Configure()<<std::endl;
    std::cout<<"Start: "<<rtc.Start()<<std::endl;
    sleep(10);
    std::cout<<"Read: "<<rtc.Read(&counts,remaining,isfinished)<<std::endl;
    std::cout<<"Counts: "<<counts<<", remaining: "<<remaining<<", "<<(isfinished ? "OFF" : "ON")<<std::endl;
    sleep(10);
    std::cout<<"Stop: "<<rtc.Stop(&counts)<<std::endl;
    std::cout<<"Counts: "<<counts<<std::endl;
    std::cout<<"Read: "<<rtc.Read(&counts,remaining,isfinished)<<std::endl;
    std::cout<<"Counts: "<<counts<<", remaining: "<<remaining<<", "<<(isfinished ? "OFF" : "ON")<<std::endl;
    
  }
  
  if(hw_counter)
  {
    SerialCamac sc("camac_serial",1,"",0.5,9600);
    Scaler::HW_CNT initial_preset=0xffff;
    Scaler::HW_CNT counts=58;
    double remaining=-1.0;
    uint16_t control_register= Scaler::HarwellCounter::A0NOW_C| Scaler::HarwellCounter::A1NXTINP_C| Scaler::HarwellCounter::A0NXTCRRY_C;
    bool isfinished=false;
    bool lam_in_use=false;
    Scaler::HarwellCounter hc(std::string("Pointdetector"),sc,16, initial_preset, control_register, lam_in_use);
    std::cout<<"Configure: "<<hc.Configure()<<std::endl;
    //std::cout<<"Read: "<<hc.Read(&counts,remaining,isfinished)<<std::endl;
    //std::cout<<"Counts: "<<counts<<", remaining: "<<remaining<<", "<<(isfinished ? "OFF" : "ON")<<std::endl;
    //std::cout<<"Getting the level of A: "<<hc.GetALevel()<<std::endl;
    //sleep(10);
    std::cout<<"Start: "<<hc.Start()<<std::endl;
    sleep(10);
    //std::cout<<"Getting the level of A: "<<hc.GetALevel()<<std::endl;
    std::cout<<"Read: "<<hc.Read(&counts,remaining,isfinished)<<std::endl;
    std::cout<<"Counts: "<<counts<<", remaining: "<<remaining<<", "<<(isfinished ? "OFF" : "ON")<<std::endl;
    //std::cout<<"Decrease number of counts by 1: "<<hc.StepDownPreset()<<std::endl;
    sleep(10);
    //std::cout<<"Decrease number of counts by 1: "<<hc.StepDownPreset()<<std::endl;
    std::cout<<"Stop: "<<hc.Stop(&counts)<<std::endl;
    std::cout<<"Counts: "<<counts<<std::endl;
    std::cout<<"Read: "<<hc.Read(&counts,remaining,isfinished)<<std::endl;
    std::cout<<"Counts: "<<counts<<", remaining: "<<remaining<<", "<<(isfinished ? "OFF" : "ON")<<std::endl;
    sleep(5);
    control_register=Scaler::HarwellCounter::A1NOW_C| Scaler::HarwellCounter::A0NXTINP_C| Scaler::HarwellCounter::A1NXTCRRY_C;
    std::cout<<"Restart with different preset : "<<hc.SetPreset(20)<<", and program: "<<hc.SetControlRegister(control_register)<<", and using different lam: "<<hc.SetLAMUse(!lam_in_use)<<std::endl;
    std::cout<<"Configure: "<<hc.Configure()<<std::endl;
    std::cout<<"Start: "<<hc.Start()<<std::endl;
    sleep(10);
    std::cout<<"Read: "<<hc.Read(&counts,remaining,isfinished)<<std::endl;
    std::cout<<"Counts: "<<counts<<", remaining: "<<remaining<<", "<<(isfinished ? "OFF" : "ON")<<std::endl;
    sleep(10);
    std::cout<<"Stop: "<<hc.Stop(&counts)<<std::endl;
    std::cout<<"Counts: "<<counts<<std::endl;
    std::cout<<"Read: "<<hc.Read(&counts,remaining,isfinished)<<std::endl;
    std::cout<<"Counts: "<<counts<<", remaining: "<<remaining<<", "<<(isfinished ? "OFF" : "ON")<<std::endl;   
  }

  
  //Monitor
  if(monitor)
  {
    SerialCamac sc("camac_serial",1,"",0.5,9600);
    bool count_or_time_preset=false;
    Scaler::MON_CNT initial_preset=10;
    Scaler::MON_CNT counts[2]={0,0};
    double remaining=0.0;
    bool isfin=true;
    bool intensity_monitoring=true;
    uint16_t low_intensity_warning_level=Scaler::MonitorCounter::DEFAULT_INTENSITY_NIVEAU;
    uint8_t times_below_low_before_warning=Scaler::MonitorCounter::DEFAULT_CONSEC;
    bool warning_suspend=false;
    bool lam_in_use=false;
    Scaler::MonitorCounter monc(std::string("Monitor"),sc, 8, count_or_time_preset, initial_preset, intensity_monitoring, low_intensity_warning_level, times_below_low_before_warning, warning_suspend, lam_in_use);
    double real_preset=100.0;
    double live_preset=90.0;
    int channels=1024;
    int discriminator[2]={10,1014};
    GATE_MODE gm=GATE_ANTICOINCIDENCE;
    double dead_delay=1.6;
    int serialno=-1;
    double timeout=0.1;
    uint16_t adc_offset=2048;
    std::string firmware("valami");
    Scaler::PSDCounter psdc(std::string("ORDELA"), real_preset, live_preset, channels,discriminator[0],discriminator[1],gm,dead_delay,serialno,timeout,adc_offset);
    psdc.Configure();
    std::cout<<"Actual status: "<<monc.GetStatus()<<std::endl;
    std::cout<<"Try to configure... "<<monc.Configure()<<std::endl;
    std::cout<<"Try to read... "<<monc.Read(counts,remaining,isfin);
    std::cout<<"Counts: "<<counts[0]<<","<<counts[1]<<", remaining: "<<remaining<<", finished?  "<<(isfin ? ", finished" : ", ON")<<std::endl;
    std::cout<<"\tGATE_MODE GetGate() is actually: ";
    std::cout<<psdc.GetGate()<<std::endl;
    psdc.Start();
    std::cout<<"Try to start... "<<monc.Start()<<std::endl;
    sleep(4);
    std::cout<<"Try to read... "<<monc.Read(counts,remaining,isfin)<<std::endl;
    std::cout<<"Counts: "<<counts[0]<<","<<counts[1]<<", remaining: "<<remaining<<", finished?  "<<(isfin ? ", finished" : ", ON")<<std::endl;
    sleep(8);
    std::cout<<"Try to read... "<<monc.Read(counts,remaining,isfin)<<std::endl;
    std::cout<<"Counts: "<<counts[0]<<","<<counts[1]<<", remaining: "<<remaining<<", finished?  "<<(isfin ? ", finished" : ", ON")<<std::endl;
    std::cout<<"Try to stop... "<<monc.Stop(counts)<<std::endl;
    uint32_t *cnts=new uint32_t [psdc.getCounterNo()];
    for(int i=0; i<psdc.getCounterNo(); i++) cnts[i]=20;
    psdc.Stop(cnts);
    for(int i=0; i<10; i++) std::cout<<"i="<<i<<", "<<cnts[i]<<std::endl;
    std::cout<<"\tActual times: ";
    psdc.GetTimes(&real_preset,&live_preset);
    std::cout<<"real: "<<real_preset<<", live: "<<live_preset<<std::endl;

    std::cout<<"Counts: "<<counts[0]<<","<<counts[1]<<", remaining: "<<remaining<<", finished?  "<<(isfin ? ", finished" : ", ON")<<std::endl;
    std::cout<<"Try to read... "<<monc.Read(counts,remaining,isfin)<<std::endl;
    std::cout<<"Counts: "<<counts[0]<<","<<counts[1]<<", remaining: "<<remaining<<", finished?  "<<(isfin ? ", finished" : ", ON")<<std::endl;
    std::cout<<"----------------------------------Reconfigure-------------------------------------------------------------"<<std::endl;
    std::cout<<"Actual status: "<<monc.GetStatus()<<std::endl;
    std::cout<<"Try to configure... "<<monc.Configure()<<std::endl;
    std::cout<<"Try to read... "<<monc.Read(counts,remaining,isfin);
    std::cout<<"Counts: "<<counts[0]<<","<<counts[1]<<", remaining: "<<remaining<<", finished?  "<<(isfin ? ", finished" : ", ON")<<std::endl;
    std::cout<<"Try to start... "<<monc.Start()<<std::endl;
    sleep(5);
    std::cout<<"Try to read... "<<monc.Read(counts,remaining,isfin)<<std::endl;
    std::cout<<"Counts: "<<counts[0]<<","<<counts[1]<<", remaining: "<<remaining<<", finished?  "<<(isfin ? ", finished" : ", ON")<<std::endl;
    sleep(3);
    std::cout<<"Try to read... "<<monc.Read(counts,remaining,isfin)<<std::endl;
    std::cout<<"Counts: "<<counts[0]<<","<<counts[1]<<", remaining: "<<remaining<<", finished?  "<<(isfin ? ", finished" : ", ON")<<std::endl;
    std::cout<<"Try to stop... "<<monc.Stop(counts)<<std::endl;
    std::cout<<"Counts: "<<counts[0]<<","<<counts[1]<<", remaining: "<<remaining<<", finished?  "<<(isfin ? ", finished" : ", ON")<<std::endl;
    std::cout<<"Try to read... "<<monc.Read(counts,remaining,isfin)<<std::endl;
    std::cout<<"Counts: "<<counts[0]<<","<<counts[1]<<", remaining: "<<remaining<<", finished?  "<<(isfin ? ", finished" : ", ON")<<std::endl;
    
  }
  
  
  //PSD
  if(psd)
  {
    double real_preset=100.0;
    double live_preset=90.0;
    int channels=1024;
    int discriminator[2]={10,1014};
    GATE_MODE gm=GATE_ANTICOINCIDENCE;
    double dead_delay=1.6;
    int serialno=-1;
    double timeout=0.1;
    uint16_t adc_offset=2048;
    std::string firmware("valami");
    
    Scaler::PSDCounter psdc(std::string("ORDELA"),real_preset, live_preset, channels,discriminator[0],discriminator[1],gm,dead_delay,serialno,timeout,adc_offset);//Construct the object
    std::cout<<"PSDCounter object created\nBelow, the check of basic variables will be done:"<<std::endl;
    std::cout<<"\tbool GetPresets("<<real_preset<<","<<live_preset<<") is actually: ";
    std::cout<<(psdc.GetPresets(&real_preset,&live_preset) ? "OK ": "ERROR!");
    std::cout<<"values are ("<<real_preset<<","<<live_preset<<")"<<std::endl;
    std::cout<<"\tbool GetChannel("<<channels<<","<<discriminator[0]<<","<<discriminator[1]<<") is actually: ";
    std::cout<<(psdc.GetChannel(&channels,discriminator,discriminator+1) ? "OK ": "ERROR!");
    std::cout<<"values are ("<<channels<<","<<discriminator[0]<<","<<discriminator[1]<<")"<<std::endl;
    std::cout<<"\tGATE_MODE GetGate()="<<gm<<" is actually: ";
    std::cout<<psdc.GetGate()<<std::endl;
    std::cout<<"++++getting some informations about the system++++: "<<std::endl;
    std::cout<<"\tActual status: "<<psdc.GetStatus()<<std::endl;
    std::cout<<"\tNumber of counters: "<<psdc.getCounterNo()<<std::endl;
    std::cout<<"\tActual times: ";
    psdc.GetTimes(&real_preset,&live_preset);
    std::cout<<"real: "<<real_preset<<", live: "<<live_preset<<std::endl;
    std::cout<<"\tAbout the card: ";
    std::cout<<(psdc.About(&serialno,&firmware) ? "OK" : "ERROR!");
    std::cout<<", serial no: "<<serialno<<", firmware: "<<firmware<<std::endl;
    
    real_preset=200.046; live_preset=0.0;
    channels=256;
    discriminator[0]=2;
    discriminator[1]=254;
    gm=GATE_COINCIDENCE;
    std::cout<<"++++modifying the configuration+++++: "<<std::endl;
    std::cout<<"\tbool ChangePresets("<<real_preset<<","<<live_preset<<"): ";
    std::cout<<(psdc.ChangePresets(real_preset,live_preset) ? "OK ": "ERROR!")<<std::endl;
    std::cout<<"\tbool GetPresets("<<real_preset<<","<<live_preset<<") is actually: ";
    std::cout<<(psdc.GetPresets(&real_preset,&live_preset) ? "OK ": "ERROR!");
    std::cout<<"values are ("<<real_preset<<","<<live_preset<<")"<<std::endl;
    std::cout<<"\tbool ChangeChannel("<<channels<<","<<discriminator[0]<<","<<discriminator[1]<<"):";
    std::cout<<(psdc.ChangeChannel(channels,discriminator[0],discriminator[1]) ? "OK ": "ERROR!")<<std::endl;
    std::cout<<"\tbool GetChannel("<<channels<<","<<discriminator[0]<<","<<discriminator[1]<<") is actually: ";
    std::cout<<(psdc.GetChannel(&channels,discriminator,discriminator+1) ? "OK ": "ERROR!");
    std::cout<<"values are ("<<channels<<","<<discriminator[0]<<","<<discriminator[1]<<")"<<std::endl;
    std::cout<<"\tbool ChangeGate("<<gm<<"): ";
    std::cout<<(psdc.ChangeGate(gm) ? "OK ": "ERROR!");
    std::cout<<"\tGATE_MODE GetGate() is actually: ";
    std::cout<<psdc.GetGate()<<std::endl;
    std::cout<<"++++Testing responses to improper operation+++:"<<std::endl;
    std::cout<<"\tChangePresets(-1 numbers): "<<psdc.ChangePresets(-1.0,1.0)<<" "<<psdc.ChangePresets(1.0,-1.0)<<std::endl;
    std::cout<<"\tChangeChannels(inappropriate channel numbers): "<<psdc.ChangeChannel(-1,discriminator[0],discriminator[1])<<" "<<psdc.ChangeChannel(0,discriminator[0],discriminator[1])<<" "<<psdc.ChangeChannel(1,discriminator[0],discriminator[1])<<" "<<psdc.ChangeChannel(4096,discriminator[0],discriminator[1])<<std::endl;
    std::cout<<"\tChangeChannels(inappropriate discriminators): "<<psdc.ChangeChannel(1024,-1,2)<<" "<<psdc.ChangeChannel(1024,3,2)<<" "<<psdc.ChangeChannel(1024,3,-1)<<" "<<psdc.ChangeChannel(1024,2,1025)<<std::endl;
    Scaler::PSD_CNT *counts=new Scaler::PSD_CNT [psdc.getCounterNo()];
    for(int i=0; i<psdc.getCounterNo(); i++) counts[i]=20;
    double remaining=-1.0;
    bool isfin=true;
    std::cout<<"++++++++++Testing the read module++++++++++"<<std::endl;
    std::cout<<"\tRead the dataset in: "<<(psdc.Read(counts, remaining, isfin) ? "OK" : "Error!");
    std::cout<<", remaining: "<<remaining<<", isfin?: "<<isfin<<std::endl;
    for(int i=0; i<256; i++) std::cout<<"i="<<i<<", "<<counts[i]<<std::endl;
    std::cout<<"\tConfigure device: "<<psdc.Configure()<<std::endl;
    std::cout<<"\tSet counts to be 30, real time 20, live time 10: "<<(psdc.TestWriteTimeData(30, 20.0, 10.0) ? "OK" : "Error!");
    std::cout<<"\tRead the dataset in: "<<(psdc.Read(counts, remaining, isfin) ? "OK" : "Error!");
    std::cout<<", remaining: "<<remaining<<", isfin?: "<<isfin<<std::endl;
    for(int i=0; i<256; i++) std::cout<<"i="<<i<<", "<<counts[i]<<std::endl;
    std::cout<<"\tActual times: ";
    psdc.GetTimes(&real_preset,&live_preset);
    std::cout<<"real: "<<real_preset<<", live: "<<live_preset<<std::endl;
    for(int i=0; i<psdc.getCounterNo(); i++) counts[i]=0;
    std::cout<<"\tRead the dataset in using PSDRead: "<<(psdc.ReadPSD(counts, &real_preset, &live_preset, remaining, isfin) ? "OK" : "Error!");
    std::cout<<", real preset: "<<real_preset<<", live: "<<live_preset<<", remaining: "<<remaining<<", isfin?: "<<isfin<<std::endl;
    for(int i=0; i<256; i++) std::cout<<"i="<<i<<", "<<counts[i]<<std::endl;
    std::cout<<"Start acquisition: ";
    std::cout<<(psdc.Start() ? "OK" : "Error")<<std::endl;
    for(int i=0; i<10; i++)
    {
      sleep(1.0);
      std::cout<<"i="<<i<<std::endl;
      if((i%2)==0)
      {
	std::cout<<"\tRead the dataset in using PSDRead: "<<(psdc.ReadPSD(counts, &real_preset, &live_preset, remaining, isfin) ? "OK" : "Error!");
	std::cout<<", real preset: "<<real_preset<<", live: "<<live_preset<<", remaining: "<<remaining<<", isfin?: "<<isfin<<std::endl;
	for(int i=0; i<256; i++) std::cout<<"i="<<i<<", "<<counts[i]<<std::endl;
      }
      else
      {
	std::cout<<"\tRead the dataset in: "<<(psdc.Read(counts, remaining, isfin) ? "OK" : "Error!");
	std::cout<<", remaining: "<<remaining<<", isfin?: "<<isfin<<std::endl;
	for(int i=0; i<256; i++) std::cout<<"i="<<i<<", "<<counts[i]<<std::endl;
	std::cout<<"\tActual times: ";
	psdc.GetTimes(&real_preset,&live_preset);
	std::cout<<"real: "<<real_preset<<", live: "<<live_preset<<std::endl;
      }
    }
    std::cout<<"Stop: "<<psdc.Stop(counts)<<std::endl;
    for(int i=0; i<256; i++) std::cout<<"i="<<i<<", "<<counts[i]<<std::endl;
    std::cout<<"\tActual times: ";
    psdc.GetTimes(&real_preset,&live_preset);
    std::cout<<"real: "<<real_preset<<", live: "<<live_preset<<std::endl;
    std::cout<<"----------------Read from stopped---------------:";
    std::cout<<"\tRead the dataset in: "<<(psdc.Read(counts, remaining, isfin) ? "OK" : "Error!");
    std::cout<<", remaining: "<<remaining<<", isfin?: "<<isfin<<std::endl;
    for(int i=0; i<256; i++) std::cout<<"i="<<i<<", "<<counts[i]<<std::endl;
    std::cout<<"\tActual times: ";
    psdc.GetTimes(&real_preset,&live_preset);
    std::cout<<"real: "<<real_preset<<", live: "<<live_preset<<std::endl;
    
    delete [] counts;
    
  }
  
  
  //Inhibit trials
  if(inhib_trial)
  {
    SerialCamac sc("camac_serial",1,"",0.5,9600);
    bool red=true;
    while(red)
    {
	std::cout<<"Actual status of the inhibit: "<<(sc.GetInhibit(true) ? "ON" : "OFF")<<std::endl;
	std::cout<<"Sets: inhibit on: 1, inhibit off: 0, exit -1"<<std::endl;
	int ans;
	std::cin>>ans;
	if(ans==-1) red=false;
	else std::cout<<"Result of setup: "<<(sc.SetResetInhibit(ans==0) ? "OK" : "FAIL")<<std::endl;
    }
  }
  
  
  /*if(io)
  {
  
    SerialCamac sc(5,1,"",.3,9600); 
    IOOperation::OutputRegister oreg(sc,14);
    IOOperation::InterruptRegister ireg(sc,13);
    
    if(i_discover)
    {
	uint32_t data=0x0;
	uint32_t prev=0x0;
	bool first=true;
	while(1)
	{
	    ireg.reset(0xffff);
	    bool succ=ireg.read(data ,false );
	    if(succ)
	    {
	      if(first)
	      {
	        std::cout<<"Data: "<<std::hex<<data<<std::dec<<std::endl;
	        first=false;
	      }
	      if (prev!=data)
	      {
	        std::cout<<std::hex<<data<<std::dec<<std::endl;
	        prev=data;
	      }
	      if(data&IOOperation::InterruptRegister::FURCOOL_ON) std::cout<<"Cooling of the furnace is on!"<<std::endl;
	      if(data&IOOperation::InterruptRegister::TURBOPUMP_ON) std::cout<<"Turbomolecular pump is OK!"<<std::endl;
	      if(data&IOOperation::InterruptRegister::HIGH_VACUUM_ON) std::cout<<"High vacuum is OK!"<<std::endl;
	    } 
	    else std::cout<<"Reading data: ERROR"<<std::endl;
	    
	    sleep(1);
	}
    }
    
    
    
    if(io_discover)
    {
    int b=0;
    while(b!=-1)
    {
	std::cout<<"Requested bit set to on(-1: exit): ";
	std::cin>>b;
	if(b>=0)
	{
	    unsigned long int c=0x1<<b;
	    std::cout<<"The requested relay to close: "<<std::dec<<c<<std::hex<<", or: 0x"<<c<<std::dec<<std::endl;
	    
	    //Writing to the correct place
	    oreg.Set((uint32_t)c,true);
	    std::cin>>b;
	    std::cout<<"Communication"<<(oreg.Read((uint32_t *)&c) ? "OK" : "FAIL")<<std::endl;
	    std::cout<<"Actual register: "<<std::hex<<c<<std::dec<<std::endl;
	    std::cout<<"Waiting for end of check...(write a number)";
	    std::cin>>b;
	    
	    //Set to zero
	    oreg.Set(0x0,true);
	}
    }
    }
    else
    {
	std::cout<<"Testing the output register:\n\tOperate vacuum pump: 1\n\tSwitch on heating current of the furnace: 2\n\tSwitch on Harwell motors: 3\n\tExit: others\n";
	int b=0;
	std::cin>>b;
	while((b>0)&&(b<4))
	{
	    uint32_t data=0;
	    switch(b)
	    {
		case 1: data=IOOperation::OutputRegister::VACUUM_PUMP_ON; break;
		case 2: data=IOOperation::OutputRegister::FURNACE_ON; break;
		case 3: data=IOOperation::OutputRegister::HW_MOTORS_ON; break;
	    }
	    //Set the data
	    oreg.Set(data,true);
	    //sleep(10);
	    //std::cout<<"Communication"<<(oreg.Read(&data,false) ? "OK" : "FAIL")<<std::endl;
	    //std::cout<<"Actual register: "<<std::hex<<data<<std::dec<<std::endl;
	    
	    std::cout<<"Your choice: ";
	    std::cin>>b;
	}
	
    }
    
  }*/
  
  //Thermometers
  if(pyrometer)
  {
    //Construction of pyrometer
    std::cout<<"Open the pyrometer device 0 with /dev/ttyusb0 with C output with 19200 baud/sec"<<std::endl;
    UPP pyrom(0, "ttyUSB0", true,19200);//Constructor: device address, serial port number, Celsius(t)/Fahrenheit(f), baud rate, timeout

    //bool retval=true;
    //Testing RO variables
    double onechannel=0.0, ratio=0.0;
    std::cout<<"Get the actual temperature: "<<( pyrom.GetTemperature(onechannel, ratio) ? "OK" : "ERROR!")<<std::endl;
    std::cout<<"Onechannel: "<<onechannel<<", ratio: "<<ratio<<std::endl;//Get the one channel temperature in the measurement unit and the ratio temperature
    std::cout<<"Get measured value: "<<pyrom.GetMeasuredValue()<<std::endl;//Returns with the measured value, -1: communication error, -2: overflow error
    std::cout<<"Get signal strength: "<<pyrom.GetSignalStrength()<<std::endl;//Returns with the strength of the signal: 0..1500
    testThermometer(&pyrom);
    //testing set only variables 
    std::cout<<"Set operation to one channel: "<<(pyrom.SetOperationMode(false)? "OK" : "ERROR!")<<std::endl;//Sets the operation mode to 2channel (ratio, true) or 1 channel (false)
    std::cout<<"Set operation to two channel: "<<(pyrom.SetOperationMode(true)? "OK" : "ERROR!")<<std::endl;//Sets the operation mode to 2channel (ratio, true) or 1 channel (false)
    //Pairs
    //subrange
    int lowlimit, upperlimit;
    std::cout<<"Get sub range: "<<(pyrom.GetSubRange(lowlimit, upperlimit)? "OK" : "ERROR!")<<std::endl;//Returns the lower and upper limits of the subrange in the requested unit
    std::cout<<" low: "<<lowlimit<<", hilimit: "<<upperlimit<<std::endl;
    std::cout<<"Set sub range: (1000,2000)"<<(pyrom.SetSubRange(1000,2000)? "OK" : "ERROR!")<<std::endl;//Sets the lower and upper limits of the subrange in the requested unit, if lowlimit=0: reset
    std::cout<<"Check: "<<((pyrom.GetSubRange(lowlimit, upperlimit)&& lowlimit==1000 && upperlimit==2000)? "OK" : "ERROR!")<<std::endl;
    std::cout<<"Set into the original one (800..2500): "<<(pyrom.SetSubRange(800,2500)? "OK" : "ERROR!")<<std::endl;
    //response time
    std::cout<<"Get response time: "<<pyrom.GetResponseTime()<<std::endl;//Returns with the response time: from 0.000 to 10.000s (with discrate steps: -1=error, 0=min; 1=0.01; 2=0.05; 3=0.25; 4=1.0; 5=3.0; 6=10.0)
    std::cout<<"Set response time: (2)"<<(pyrom.SetResponseTime(2)? "OK" : "ERROR!")<<std::endl;//Sets the response time: from 0.000 to 10.000s (with discrate steps:0=min; 1=0.01; 2=0.05; 3=0.25; 4=1.0; 5=3.0; 6=10.0)
    std::cout<<"Check: "<<((pyrom.GetResponseTime()==2)? "OK" : "ERROR!")<<std::endl;
    std::cout<<"Set into the original one (0): "<<(pyrom.SetResponseTime(2)? "OK" : "ERROR!")<<std::endl;
    //peak memory time
    std::cout<<"Get peak memory time: "<<pyrom.GetPeakMemoryTime()<<std::endl;//Gets the peak memory time. It changes by values (-1=error, 0=OFF; 1=0.01s; 2=0.05s; 3=0.25s; 4=1.0s; 5=5.0s; 6=25.0s; 7=EXTERN; 8=AUTO; 9=HOLD)
    std::cout<<"Set peak memory time: (7)"<<(pyrom.ClearPeakMemoryTime(7)? "OK" : "ERROR!")<<std::endl;//Clear peak memory time. It changes by values (0=OFF; 1=0.01s; 2=0.05s; 3=0.25s; 4=1.0s; 5=5.0s; 6=25.0s; 7=EXTERN; 8=AUTO; 9=HOLD)
    std::cout<<"Check: "<<((pyrom.GetPeakMemoryTime()==7)? "OK" : "ERROR!")<<std::endl;
    std::cout<<"Clear max strorage: "<<(pyrom.ClearMaxStorage()? "OK" : "ERROR!")<<std::endl;//Clears the maximum storage (software simulation of external clearance)
    std::cout<<"Set into the original one (0): "<<(pyrom.ClearPeakMemoryTime(0)? "OK" : "ERROR!")<<std::endl;
    //Pyrometer not frequent set and get parameters
    bool temp_unit;
    int curr_internal_temp, max_internal_temp, minint_switchoff_level;
    double emissivity, window_trans, emiss_ratio;
    std::cout<<"Get some rural parameters by single, then by bulky: "<<std::endl;
    pyrom.getRuralSetParameters(&temp_unit, &curr_internal_temp, &max_internal_temp, &minint_switchoff_level, &emissivity, &window_trans, &emiss_ratio, false);//Gives information about the not so frequently set parameters: temperature unit(C-t, F-f), current and maximal internal temperature, minimum intensity switch-off level, emissivity in onechannel mode, window transmittance, emissivity ratio (epsilon1/epsilon2). 
    std::cout<<"Temperature unit: "<<(temp_unit ? "^oC" : "F")<<", current internal temperature:  "<<curr_internal_temp<<", maximal internal temperature: "<<max_internal_temp<<", minimal intensity switch-off level: "<<minint_switchoff_level<<", emissivity: "<<emissivity<<", window transmission: "<<window_trans<<", emissivity ratio: "<<emiss_ratio<<std::endl;
    pyrom.getRuralSetParameters(&temp_unit, &curr_internal_temp, &max_internal_temp, &minint_switchoff_level, &emissivity, &window_trans, &emiss_ratio, true);//Gives information about the not so frequently set parameters: temperature unit(C-t, F-f), current and maximal internal temperature, minimum intensity switch-off level, emissivity in onechannel mode, window transmittance, emissivity ratio (epsilon1/epsilon2). 
    std::cout<<"Temperature unit: "<<(temp_unit ? "^oC" : "F")<<", current internal temperature:  "<<curr_internal_temp<<", maximal internal temperature: "<<max_internal_temp<<", minimal intensity switch-off level: "<<minint_switchoff_level<<", emissivity: "<<emissivity<<", window transmission: "<<window_trans<<", emissivity ratio: "<<emiss_ratio<<std::endl;
    std::cout<<"Set to F,switch-off: 2, emiss: 0.99, window trans: 0.99, emiss_ratio: 0.99, window warning: 1%: "<<(pyrom.setRuralSetParameters(false, 2, .99, .99, .99, 1)? "OK" : "ERROR!")<<std::endl;//Set the values of not frequently set parameters: temperature unit(C-t, F-f), minimum intensity switch-off level, emissivity in onechannel mode, window transmittance, emissivity ratio (epsilon1/epsilon2), "Dirty window" warning level: from 0 to 99%.
    pyrom.getRuralSetParameters(&temp_unit, &curr_internal_temp, &max_internal_temp, &minint_switchoff_level, &emissivity, &window_trans, &emiss_ratio, false);//Gives information about the not so frequently set parameters: temperature unit(C-t, F-f), current and maximal internal temperature, minimum intensity switch-off level, emissivity in onechannel mode, window transmittance, emissivity ratio (epsilon1/epsilon2). 
    std::cout<<"Check: "<<((!temp_unit && (minint_switchoff_level==2) &&(emissivity==0.99) && (window_trans==0.99) && (emiss_ratio=0.99))? "OK" : "ERROR!")<<std::endl;
    std::cout<<"Reset to T,switch-off: 0, emiss: 1.0, window trans: 1.0, emiss_ratio: 1.0, window warning: 0%: "<<(pyrom.setRuralSetParameters(true, 10, 1.0, 1.0, 1.0, 0)? "OK" : "ERROR!")<<std::endl;//Set the values of not frequently set parameters: temperature unit(C-t, F-f), minimum intensity switch-off level, emissivity in onechannel mode, window transmittance, emissivity ratio (epsilon1/epsilon2), "Dirty window" warning level: from 0 to 99%.
    //Get and set communication parameters
    int address, baud;
    bool analog_output;
    std::cout<<"Get communication parameters by bulky: "<<(pyrom.getCommunication(&address, &baud, &analog_output,true)? "OK" : "ERROR!")<<std::endl;//Get communication related parameters: address, baud rate, analog output setting (t: 0..20mA, f:4..20mA), bulky reading, returns with true if communication succeeded
    std::cout<<"Address: "<<address<<", baud: "<<baud<<", analog output: "<<(analog_output ? "0..20mA" : "4..20mA")<<std::endl;
    std::cout<<"Get communication parameters by individual: "<<(pyrom.getCommunication(&address, &baud, &analog_output,false)? "OK" : "ERROR!")<<std::endl;//Get communication related parameters: address, baud rate, analog output setting (t: 0..20mA, f:4..20mA), bulky reading, returns with true if communication succeeded
    std::cout<<"Address: "<<address<<", baud: "<<baud<<", analog output: "<<(analog_output ? "0..20mA" : "4..20mA")<<std::endl;
    std::cout<<"Change communication to address 1, baud 9600, analog out: 4..20mA: "<<(pyrom.setCommunication(1, 9600, false)? "OK" : "ERROR!")<<std::endl;//Changes the communication related parameters: device address (00 to 97 (regular); 98, 99 (global address with response)), baud rate (1200 to 115200 baud) and analog output (t: 0..20mA, f:4..20mA)
    std::cout<<"Check: "<<((pyrom.getCommunication(&address, &baud, &analog_output,false) && (address==1) && (baud==9600) && !analog_output)? "OK" : "ERROR!")<<std::endl;
    std::cout<<"Set it back to 0, 19200 baud and 0..20 mA: "<<(pyrom.setCommunication(0, 19200, true)? "OK" : "ERROR!")<<std::endl;//Changes the communication related parameters: device address (00 to 97 (regular); 98, 99 (global address with response)), baud rate (1200 to 115200 baud) and analog output (t: 0..20mA, f:4..20mA)
    std::cout<<"Check: "<<((pyrom.getCommunication(&address, &baud, &analog_output,false) && (address==0) && (baud==19200) && analog_output)? "OK" : "ERROR!")<<std::endl;
    //About
    std::string ref_num, dev_type, ser_num, soft_ver, comm_det, soft_det;
    pyrom.About(&ref_num, &dev_type, &ser_num, &soft_ver, &comm_det, &soft_det, &lowlimit, &upperlimit);//Gives general information about the device: reference number, device type, detector serial number, device type and software version, communication deatails, software details, and finally with basic range limits
    std::cout<<"About:"<<std::endl<<"Reference number: "<<ref_num<<std::endl<<"Device type: "<<dev_type<<std::endl<<"Detector serial number: "<<ser_num<<std::endl<<"Device type and software version: "<<soft_ver<<std::endl<<"Communication details: "<<comm_det<<std::endl<<"Software details: "<<soft_det<<std::endl<<"Basic range: "<<lowlimit<<"~"<<upperlimit<<std::endl;
   
    std::cout<<"Reset communication: "<<(pyrom.ResetCommunication("ttyUSB0", 19200, 0.1)? "OK" : "ERROR!")<<std::endl;//Reset the communication parameters: serial port, transmission rate, timeout
    
  }
  
  if(ohm2)
  {
    OHM ohm("thermometer_ohm",1);
    
    std::cout<<"Testing print-out of available registers: "<<std::endl;
    std::cout<<"Printing out the registered datasets: "<<ohm.GetLoggedData()<<std::endl;
    std::cout<<"Erease data in flash memory: "<<(ohm.ResetLog() ? "OK" : "Fail")<<std::endl;
    bool chbA,chbB;
    double chA,chB;
    std::cout<<"Getting actual temperature: "<<(ohm.GetTemperature(chbA, chbB, chA, chB)? "OK" : "Fail")<<std::endl;
    if (chbA) std::cout<<"Channel \"A\": "<<chA<<std::endl; else std::cout<<"Channel \"A\": Not available"<<std::endl;
    if (chbB) std::cout<<"Channel \"B\": "<<chB<<std::endl; else std::cout<<"Channel \"B\": Not available"<<std::endl;
    
    testThermometer (&ohm);

    std::cout<<"Testing set-unset pairs"<<std::endl;
    std::cout<<"Sleep mode log on, auto power-off enabled, enable relative temperature mode: "<<(ohm.SetModes(true,true,true) ? "OK" : "Fail")<<std::endl;
    std::cout<<"Sleep mode log off, auto power-off disabled, disable relative temperature mode: "<<(ohm.SetModes(false,false,false) ? "OK" : "Fail")<<std::endl;
    std::cout<<"Start data logging: "<<(ohm.DataLog(true) ? "OK" : "Fail")<<std::endl;//Start(t)/stop(f) data logging
    std::cout<<"Stop data logging: "<<(ohm.DataLog(false) ? "OK" : "Fail")<<std::endl;//Start(t)/stop(f) data logging

    std::cout<<"Testing general read-only datasets"<<std::endl;
    std::cout<<"Channel A temperature unit: "<<ohm.GetTemperatureUnit()<<std::endl;
    std::cout<<"Channel B temperature unit: "<<ohm.GetTemperatureUnit(false)<<std::endl;
    std::cout<<"Number of free logpages: "<<ohm.GetFreeLogPages()<<std::endl;

    std::cout<<"Testing Read-only datasets (about) of OHM"<<std::endl;
    std::string model, model_desc, inst_sn, firm_ver, firm_date, cal_date, probe, probe_sn, probe_cal;
    double battery;
    ohm.About(model, model_desc, inst_sn, firm_ver, firm_date, cal_date, probe, probe_sn, probe_cal,battery);
    std::cout<<"Model number: "<<model<<std::endl;//Returns the model number
    std::cout<<"Model description: "<<model_desc<<std::endl;//Returns with the model description
    std::cout<<"Instrument serial number: "<<inst_sn<<std::endl;//Returns with the serial number of the instrument
    std::cout<<"Firmware version: "<<firm_ver<<std::endl;//Returns with firmware version
    std::cout<<"Firmware date: "<<firm_date<<std::endl;//Returns with firmware date
    std::cout<<"Calibration date and time: "<<cal_date<<std::endl;//Returns with calibration date and time
    std::cout<<"Type of the probe: "<<probe<<std::endl;//Returns with the type of the probe set
    std::cout<<"SN of the probe: "<<probe_sn<<std::endl;//Returns with the serial number of the probe
    std::cout<<"Calibration date of the probe: "<<probe_cal<<std::endl;//Returns with the date of calibration of the probe
    std::cout<<"Battery level: "<<battery<<"V "<<std::endl;
    std::cout<<"Head information: "<<ohm.GetAbout()<<std::endl;//Returns with head information -- DIRECT/CANONICAL

    std::cout<<"Testing RW-pairs"<<std::endl;
    std::string name("0123456789abcdef");
    std::cout<<"Setting user code to 0123456789abcdef: "<<(ohm.SetUserCode(name) ? "OK" : "fail")<<std::endl;
    std::cout<<"Usercode: "<<ohm.GetUserCode()<<std::endl;
    std::cout<<"Set time interval to 1 sec: "<<(ohm.SetTimeInterval(1) ? "OK" : "fail")<<std::endl;
    std::cout<<"Time interval: "<<ohm.GetTimeInterval()<<std::endl;
  }
  
  //Opening serial communication line
  if (kd48)
  {
      std::cout<<std::endl<<"This is the KD48P testing module"<<std::endl;
      Modbus mb("haga_controller",19200,8,'N',1,0.5);
      KD48P regulator(&mb,1);
  
      bool analog_out=false;
      bool cool_heat=true;
      bool final_term=false;
      bool scalable=false;
      bool str=false;
      bool strong_filter=false;
      bool tsc=true;
      double gain=0.0;
      int heat_speed=0;
      double p=0;
      double sp=0;
      double y=0;
      ECODE errorcode=NO_ERROR;
      FINISHEDPRG finishedprogno;
      int deadzone=0;
      int decimal_point=0;
      int deriv_time=0;
      int executedprogno=0;
      int hi=0;
      int int_time=0;
      int keep=60;
      int lo=0;
      int offset=0;
      int period=5;
      int progno=1;
      int signal_pv_y_sp=3;
      int target=0;
      int time_before=5;
      std::string prog_ver("0xyz");
      TC thermocouple=TC::J;
      Vec<int> alarms(4);

      std::cout<<"This is the testing of reading and writing of the configuration and PID part"<<std::endl;
      //PID parameters test
      std::cout<<"Get PID: ";
      if (regulator.GetPID(&gain, &int_time, &deriv_time, &offset, &deadzone))
      {
	std::cout<<"OK"<<std::endl;
	std::cout<<"Gain: "<<gain<<", integration time: "<<int_time<<"s, derivation time: "<<deriv_time<<"s, offset: "<<offset<<", deadzone: "<<deadzone<<std::endl;
      } else std::cout<<"Fail"<<std::endl;
      std::cout<<"Trying to set PID out from the valid range"<<std::endl;
      std::cout<<"Set PID (-0.1, 129, 28, 0, 0): "<<(regulator.SetPID(-0.1, 129, 28, 0, 0) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set PID (100.1, 129, 28, 0, 0): "<<(regulator.SetPID(100.1, 129, 28, 0, 0) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set PID (20.3, -1, 28, 0, 0): "<<(regulator.SetPID(20.3, -1, 28, 0, 0) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set PID (20.3, 1001, 28, 0, 0): "<<(regulator.SetPID(20.3, 1001, 28, 0, 0) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set PID (20.3, 129, -1, 0, 0): "<<(regulator.SetPID(20.3, 129, -1, 0, 0) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set PID (20.3, 129, 1001, 0, 0): "<<(regulator.SetPID(20.3, 129, 1001, 0, 0) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set PID (20.3, 129, 28, -1001, 0): "<<(regulator.SetPID(20.3, 129, 28, -1001, 0) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set PID (20.3, 129, 28, 1001, 0): "<<(regulator.SetPID(20.3, 129, 28, 1001, 0) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set PID (20.3, 129, 28, 0, -1): "<<(regulator.SetPID(20.3, 129, 28, 0, -1) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set PID (20.3, 129, 28, 0, 1001): "<<(regulator.SetPID(20.3, 129, 28, 0, 1001) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set PID (26.5, 140, 32, 63, 649) -- real: ";
      if(regulator.SetPID(26.5, 140, 32, 63, 649) && regulator.GetPID(&gain, &int_time, &deriv_time, &offset, &deadzone))
      {
	std::cout<<"OK"<<std::endl;
	std::cout<<"Gain: "<<((gain==26.5)? "OK" : "Fail")<<", integration time: "<<((int_time==140) ? "OK" : "Fail")<<", derivation time: "<<((deriv_time==32) ? "OK" : "Fail")<<", offset: "<<((offset==63) ? "OK" : "Fail")<<", deadzone: "<<((deadzone==649) ? "OK" : "Fail")<<std::endl;
      } else std::cout<<"Fail"<<std::endl;
      
      //Input configuration
      std::cout<<"Getting the input configuration part: ";
      if (regulator.GetInputConfig(&tsc, &thermocouple, &scalable, &strong_filter, &decimal_point, &offset, &lo, &hi))
      {
	std::cout<<"OK"<<std::endl;
	if(tsc)
	{
	  std::cout<<"Thermocouple type: "<<thermocouple<<", filter: "<<(strong_filter ? "strong" : "general")<<", offset: "<<offset<<std::endl;
	}
	else std::cout<<"Scalable input type: "<<(scalable ? "0..20mA" : "4..20mA")<<", filter: "<<(strong_filter ? "strong" : "general")<<", decimal point: "<<decimal_point<<", offset: "<<offset<<", scaling from "<<lo<<" to "<<hi<<std::endl;
      } else std::cout<<"Fail"<<std::endl;
      std::cout<<"Trying to set input out from the valid range: "<<std::endl;
      thermocouple=TC::S;
      std::cout<<"Set InputConfig(S, NULL, true, 0, -1001, 0, 0): "<<(regulator.SetInputConfig(&thermocouple, NULL, true, 0, -1001, 0, 0)? "ERROR" : "")<<std::endl;
      std::cout<<"Set InputConfig(S, NULL, true, 0, 1001, 0, 0): "<<(regulator.SetInputConfig(&thermocouple, NULL, true, 0, 1001, 0, 0)? "ERROR" : "")<<std::endl;
      std::cout<<"Set InputConfig(S, NULL, true, 0, 0, -2000, 0): "<<(regulator.SetInputConfig(&thermocouple, NULL, true, 0, 0, -2000, 0)? "ERROR" : "")<<std::endl;
      std::cout<<"Set InputConfig(S, NULL, true, 0, 0, 0, -2000): "<<(regulator.SetInputConfig(&thermocouple, NULL, true, 0, 0, 0, -2000)? "ERROR" : "")<<std::endl;
      std::cout<<"Set InputConfig(S, NULL, true, 0, 0, 4001, 0): "<<(regulator.SetInputConfig(&thermocouple, NULL, true, 0, 0, 4001, 0)? "ERROR" : "")<<std::endl;
      std::cout<<"Set InputConfig(S, NULL, true, 0, 0, 0, 4001): "<<(regulator.SetInputConfig(&thermocouple, NULL, true, 0, 0, 0, 4001)? "ERROR" : "")<<std::endl;
      scalable=false;
      std::cout<<"Set InputConfig(S, false, true, 0, 0, 0, 0): "<<(regulator.SetInputConfig(&thermocouple, &scalable, true, 0, 0, 0, 0)? "ERROR" : "")<<std::endl;
      std::cout<<"Set InputConfig(NULL, false, true, -1, 0, 0, 0): "<<(regulator.SetInputConfig(NULL, &scalable, true, -1, 0, 0, 0)? "ERROR" : "")<<std::endl;
      std::cout<<"Set InputConfig(NULL, false, true, 4, 0, 0, 0): "<<(regulator.SetInputConfig(NULL, &scalable, true, 4, 0, 0, 0)? "ERROR" : "")<<std::endl;
      std::cout<<"Set InputConfig(NULL, false, true, 2, 1, 10, 100): --- real: ";
      scalable=false;
      if(regulator.SetInputConfig(NULL, &scalable, true, 2, 1, 10, 100) && regulator.GetInputConfig(&tsc, &thermocouple, &scalable, &strong_filter, &decimal_point, &offset, &lo, &hi))
      {
	std::cout<<"OK"<<std::endl;
	if(tsc)
	{
	  std::cout<<"Input type: Fail"<<std::endl;
	}
	else std::cout<<"Scalable input type (OK): "<<(scalable ? "Fail" : "OK")<<", filter: "<<(strong_filter ? "OK" : "Fail")<<", decimal point: "<<((decimal_point==2) ? "OK" : "Fail")<<", offset: "<<((offset==1) ? "OK" : "Fail")<<", scaling from: "<<((lo==10) ? "OK" : "Fail")<<", scaling to "<<((hi==100)? "OK" : "Fail")<<std::endl;
      }else std::cout<<"Fail"<<std::endl;
      std::cout<<"Set InputConfig(S, NULL, true, 2, 1, 10, 100): --- real: ";
      if(regulator.SetInputConfig(&thermocouple, NULL, true, 2, 1, 101, 9) && regulator.GetInputConfig(&tsc, &thermocouple, &scalable, &strong_filter, &decimal_point, &offset, &lo, &hi))
      {
	std::cout<<"OK"<<std::endl;
	if(tsc)
	{
	  std::cout<<"Thermocouple type (OK): "<<((thermocouple==TC::S) ? "OK" : "Fail")<<", filter: "<<(strong_filter ? "OK" : "Fail")<<", decimal point: "<<((decimal_point==2) ? "OK" : "Fail")<<", offset: "<<((offset==1) ? "OK" : "Fail")<<", scaling from: "<<((lo==9) ? "OK" : "Fail")<<", scaling to "<<((hi==101)? "OK" : "Fail")<<std::endl;
	}
	else std::cout<<"Scalable input type: Fail"<<std::endl;
      }else std::cout<<"Fail"<<std::endl;

      //Testing the input range
      std::cout<<"Getting the input range part: ";
      if (regulator.GetInputRangeConfig(lo, hi))
      {
	std::cout<<"OK"<<std::endl;
	std::cout<<"Low: "<<lo<<", high: "<<hi<<std::endl;
      } else std::cout<<"Fail"<<std::endl;
      std::cout<<"Trying to set range out from the valid range"<<std::endl;
      std::cout<<"Set Input range(-2000, 0): "<<(regulator.SetInputRangeConfig(-2000, 100) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set Input range(0, -2000): "<<(regulator.SetInputRangeConfig(0, -2000) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set Input range(4001, 0): "<<(regulator.SetInputRangeConfig(4001, 100) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set Input range(0, 4001): "<<(regulator.SetInputRangeConfig(0, 4001) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set Input range(200, 10): -- real (+exchange task): ";
      if (regulator.SetInputRangeConfig(200, 10) && regulator.GetInputRangeConfig(lo, hi))
      {
	std::cout<<"OK"<<std::endl;
	  std::cout<<"Low: "<<((lo==10) ? "OK" : "Fail")<<", high: "<<((hi==200) ? "OK" : "Fail")<<std::endl;
      }else std::cout<<"Fail"<<std::endl;

      //Testing the regulator config part
      std::cout<<"Getting the regulator configuration part: ";
      if (regulator.GetRegulatorConfig(&cool_heat, &str, &period, &deadzone, &lo, &hi))
      {
	std::cout<<"OK"<<std::endl;
	std::cout<<"Regulator set as "<<(cool_heat ? "cooler" : "heater")<<" controlled by "<<(str ? "STR" : "general")<<", periodic time: "<<period<<"s, dead zone: "<<deadzone<<", low: "<<lo<<", high: "<<hi<<std::endl;
      } else std::cout<<"Fail"<<std::endl;
      std::cout<<"Trying to set regulator out from the valid range"<<std::endl;
      std::cout<<"Set regulator config(false, true,-1,0,0,100): "<<(regulator.SetRegulatorConfig(false, true, -1, 0, 0, 100) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set regulator config(false, true,256,0,0,100): "<<(regulator.SetRegulatorConfig(false, true, 256, 0, 0, 100) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set regulator config(false, true,1,-1,0,100): "<<(regulator.SetRegulatorConfig(false, true, 1, -1, 0, 100) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set regulator config(false, true,1,101,0,100): "<<(regulator.SetRegulatorConfig(false, true, 1, 101, 0, 100) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set regulator config(false, true,1,0,-1,100): "<<(regulator.SetRegulatorConfig(false, true, 1, 0, -1, 100) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set regulator config(false, true,1,0,101,100): "<<(regulator.SetRegulatorConfig(false, true, 1, 0, 101, 100) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set regulator config(false, true,1,0,0,101): "<<(regulator.SetRegulatorConfig(false, true, 1, 0, 0, 101) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set regulator config(false, true,1,0,0,-1): "<<(regulator.SetRegulatorConfig(false, true, 1, 0, 0, -1) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set regulator config(true, false,64,2,73,5): -- real (+exchange task): ";
      if (regulator.SetRegulatorConfig(true, false,64, 2, 73, 5) && regulator.GetRegulatorConfig(&cool_heat, &str, &period, &deadzone, &lo, &hi))
      {
	std::cout<<"OK"<<std::endl;
	std::cout<<"Regulator set: "<<(cool_heat ? "OK" : "Fail")<<" controller: "<<(str ? "Fail" : "OK")<<", periodic time: "<<((period==64) ? "OK" : "Fail")<<", dead zone: "<<((deadzone==2) ? "OK" : "Fail")<<", low: "<<((lo==5) ? "OK" : "Fail")<<", high: "<<((hi==73) ? "OK" : "Fail")<<std::endl;
      }else std::cout<<"Fail"<<std::endl;
      
      //Testing the analog output configuration
      std::cout<<"Getting the analog output configuration part: ";
      if (regulator.GetAnalogOutConfig(&analog_out, &signal_pv_y_sp, &lo, &hi))
      {
	std::cout<<"OK"<<std::endl;
	std::cout<<"Analog output: "<<(analog_out  ? "0..20mA" : "4..20mA")<<",  signal type: "<<signal_pv_y_sp<<", low: "<<lo<<", high: "<<hi<<std::endl;
      } else std::cout<<"Fail"<<std::endl;
      std::cout<<"Trying to set analog output out from the valid range"<<std::endl;
      std::cout<<"Set analog output config(false, 0,0,100): "<<(regulator.SetAnalogOutConfig(false, 0, 0, 100) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set analog output config(false, 4,0,100): "<<(regulator.SetAnalogOutConfig(false, 4, 0, 100) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set analog output config(false, 2,-2000,100): "<<(regulator.SetAnalogOutConfig(false, 0, -2000, 100) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set analog output config(false, 2,4000,100): "<<(regulator.SetAnalogOutConfig(false, 0, 4000, 100) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set analog output config(false, 2,0,-2000): "<<(regulator.SetAnalogOutConfig(false, 2, 0, -2000) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set analog output config(false, 2,0,4000): "<<(regulator.SetAnalogOutConfig(false, 2, 0, 4000) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set analog output config(false, 3,62,40): -- real (exchange): ";
      if (regulator.SetAnalogOutConfig(false, 3, 62, 40) && regulator.GetAnalogOutConfig(&analog_out, &signal_pv_y_sp, &lo, &hi))
      {
	std::cout<<"OK"<<std::endl;
	std::cout<<"Analog output: "<<(analog_out  ? "Fail" : "OK")<<",  signal type: "<<((signal_pv_y_sp==3) ? "OK" : "Fail")<<", low: "<<((lo==40) ? "OK" : "Fail")<<", high: "<<((hi==62) ? "OK" : "Fail")<<std::endl;
      }else std::cout<<"Fail"<<std::endl;
      
      std::cout<<"Try to reset alarm configuration: "<<(regulator.ResetAlarmConfig() ? "OK" : "Fail")<<std::endl;
      std::cout<<"Set Time-baseline config: "<<(regulator.SetTimeBaselineConfig() ? "OK" : "Fail")<<std::endl;
      std::cout<<"Get information about controller: ";
      if (regulator.About(&prog_ver))
      {
	std::cout<<"OK: "<<prog_ver<<std::endl;
      } else std::cout<<"Fail"<<std::endl;
      
      std::cout<<"Try to reset alarm properties: "<<(regulator.ResetAlarmProperties() ? "OK" : "Fail")<<std::endl;
      std::cout<<"Try to reset everything as solid-state relay: "<<(regulator.ResetAll_SolidStateRelay(10, 500,TC::K) ? "OK" : "Fail")<<std::endl;
      
      //Test slave routines
      std::cout<<"Testing slave number related routines: "<<std::endl;
      std::cout<<"Getting the slave number: "<<regulator.getSlaveNumber()<<std::endl;
      std::cout<<"Set it to 2: ";
      regulator.setSlaveNumber(2);
      std::cout<<((2==regulator.getSlaveNumber()) ? "OK" : "Fail")<<std::endl;
      std::cout<<"Set it back to 1: ";
      regulator.setSlaveNumber(1);
      std::cout<<((1==regulator.getSlaveNumber()) ? "OK" : "Fail")<<std::endl;
      
      //Test temperature reading routines
      std::cout<<"Checking class realated reading routine: ";
      if(regulator.getRegVals(p, sp, y)) std::cout<<"OK"<<std::endl<<"P: "<<p<<", SP: "<<sp<<", Y: "<<y<<std::endl;
      else std::cout<<"Fail"<<std::endl;
      std::cout<<"Checking by parent's class routine: "<<std::endl;
      testThermometer (&regulator);
      
      //Testing temperature program reading and writing
      std::cout<<"Testing temperature program reading and writing"<<std::endl;
      std::cout<<"Get the actual setup "<<std::endl;
      for(int i=0; i<2; i++)
      {
	for(int j=0; j<4; j++)
	{
	  std::cout<<"Program: "<<i<<", subprogram: "<<j;
	  if (regulator.GetProgram(i, j, &time_before, &heat_speed, &target, &keep, &final_term))
	  {
	    std::cout<<", pre time: "<<time_before<<"min, heat speed: "<<heat_speed<<"C/h, temperature: "<<target<<"C, keep time: "<<keep<<"mins "<<(final_term ? "END" : "")<<std::endl;
	  }else std::cout<<" FAIL"<<std::endl;   
	}
      }
      std::cout<<"Testing the program writing with values out from the valid range:"<<std::endl;
      std::cout<<"Set program (-1,0,0,0,50,0,t): "<<(regulator.SetProgram(-1, 0, 0, 0, 50, 0, true) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set program (2,0,0,0,50,0,t): "<<(regulator.SetProgram(2, 0, 0, 0, 50, 0, true) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set program (0,-1,0,0,50,0,t): "<<(regulator.SetProgram(0, -1, 0, 0, 50, 0, true) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set program (0,4,0,0,50,0,t): "<<(regulator.SetProgram(0, 4, 0, 0, 50, 0, true) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set program (0,0,-1,0,50,0,t): "<<(regulator.SetProgram(0, 0, -1, 0, 50, 0, true) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set program (0,0,6000,0,50,0,t): "<<(regulator.SetProgram(0, 0, 6000, 0, 50, 0, true) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set program (0,0,0,-200,50,0,t): "<<(regulator.SetProgram(0, 0, 0, -2000, 50, 0, true) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set program (0,0,0,400,50,0,t): "<<(regulator.SetProgram(0, 0, 0, 4000, 50, 0, true) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set program (0,0,0,0.0,0,0,t): "<<(regulator.SetProgram(0, 0, 0, 0, 0, 0, true) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set program (0,0,0,0.0,1200,0,t): "<<(regulator.SetProgram(0, 0, 0, 0, 1200, 0, true) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set program (0,0,0,0.0,0,-1,t): "<<(regulator.SetProgram(0, 0, 0, 0, 50, -1, true) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set program (0,0,0,0.0,0,6000,t): "<<(regulator.SetProgram(0, 0, 0, 0, 50, 6000, true) ? "ERROR" : "")<<std::endl;
      
      std::cout<<"Set program all programs with (i,j,2,0.1,33,0,false) "<<std::endl;
      for(int i=0; i<2; i++)
      {
	for(int j=0; j<4; j++)
	{
	  std::cout<<"Program: "<<i<<", subprogram: "<<j;
	  if (!regulator.SetProgram(i, j, 2, 1, 33, 0, false)) std::cout<<" FAIL"<<std::endl;
	  else std::cout<<" OK"<<std::endl;   
	}
      }
      std::cout<<"Set program all programs with (i,j,7,0.4,40,5,true) "<<std::endl;
      for(int i=0; i<2; i++)
      {
	for(int j=0; j<4; j++)
	{
	  std::cout<<"Program: "<<i<<", subprogram: "<<j;
	  if (!regulator.SetProgram(i, j, 7, 4, 40, 5, true)) std::cout<<" FAIL"<<std::endl;
	  else std::cout<<" OK"<<std::endl;   
	}
      }
      
      //Check the program numbers
      std::cout<<"Checking the program number related things: "<<std::endl;
      std::cout<<"Read actual program number: ";
      if(regulator.GetActualProgramNumber(&progno))
      {
	std::cout<<"OK"<<std::endl;
	std::cout<<"Program number: "<<progno<<std::endl;
      } else std::cout<<"Fail"<<std::endl;
      std::cout<<"Set program number outside of the valid range: "<<std::endl;
      std::cout<<"Set program  number(-1): "<<(regulator.SetActualProgramNumber(-1) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set program  number(2): "<<(regulator.SetActualProgramNumber(2) ? "ERROR" : "")<<std::endl;
      std::cout<<"Set program  number(0): ";
      if(regulator.SetActualProgramNumber(0) && regulator.GetActualProgramNumber(&progno)) std::cout<<"OK, actual program number: "<<progno<<std::endl;
      else std::cout<<"Fail"<<std::endl;
      
      //set program before
      std::cout<<"Set the (0,0,7,0.4,40,5,false), (0,1,7,0.5,50,2,true), (1,0,0,0.4,30,5,false), (1,1,0,0.5,35,2,true) program";
      if(regulator.SetProgram(0, 0, 7, 4, 40, 5, false)&& regulator.SetProgram(0, 1, 7, 5, 50, 2, true)&&regulator.SetProgram(1, 0, 7, 4, 30, 5, false)&& regulator.SetProgram(1, 1, 7, 5, 35, 2, true)) std::cout<<" OK"<<std::endl;
      else std::cout<<" Fail"<<std::endl;
      //Testing switch on and off in relation with the status checking
      sleep(1);
      std::cout<<"Checking the status and switch on/off"<<std::endl;
      std::cout<<"Get actual status (in status OFF): ";
      if(regulator.getStatus(alarms, finishedprogno, executedprogno, errorcode))
      {
        std::cout<<"OK"<<std::endl<<"Program step: "<<finishedprogno<<", program number: "<<executedprogno<<", errorcode: "<<errorcode<<std::endl;
        std::cout<<"Alarms: ";
        for(int i=0; i<4; i++) std::cout<<(alarms[i] ? " ON" : " OFF");
        std::cout<<std::endl;
      }else std::cout<<"FAIL"<<std::endl;
      std::cout<<"Trying to switch on the device: Status: "<<((regulator.switchOn(true)) ? "ON" : "OFF")<<std::endl;
      sleep(2);
      std::cout<<"Second trial to switch on: Status: "<<((regulator.switchOn(true)) ? "ON" : "OFF")<<std::endl;
      sleep(2);
      std::cout<<"Get actual status (probably status ON): ";
      if(regulator.getStatus(alarms, finishedprogno, executedprogno, errorcode))
      {
        std::cout<<"OK"<<std::endl<<"Program step: "<<finishedprogno<<", program number: "<<executedprogno<<", errorcode: "<<errorcode<<std::endl;
        std::cout<<"Alarms: ";
        for(int i=0; i<4; i++) std::cout<<(alarms[i] ? " ON" : " OFF");
        std::cout<<std::endl;
      }else std::cout<<"FAIL"<<std::endl;
      sleep(10);
      std::cout<<"Go to the next program step: "<<(regulator.goToNext() ? "OK" : "Fail")<<std::endl;
      std::cout<<"Get actual status (probably status ON): ";
      if(regulator.getStatus(alarms, finishedprogno, executedprogno, errorcode))
      {
        std::cout<<"OK"<<std::endl<<"Program step: "<<finishedprogno<<", program number: "<<executedprogno<<", errorcode: "<<errorcode<<std::endl;
        std::cout<<"Alarms: ";
        for(int i=0; i<4; i++) std::cout<<(alarms[i] ? " ON" : " OFF");
        std::cout<<std::endl;
      }else std::cout<<"FAIL"<<std::endl;
      sleep(10);
      std::cout<<"Go to the next program step: "<<(regulator.goToNext() ? "OK" : "Fail")<<std::endl;
      std::cout<<"Get actual status (probably status ON): ";
      if(regulator.getStatus(alarms, finishedprogno, executedprogno, errorcode))
      {
        std::cout<<"OK"<<std::endl<<"Program step: "<<finishedprogno<<", program number: "<<executedprogno<<", errorcode: "<<errorcode<<std::endl;
        std::cout<<"Alarms: ";
        for(int i=0; i<4; i++) std::cout<<(alarms[i] ? " ON" : " OFF");
        std::cout<<std::endl;
      }else std::cout<<"FAIL"<<std::endl;
      sleep(10);
      std::cout<<"Try to switch off the regulator: Status: "<<((regulator.switchOff(true)) ? "OFF" : "ON")<<std::endl;
      sleep(2);
      std::cout<<"Try to switch off the regulator(2nd): Status: "<<((regulator.switchOff(true)) ? "OFF" : "ON")<<std::endl;
      std::cout<<"Get actual status (probably status OFF): ";
      if(regulator.getStatus(alarms, finishedprogno, executedprogno, errorcode))
      {
        std::cout<<"OK"<<std::endl<<"Program step: "<<finishedprogno<<", program number: "<<executedprogno<<", errorcode: "<<errorcode<<std::endl;
        std::cout<<"Alarms: ";
        for(int i=0; i<4; i++) std::cout<<(alarms[i] ? " ON" : " OFF");
        std::cout<<std::endl;
      }else std::cout<<"FAIL"<<std::endl;
      
      std::cout<<"Setting program number to 0"<<(regulator.SetActualProgramNumber(0) ? "OK" : "Fail")<<std::endl;
      
      std::cout<<std::endl<<"Listing the content of the registers on screen of slave: "<<regulator.getSlaveNumber()<<std::endl;
      regulator.readConfig(std::cout);
      std::cout<<std::endl;
      
      std::cout<<"Reset all temperature programs: "<<(regulator.ResetAllPrograms() ? "OK" : "Fail")<<std::endl;
      std::cout<<std::endl<<"After program reset: "<<std::endl;
      regulator.readConfig(std::cout);
      
      std::cout<<"We have already seen, that reset to factory is working"<<std::endl;
      //std::cout<<"Reset back to factory configuration: "<<(regulator.ResetFactoryConfig() ? "OK" : "Fail")<<std::endl;
      //std::cout<<std::endl<<"After config reset: "<<std::endl;
      //regulator.readConfig(std::cout);
      
      std::cout<<"Set solid state relay output: "<<(regulator.ResetAll_SolidStateRelay(10, 500,TC::K) ? "OK" : "Fail")<<std::endl;
      std::cout<<std::endl<<"After STR config: "<<std::endl;
      regulator.readConfig(std::cout);
    }
    
      //Motors
  if(m14motor)
  {
    
    //ParallelCamac sc(0x230,7);
    //CamacTest sc(1);
    SerialCamac sc("serial_camac",1,"",.3,9600); 
    //sc.Reset();
    std::cout<<"Inhibit: "<<sc.GetInhibit(true)<<" "<<sc.GetInhibit(false)<<std::endl;
    std::cout<<"Set Inhibit: "<<sc.SetResetInhibit(false)<<std::endl;
    std::cout<<"Inhibit: "<<sc.GetInhibit(true)<<" "<<sc.GetInhibit(false)<<std::endl;
    std::cout<<"Reset Inhibit: "<<sc.SetResetInhibit(true)<<std::endl;
    std::cout<<"Inhibit: "<<sc.GetInhibit(true)<<" "<<sc.GetInhibit(false)<<std::endl;
    
    /*std::cout<<"Checking test functions: "<<std::endl;
    std::cout<<"Do by-pass: "<<sc.SetResetDeviceTest(BY_PASS,true)<<"...leave 5 seconds for visual"<<std::endl;
    sleep(5);
    std::cout<<"Go to the original: "<<sc.SetResetDeviceTest(BY_PASS,false)<<std::endl;
    std::cout<<"Switch to offline: "<<sc.SetResetDeviceTest(OFF_LINE,true)<<std::endl;
    std::cout<<"Checking: "<<sc.CheckDeviceTest(OFF_LINE)<<"...leave 5 seconds for visual"<<std::endl;
    sleep(5);
    std::cout<<"Go to the original: "<<sc.SetResetDeviceTest(OFF_LINE,false)<<std::endl;
    std::cout<<"Checking: "<<sc.CheckDeviceTest(OFF_LINE)<<std::endl;
    std::cout<<"Do loop collapse: "<<sc.SetResetDeviceTest(LOOP_COLLAPSE,true)<<std::endl;
    std::cout<<"Checking: "<<sc.CheckDeviceTest(LOOP_COLLAPSE)<<"...leave 5 seconds for visual"<<std::endl;
    sleep(5);
    std::cout<<"Go to the original: "<<sc.SetResetDeviceTest(LOOP_COLLAPSE,false)<<std::endl;
    std::cout<<"Checking: "<<sc.CheckDeviceTest(LOOP_COLLAPSE)<<std::endl;
    
    std::cout<<"Manual offline switch is "<<sc.CheckDeviceTest(HW_OFFLINE)<<" , please switch it off manually, then write a character"<<std::endl;
    char ans;
    std::cin>>ans;
    std::cout<<"Manual offline switch is "<<sc.CheckDeviceTest(HW_OFFLINE)<<" , please switch to online manually, then write a character"<<std::endl;
    std::cin>>ans;
    std::cout<<"Manual offline switch is "<<sc.CheckDeviceTest(HW_OFFLINE)<<std::endl;
    uint32_t prev_data=0x0;
    std::cout<<"The previously read data has been read: "<<sc.GetPData(prev_data,false);
    std::cout<<", value: "<<prev_data<<", its flags: "<<(int) sc.CheckPDataFlags()<<std::endl;*/
    
    IOOperation::OutputRegister oreg(sc,14);
    IOOperation::InterruptRegister ireg(sc,13);
    Positioner::StepMotorController smc(sc,4);
    
    /*std::cout<<"Status: "<<std::hex<<smc.ReadStatusRegister(1)<<std::dec<< std::endl;
    std::cout<<"TestAnyLAM: "<<smc.TestAnyLAMMotor(1)<<std::endl;//Test LAM 1-7 for the requested motor
    for(uint8_t j=1; j<=7; j++)
    {
      std::cout<<"TestLAM: "<<(int)j<<" "<<smc.TestLAM(1, j)<<std::endl;//Test for motor and specific LAM
      std::cout<<"ClearLAM: "<<(int)j<<" "<<smc.ClearLAM(1, j)<<std::endl;//Clear for motor and specific LAM
    }*/
    
    
    Positioner::ParallelMotor m1(sc,"M1",1,1,(uint16_t) 1000, (uint16_t) 4150, oreg, ireg, smc, 1.0, 144, 2445, 2, 0,13);
    Positioner::ParallelMotor m2(sc,"M2",2,3,(uint16_t) 200, (uint16_t) 7000, oreg, ireg, smc, 1.0, 144, 2445, 2, 0,13);
    Positioner::AngleParallelMotor m3(sc,"M3",3,5, (uint16_t) 23568, (uint16_t) 39051, oreg, ireg, smc, 2.0, 228, 1590, 0.0, -1.0/180.0, 2, 0,16);
    Positioner::CushionAPMotor m4(sc,"M4",4, 6, (uint16_t) 1690, (uint16_t) 4289, oreg, ireg, smc, 2.0, 228, 1590, 0.0, -0.01, 2, 0, 16);
    std::cout<<"Get position"<<std::endl;
    uint16_t upos1=  m1.getPosition();
    uint16_t upos3=  m3.getPosition();
    uint16_t upos4=  m4.getPosition();
    uint16_t upos2=  m2.getPosition();
    std::cout<<m1.getName()<<" in uniform pulses: [2645]  "<<std::hex<<(int)upos1<<std::dec<<" (hex), "<<(int)upos1<<" (dec)"<<std::endl;
    std::cout<<m2.getName()<<" in uniform pulses: [3399]  "<<std::hex<<(int)upos2<<std::dec<<" (hex), "<<(int)upos2<<" (dec)"<<std::endl;
    std::cout<<m3.getName()<<" in uniform pulses: [32779] "<<std::hex<<(int)upos3<<std::dec<<" (hex), "<<(int)upos3<<" (dec)"<<std::endl;
    std::cout<<m4.getName()<<" in uniform pulses: [3700]  "<<std::hex<<(int)upos4<<std::dec<<" (hex), "<<(int)upos4<<" (dec)"<<std::endl;
    std::cout<<"Status register: "<<std::hex<<smc.ReadStatusRegister(1)<<std::dec<<std::endl;
    smc.EnableDisableLAM(1, 1, true);
    std::cout<<"Status register: "<<std::hex<<smc.ReadStatusRegister(1)<<std::dec<<std::endl;
    std::cout<<"Requested motor to move (5: evacuate air from M4): ";
    int motor_no;
    std::cin>>motor_no;
    switch(motor_no)
    {
        case 5: std::cout<<"Evacuate air from M4: "; break;
	case 4: std::cout<<"Move M4 to: "; break;
	case 2: std::cout<<"Move M2 to: "; break;
	case 3: std::cout<<"Move M3 to: "; break;
	default: std::cout<<"Move M1 to: ";
    }
    int target;
    if (motor_no!=5) std::cin>>target;
    switch(motor_no)
    {
	case 4: std::cout<<(m4.moveTo((uint16_t)target) ? "OK" : "Fail")<<std::endl; break;
	case 2: std::cout<<(m2.moveTo((uint16_t)target) ? "OK" : "Fail")<<std::endl; break;
	case 3: std::cout<<(m3.moveTo((uint16_t)target) ? "OK" : "Fail")<<std::endl; break;
	case 5: std::cout<<(m4.EvacuateAir() ? "OK" : "Fail")<<std::endl; break;
	default: std::cout<<(m1.moveTo((uint16_t)target) ? "OK" : "Fail")<<std::endl;
    }
    std::cout<<"Status register: "<<std::hex<<smc.ReadStatusRegister(1)<<std::dec<<std::endl;
  }
 
  if (harwell_motor)
  {
    //ParallelCamac  sc(0x230,7);
    SerialCamac sc("serial_camac",1,"",0.5,9600);
    IOOperation::OutputRegister oreg(sc,14);
    /*std::cout<<"Reset: "<<sc.Reset()<<std::endl;
    std::cout<<"Reset Comm: "<<sc.ResetComm(1,0.2)<<std::endl;*/
    Positioner:: HarwellMotor mot(sc, "2Theta", 17, 0xFFFF, 0x0, oreg, 0);
    Positioner:: HarwellMotor omega(sc, "Omega", 18, 0xFFFF, 0x0, oreg, 0);
    Positioner:: HarwellMotor chi(sc, "Chi", 19, 0xFFFF, 0x0, oreg, 0);
    Positioner:: HarwellMotor phi(sc, "Phi", 20, 0xFFFF, 0x0, oreg, 0);
    
    phi.moveATo(0.0);
    chi.moveATo(0.0);
    omega.moveATo(29.0);
    

    std::cout<<"Get status: "<<(int)mot.getStatus()<<std::endl;
    std::cout<<"Set to manual mode: "<<(int)mot.setManual()<<std::endl;
    std::cout<<"Get status: "<<(int)mot.getStatus()<<std::endl;
    std::cout<<"Get position: ";
    double pos= mot.getAPosition();
    uint32_t upos=  mot.getPosition();
    std::cout<<" angular: "<<pos<<", in uniform pulses: "<<std::hex<<(int)upos<<std::dec<<" (hex), "<<(int)upos<<" (dec)"<<std::endl;
    std::cout<<"Setting the limits"<<std::endl;
    std::cout<<"Set low  to 0x1: "<<(mot.setLimit(0x1,false) ? "OK" : "Fail")<<std::endl;
    std::cout<<"Set high to 0xfffe: "<<(mot.setLimit(0xfffe,true) ? "OK" : "Fail")<<std::endl;
    std::cout<<"Get the actual limits...";
    upos=mot.getLimit(false);
    uint32_t upos2=mot.getLimit(true);
    std::cout<<"The limits are: "<<std::hex<<(int)upos<<"~"<<(int)upos2<<std::dec<<" or "<<(int)upos<<"~"<<(int)upos2<<std::endl;
    std::cout<<"Try to set low/high limits higher/lower than high/low limit"<<std::endl;
    std::cout<<"Set low  to 0xffff: "<<(mot.setLimit(0xffff,false) ? "OK (ERROR)" : "Fail (NORMAL)")<<std::endl;
    std::cout<<"Set high  to 0x0: "<<(mot.setLimit(0x0,true) ? "OK (ERROR)" : "Fail (NORMAL)")<<std::endl;
    std::cout<<"Setting the limits by angular way"<<std::endl;
    std::cout<<"Set low  to -20: "<<(mot.setALimit(-20.0,false) ? "OK" : "Fail")<<std::endl;
    std::cout<<"Set high to 150: "<<(mot.setALimit(150.0,true) ? "OK" : "Fail")<<std::endl;
    std::cout<<"Get the actual limits...";
    pos=mot.getALimit(false);
    double pos2=mot.getALimit(true);
    std::cout<<"The limits are: "<<pos<<"~"<<pos2<<std::endl;
    upos=mot.getLimit(false);
    upos2=mot.getLimit(true);
    std::cout<<"The positional limits are: "<<std::hex<<(int)upos<<"~"<<(int)upos2<<std::dec<<" or "<<(int)upos<<"~"<<(int)upos2<<std::endl;
    std::cout<<"Getting the conversion variables"<<std::endl;
    double pshift=mot.getConversion(true);
    double pslope=mot.getConversion(false);
    std::cout<<"Original shift: "<<pshift<<", and slope: "<<pslope<<std::endl;
    std::cout<<"Set 0.25 degrees as zero point";
    mot.setZero(0.25);
    pos=mot.getAPosition();
    std::cout<<"The new position is "<<pos<<std::endl;
    std::cout<<"The new shift is: "<<mot.getConversion(true)<<", and the new slope: "<<mot.getConversion(false)<<std::endl;
    std::cout<<"Setting back the conversion values into originals.."<<std::endl;
    mot.setConversion(pshift,true); mot.setConversion(pslope,false);
    std::cout<<"Checking the settings... shift is: "<<mot.getConversion(true)<<", and the slope: "<<mot.getConversion(false)<<std::endl;
    std::cout<<"Move the detector to 100 degrees: "<<(mot.moveATo(100.0) ? "OK" : "Fail")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Try to move to 100 degrees again: "<<(mot.moveATo(100.0) ? "OK (ERROR)" : "Fail (NORMAL)")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Changing the upper limit to be 99 degrees: "<<(mot.setALimit(99.0,true) ? "OK" : "Fail")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Try to move to 101 degrees: "<<(mot.moveATo(101.0) ? "OK (ERROR)" : "Fail (NORMAL)")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Try to move to 99.99 degrees: "<<(mot.moveATo(99.99) ? "OK" : "Fail")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Try to move to 96.0 degrees: "<<(mot.moveATo(96.0) ? "OK" : "Fail")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Try to move out from the valid range (101 degrees): "<<(mot.moveATo(101.0) ? "OK (ERROR)" : "Fail (NORMAL)")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Changing the high limit to 110 degrees: "<<(mot.setALimit(110.0,true) ? "OK" : "Fail")<<std::endl;
    std::cout<<"Changing the low limit to 105 degrees: "<<(mot.setALimit(105.0,false) ? "OK" : "Fail")<<std::endl;
    std::cout<<"Try to move to 96.0 degrees again: "<<(mot.moveATo(96.0) ? "OK (ERROR)" : "Fail (NORMAL)")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Try to move to 96.1 degrees: "<<(mot.moveATo(96.1) ? "OK" : "Fail")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Try to move to 96.0 degrees again: "<<(mot.moveATo(96.0) ? "OK (ERROR)" : "Fail (NORMAL)")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Try to move to 125.0 degrees: "<<(mot.moveATo(125.0) ? "OK(ERROR)" : "Fail(NORMAL)")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Try to move to 110.0 degrees: "<<(mot.moveATo(110.0) ? "OK" : "Fail")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Move the detector to 106 degrees by pulses: "<<(mot.moveTo(0xa968) ? "OK" : "Fail")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Try to move to 96.0 degrees again: "<<(mot.moveATo(96.0) ? "OK (ERROR)" : "Fail (NORMAL)")<<std::endl;
    while(!mot.isAllStopped()){std::cout<<"\tactual position: "<<mot.getAPosition()<<std::endl; sleep(1u);}
    std::cout<<"Trying emergency stop: restore the limits: "<<((mot.setALimit(150.0,true) && mot.setALimit(-20.0,false)) ? "OK" : "Failed")<<std::endl; 
    std::cout<<"Move the detector to 70 degrees: "<<(mot.moveATo(70.0) ? "OK" : "Fail")<<std::endl;
    std::cout<<"Wait..."<<std::endl;
    sleep(4);
    std::cout<<"Get status: "<<(int)mot.getStatus()<<std::endl;
    std::cout<<"Get position: "<<mot.getAPosition()<<std::endl;
    std::cout<<"Emergency stop here: "<<mot.Stop()<<std::endl;
    
    std::cout<<"Actual position of the motors"<<std::endl;
    std::cout<<"Phi: "<<phi.getAPosition()<<std::endl;
    std::cout<<"Chi: "<<chi.getAPosition()<<std::endl;
    std::cout<<"Omega: "<<omega.getAPosition()<<std::endl;
    std::cout<<"2Theta: "<<mot.getAPosition()<<std::endl;
    /*sleep(2);
    std::cout<<"Actual position of the motors"<<std::endl;
    std::cout<<"Phi: "<<phi.getAPosition()<<std::endl;
    std::cout<<"Chi: "<<chi.getAPosition()<<std::endl;
    std::cout<<"Omega: "<<omega.getAPosition()<<std::endl;
    std::cout<<"2Theta: "<<mot.getAPosition()<<std::endl;*/
  }
    return 0;
}

void testThermometer (Thermometer *therm)
{
    std::cout<<"Now, testing the methods of the base class"<<std::endl;
    std::cout<<"Number of available temperature channels: "<<therm->getNoChannels()<<std::endl;
    std::cout<<"The number of available datasets for first trial: "<<((int)(therm->Temperature()).size()==0 ? "0 -OK" : "ERROR")<<std::endl;
    std::cout<<"Get the values of each channel: "<<std::endl;
    for(int i=0; i<therm->getNoChannels(); i++)
    {
	therm->enabledisableChannel(i,true);
	if(i!=0) therm->enabledisableChannel(i-1,false);
	std::cout<<"Channel no. "<<i<<": "<<(therm->Temperature())[0]<<std::endl;
    }
    std::cout<<std::endl<<"Now, print out all of the datasets: "<<std::endl;
    for(int i=0; i<therm->getNoChannels(); i++) therm->enabledisableChannel(i,true);
    Vec<double> dataset=therm->Temperature();
    std::cout<<"The number of datasets: "<<(int)dataset.size()<<std::endl;
    for(int i=0; i<(int)dataset.size(); i++) 	std::cout<<"Channel no. "<<i<<": "<<dataset[i]<<std::endl;
    return;
}

bool update_default_file(std::string motor_name, int position, bool adata, double angle, uint32_t low_limit, uint32_t high_limit)
//This function updates the default file with the provided position belonging to motor_name. If adata is present, the angular data should be provided. Also low and high limit are essentials 
{
  std::vector<std::string> lines;
  std::string line;
  std::ifstream is("default");
  if(!is) return false;
  for(int i=0; i<7; i++)
  {
    std::getline(is,line);
    lines.push_back(line);
  }
  std::getline(is,line);
  lines.push_back(line);
  std::stringstream ss(line);
  int no_motors;
  ss>>no_motors;
  for(int i=0; i<no_motors; i++)
  {
    std::getline(is,line);
    ss.clear(); ss.str(line);
    std::string line_name;
    ss>>line_name;
    if(line_name==motor_name)
    {
      //We have to create motor name, etc
      line=motor_name+" "+std::to_string(position)+" "+std::to_string(low_limit)+" "+std::to_string(high_limit);
      if(adata) line+=" "+std::to_string(angle);
    }
    lines.push_back(line);
  }
  std::getline(is,line);
  ss.clear(); ss.str(line);
  int no_mods, no_tubes;
  ss>>no_mods>>no_tubes;
  lines.push_back(line);
  for(int i=0; i<no_tubes; i++)
  {
    std::getline(is,line);
    lines.push_back(line);
  }
  is.close();
  std::ofstream os("default");
  if(!os) return false;
  for(int i=0; i<(int)lines.size(); i++) os<<lines[i]<<std::endl;
  os.close();
  return true;
}
