#include <ctime>
#include <fstream>
#include <string>
#include <sstream>
#include <unistd.h>
#include "kd48p.h"
#include "Modbus.h"
#include "OHM.h"
#include "Lock.h"


int main(int argc, char *argv[])
{
  if(argc!=2)
  {
    std::cerr<<"This program registers the temperature datasets and writes to std out with timestamp"<<std::endl<<" Usage: ./temp_register time[integer s]"<<std::endl;
    return -1;
  }
  
  int collection_time;
  std::stringstream str(argv[1]);
  const int number_of_trials=5;
  str>>collection_time;
  Modbus mb("haga_controller",19200,8,'N',1,1.0);
  KD48P regulator(&mb,1);
  //OHM ohm("thermometer_ohm",1.0);
  std::ofstream of("regulator_configuration.dat");
  of<<"Setup of the regulator:"<<std::endl;
  if(!regulator.readConfig(of)) std::cout<<"ERROR: Reading the regulator has not been succeeded!"<<std::endl;
  regulator.setSlaveNumber(2);
  of<<"Setup of the other:"<<std::endl;
  if(!regulator.readConfig(of)) std::cout<<"ERROR: Reading the other has not been succeeded!"<<std::endl;
  regulator.setSlaveNumber(1);
  of.close();
  std::cout<<"#Collection time: each "<<collection_time<<"s"<<std::endl<<"#time(day_of_week month day hour:min:sec year) ,  regulator(C), other(C),"; 
  std::cout/*<<"A thermometer("<<ohm.GetTemperatureUnit(true)<<"), B thermometer("<<ohm.GetTemperatureUnit(false)<<")"*/<<std::endl;
  while(true)
  {
    {
      double p=0.0;
      double sp=0.0;
      double y=0.0;
      //Get local time
      time_t timer;
      Lock lck("temperature_lock");
      lck.lock(true);
      time(&timer);
      std::string t(asctime(localtime(&timer)));
      std::cout<<t.substr(0,t.size()-1)<<" ";
      for(int i=0; i<2; i++)
      {
        bool finished=false;
        for(int j=0; (j<number_of_trials)&&(!finished); j++)
        {
          if(regulator.getRegVals(p,sp,y))
          {
            std::cout<<p<<" ";
            finished=true;
          }
          else if(j<number_of_trials-1)
          {
            sleep((unsigned int)1);
          }
          else if(j==number_of_trials-1) 
          {
            std::cout<<"-1000 ";
          }
        }
        regulator.setSlaveNumber(2-(i%2));
      }
      //Italian thermometer
      /*bool AchOK,BchOK;
      double Ach,Bch;
      bool fin=false;
      for (int j=0; (j<number_of_trials)&&(!fin); j++)
      {
	  if(ohm.GetTemperature(AchOK,BchOK,Ach,Bch))
	  {
	      std::cout<<(AchOK ? Ach : -900.0)<<" "<<(BchOK ? Bch : -900.0)<<" ";
	      fin=true;
	  }
	  else if(j<number_of_trials-1)
          {
    	      sleep((unsigned int)1);
          }
          else if(j==number_of_trials-1) 
          {
    	      std::cout<<"-1000 ";
          }
      }*/
      lck.lock(false);
    }
    std::cout<<std::endl;
    sleep((unsigned int)collection_time);
  }
}
